unit u_ShCompUtils;

(*
  ------------------------------------------------------------------------------
  Utilities fr Componenten
  ------------------------------------------------------------------------------
*)

(*
  ------------------------------------------------------------------------------
*)
interface
(*
  ------------------------------------------------------------------------------
*)

uses
  StdCtrls;

(*
  Zweck:
    Ermittelt, an welcher Position der Text im Eingabefeld der ComboBox
    in ihrer Auswahlliste steht.
    Dabei wird die Gro-Kleinschreibung ignoriert.
  Eingabe:
    'ComboBox' Ist die zu prfende ComboBox
  Rckgabe:
    Index des Textes in der Auswahlliste der Compobox.
    -1, wenn der Text nicht enthalten ist.

  27.05.2010  Vers. 3.5.0.79  [2590]  r.sch.
*)
function getTextIdx (comboBox: TComboBox): Integer;


(*
  Zweck:
    Bei der Eingabe in eine ComboBox kann der Eintrag zwar ein der Liste
    vorhanden sein, aber die Gro-Kleinschreibung pat nicht.
    Das wird korrigiert.
  Ein/Ausgabe:
    Box: Ist die ComboBox, deren Eingabefeld korrigiert werden soll.
  Rckgabe:
    true : Der Text ist korrigiert worden
    false: Der Text ist nicht in der Auswahlliste enthalten

  27.05.2010  Vers. 3.5.0.79  [2590]  r.sch.
*)
function adjustText (Box: TComboBox): Boolean;


(*
  Zweck:
    Trgt den Text im Eingabefeld der ComboBox in die Auswahlliste der ComboBox
    ein, falls er noch nicht vorhanden ist.
  Ein/Ausgabe:
    'Box' : Ist die ComboBox, deren Auswahlliste korrigiert werden soll.

  27.05.2010  Vers. 3.5.0.79  [2590]  r.sch.
*)
procedure TextIntoList (Box: TComboBox);


(*
  Zweck:
    Lscht den letzten Eintrag aus der Auswahlliste der ComboBox,
    falls er <> '' ist.
  Ein/Ausgabe:
    'Box' : Ist die ComboBox, deren Auswahlliste korrigiert werden soll.

  10.07.2010  Vers. 3.5.0.80  [2264]  r.sch.
*)
procedure DelLastFromList (Box : TComboBox);

(*
  ------------------------------------------------------------------------------
*)
implementation
(*
  ------------------------------------------------------------------------------
*)

uses
  SysUtils;
  
function getTextIdx (comboBox: TComboBox): Integer;

  var
    xText : string;
    xEntry: string;
    xIdx  : Integer;
    xSize : Integer;

  begin
    Result := -1;
    xText  := LowerCase (comboBox.Text);
    xSize  := comboBox.Items.Count;

    for xIdx := 0 to xSize-1 do
    begin
      xEntry := LowerCase (comboBox.Items [xIdx]);

      if xText = xEntry then
      begin
        Result := xIdx;
        Break;
      end;
    end;
  end;


function adjustText (Box: TComboBox): Boolean;

  var
    xIdx: Integer;

  begin
    xIdx   := getTextIdx (Box);
    Result := xIdx >= 0;

    if Result then
      Box.Text := Box.Items [xIdx];
  end;


procedure TextIntoList (Box : TComboBox);

  var
    xIdx: Integer;

  begin
    xIdx := getTextIdx (Box);

    if xIdx < 0 then
      Box.Items.Add (Box.Text);
  end;


procedure DelLastFromList (Box : TComboBox);

  var
    xText,
    xS   : string;
    xIdx : Integer;

  begin
    xText := Box.Text;
    xIdx  := Box.Items.Count -1;
    xS    := Box.Items.Strings [xIdx];

    if xS <> '' then
    begin
      Box.Items.Delete (xIdx);
      Box.Text := xText;         //  durch 'Delete' verschwindet auch der 'Text'
    end;
  end;

end.
