//-----------------------------------------------------------------------------
// Projekt:    Bitfarm-Archiv
// Entwickler: m.frey
// Erstellt:   2009-09-16
// Name:       u_ShLowUtils
// Typ:        Unit
// Zweck:      Utils fr einfacheren DB-Zugriff
//-----------------------------------------------------------------------------
//----------------------------------------------------------------------------}

unit u_ShDbUtils;

{###############################} INTERFACE {###############################}

{$WRITEABLECONST ON}

uses
  Windows, SysUtils, Classes, Dialogs, ShlObj, Activex,
  SvcMgr, Forms, // SvcMgr muss vor Forms kommen 
  MyAccess, gnugettext,
  u_ShLowUtils;

const
   my_Con : TMyConnection = nil;  // muss vom Hauptprogramm gesetzt werden
   LastSQL : string=''; // letzter benutzter SQL-Befehl
   LastInsertId : int64=0;

// Grundfunktionen
function DbCreateQuery:TMyQuery;
function DbCreateCommand:TMyCommand;
function DbDoSql(const MyQuery:TMyQuery; Sql:string):Boolean;
function DbDoSkySQL(const MyQuery:TMyQuery; Sql: string): boolean;
function DbExecSql(const MyQuery:TMyQuery; Sql:string):Boolean;
function DbX(Sql:string):Boolean;
   function DbXParam(Sql,Param,Value:string):Boolean;
function DbXMsg(Sql,ErrMod,ErrMsg:string):Boolean;
function DbXLog(Sql,Msg:string):Boolean;

function AddQuote(Par:string):string;
function AddDblQuote(Par:string):string;

// Etwas hherer Level
function DbCountSql(Sql:string):integer;
function DbTableExist(Name:string):Boolean;
function DbTableColumnExist(Table,Column:string):Boolean;
function DbTableColumnType(Table,Column:string):string;
function DbTableColumnValue(Sql:string):string;
function DbTableColumnGetString(Sql:string):string;
function DbTableColumnGetInteger(Sql:string):integer;
function DbGetFirstColumns(Sql:string):string;

   // noch hher  :-)
   function DBGetUserName(UserID : Integer): string;
   function DBGetUserID(UserName : string): Integer;
   function DBGetDocRights(ArcID, DocID, UserID : Integer): Integer;

{############################} IMPLEMENTATION {#############################}

//-----------------------------------------------------------------------------
function DbCreateQuery:TMyQuery;
begin
   Result := TMyQuery.Create(nil);
   Result.Connection := my_Con;
end;

function DbCreateCommand:TMyCommand;
begin
   Result := TMyCommand.Create(nil);
   Result.Connection := my_Con;
end;

//-----------------------------------------------------------------------------
// m.frey: Es gibt bei bitfarm die drei Muster DbExecSql, DbDoSql und DbDoSkySQL
// Der Unterschied ist mir nicht ganz klar. Solange das so ist, bleiben
// alle Muster drin
//-----------------------------------------------------------------------------
function DbExecSql(const MyQuery:TMyQuery; Sql:string):Boolean;
begin
   Result:=false;
   MyQuery.SQL.Text := Sql;
   MyQuery.Execute;
end;

//-----------------------------------------------------------------------------
function DbDoSql(const MyQuery:TMyQuery; Sql:string):Boolean;
begin
   result := True;
   LastSQL:=Sql;
   try
      MyQuery.Active := False;
      MyQuery.SQL.Text := Sql;
      MyQuery.Active := True;
      MyQuery.First;
   except
      on e: Exception do
      begin
         LogEvnt('1000    Fehler in function DbDoSql mit Query '+MyQuery.Name+ ' ' + e.message);
         result := False;
      end;
   end;
end;

//-----------------------------------------------------------------------------
function DbDoSkySQL(const MyQuery:TMyQuery; Sql: string): boolean;
begin
   GgfLogevnt('DbDoSkySQL:'+MyQuery.Name+' '+Sql);
   LastSQL:=Sql;
   try
      // skywalker new procedure
      // now check if the query is active, if so just execute
      // else close and open the query
      MyQuery.SQL.Text := Sql;
      // not was added cause open should be done with
      // a query which returns only one value accord to the
      // error message
      if not MyQuery.Active then
         begin
            // Just do an execute!
            //Execute;
             MyQuery.Open;
         end
      else
         begin 
            MyQuery.Close;
            MyQuery.Open;
         end;
      MyQuery.First;
      Result := True;
   except
      on e: Exception do
      begin
         LogEvnt('1000    ' + _('Fehler') + ' in Modul: DbDoSkySQL    ' + e.message + #13 +' '+ LastSQL);
         Result := False;
      end;
   end;
end;

//-----------------------------------------------------------------------------
function DbX(Sql:string):Boolean;
var lmc: TMyCommand;
begin
   result := True;
   lmc := DbCreateCommand;
   try
      LastSQL:=Sql;
      lmc.Name := 'DbX';
      lmc.SQL.Text := Sql;
      lmc.Execute;
      LastInsertId := lmc.InsertId;
   finally
      lmc.Free;
   end;
end;

function DbXParam(Sql,Param,Value:string):Boolean;
var lmc: TMyCommand;
begin
   result := True;
   lmc := DbCreateCommand;
   try
      LastSQL:=Sql;
      lmc.Name := 'DbX';
      lmc.SQL.Text := Sql;
      lmc.ParamByName(Param).AsString := Value;
      lmc.Execute;
      LastInsertId := lmc.InsertId;
   finally
      lmc.Free;
   end;
end;

//-----------------------------------------------------------------------------
function DbXMsg(Sql,ErrMod,ErrMsg:string):Boolean;
begin
   Result := True;
   try
      DbX(SQL);
   except
      on e: Exception do
      begin
         ErrorMsg(ErrMod, ErrMsg + ' ' + e.Message, 1003, True);
         Result := False;
      end;
   end;
end;

//-----------------------------------------------------------------------------
function DbXLog(Sql,Msg:string):Boolean;
var lsE:string;
begin
   Result := True;
   try
      DbX(SQL);
   except
      on e: Exception do
      begin
         lsE := StringReplace(Msg, '%e%', e.Message, [rfReplaceAll]);
         Logevnt(lsE);
         Result := False;
      end;
   end;
end;

//-----------------------------------------------------------------------------
function AddQuote(Par:string):string;
begin
   Result := #39 + Par + #39;
end;

function AddDblQuote(Par:string):string;
begin
   Result := #34 + Par + #34;
end;

//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Routinen etwas hherer Level 
//-----------------------------------------------------------------------------

// Zhlt Reihen einer Abfrage - Parameter ist Abfrage ab incl 'from'
function DbCountSql(Sql:string):integer;
var lmq: TMyQuery;
begin
   Result:=0;
   lmq := DbCreateQuery;
   try
      LastSQL:=Sql;
      try
         lmq.SQL.Text := 'select count(*) '+Sql;
         lmq.Execute;
         Result:=lmq.fields[0].asinteger;
      except
         on e: Exception do
         begin
            LogEvnt('1000    Fehler in function DbCountSql mit Abfrage '+Sql+ ' ' + e.message);
         end;
      end;
   finally
      lmq.Free;
   end;
end;

//-----------------------------------------------------------------------------
function DbTableExist(Name:string):Boolean;
var lmq: TMyQuery;
begin
   lmq := DbCreateQuery;
   try
      // Prfen ob die Tabelle da ist.
      LastSQL:='show tables like "' + Name + '"';
      DbExecSql(lmq,LastSQL);
      Result:=lmq.recordcount > 0;
   finally
      lmq.Free;
   end;
end;

//-----------------------------------------------------------------------------
// Prft ob eine Spalte in einer Tabelle da ist
// es wird nicht geprft ob die Tabelle berhaupt da ist. Was passiert wenn nicht ist undefiniert.
function DbTableColumnExist(Table,Column:string):Boolean;
var lmq: TMyQuery;
begin
   lmq := DbCreateQuery;
   try
      LastSQL := 'select * from '+Table+' where 1=2'; // Leere Menge holen
      DbExecSql(lmq,LastSQL);
      Result := lmq.FieldDefList.Find(Column) <> nil;
   finally
      lmq.Free;
   end;
end;

//-----------------------------------------------------------------------------
// Holt den Typ einer Spalte
// es wird nicht geprft ob die Tabelle berhaupt da ist. Was passiert wenn nicht ist undefiniert.
function DbTableColumnType(Table,Column:string):string;
var lmq: TMyQuery;
begin
   Result := '';
   lmq := DbCreateQuery;
   try
      LastSQL := 'SHOW COLUMNS from '+Table+' like '+#39+Column+#39;
      DbExecSql(lmq,LastSQL);
      Result := lmq.FieldByName('type').AsString;
   finally
      lmq.Free;
   end;
end;

//-----------------------------------------------------------------------------
// Holt das erste Feld der ersten Spalte in einer Tabelle
// es wird nicht geprft ob die Tabelle berhaupt da ist. Was passiert wenn nicht ist undefiniert.
// Im gegensatz zu den zwei vorigen wird hier ein kompletter SQL Query angegeben.
// Beispiel "select Options from sys_arc where
// todo: umbenennen
function DbTableColumnValue(Sql:string):string;
var lmq: TMyQuery;
begin
   lmq := DbCreateQuery;
   try
      LastSQL := Sql;
      DbExecSql(lmq,LastSQL);
      Result:=lmq.fields[0].AsString;
   finally
      lmq.Free;
   end;
end;

// das gleiche mit Fehlerbehandlung, dann Leerstring
function DbTableColumnGetString(Sql:string):string;
begin
   try
      result := DbTableColumnValue(Sql);
   except
      on e: Exception do
      begin
         result := '';
      end;
   end;
end;

// das gleiche mit integer, dann -1
function DbTableColumnGetInteger(Sql:string):integer;
var lmq: TMyQuery;
begin
   try
      lmq := DbCreateQuery;
      try
         LastSQL := Sql;
         DbExecSql(lmq,LastSQL);
         Result:=lmq.fields[0].AsInteger;
      finally
         lmq.Free;
      end;
   except
      on e: Exception do
      begin
         result := -1;
      end;
   end;
end;


//-----------------------------------------------------------------------------
// Holt das alle Werte vom ersten Spalte einer Abfrage
// es wird nicht geprft ob die Tabelle berhaupt da ist. Was passiert wenn nicht ist undefiniert.
// Alle Werte werden kommagetrennt zurckgegeben
// Beispiel:
function DbGetFirstColumns(Sql:string):string;
var lmq: TMyQuery; ls:string;
begin
   lmq := DbCreateQuery;
   try
      LastSQL := Sql; 
      DbExecSql(lmq,LastSQL);
      while not lmq.Eof do
      begin
         ls:=lmq.fields[0].AsString;
         if length(Result)>0 then
            Result := Result + ',' + ls
         else
            Result := ls;
         lmq.Next;
      end;
   finally
      lmq.Free;
   end;
end;

function DBGetUserName(UserID : Integer): string;
begin
	Result := DbTableColumnGetString('select usr_name from sys_usr where usr_id = ' + inttostr(UserID));

end;

function DBGetUserID(UserName : string): Integer;
begin
	Result := DbTableColumnGetInteger('select usr_id from sys_usr where usr_name = ' +#39+ UserName +#39);

end;

function DBGetDocRights(ArcID, DocID, UserID : Integer): Integer;
begin
	Result := DbTableColumnGetInteger('Select Sql_No_cache bit_or(rights) as rechte from sys_docxs where arc_id = '
         + IntToStr(ArcID) + ' and usr_id = ' + IntToStr(UserID) +
         ' and doc_id = ' + IntToStr(DocID) + ' Group by doc_id');

end;



//-----------------------------------------------------------------------------
initialization

finalization

end.
