unit u_ShDefs;

(*
  ------------------------------------------------------------------------------
  Diese Unit enthlt Definitionen
  ------------------------------------------------------------------------------
*)

(*
  ------------------------------------------------------------------------------
*)
interface
(*
  ------------------------------------------------------------------------------
*)

uses
  Classes, DB, u_ShRecTab;


type
  (*
    Typen der Zusatzfeld-Komponenten
  *)
  eAddFieldType = (zfld00_Universal,
                   zfld01_Datum,
                   zfld02_Auswahl,
                   zfld03_MehrfachAuswahl,
                   zfld04_NichtDefiniert,
                   zfld05_Intervall,
                   zfld06_GanzZahl,
                   zfld07_FliessKomma,
                   zfld08_Waehrung       );
  sAddFieldType = set of eAddFieldType;


  (*  Zweck:
        Enthlt Informationen zu allen Zusatzfeldern
      Achtung!
        Es mu geprft werden, ob diese Informationen wirklich an jeder Stelle
        aktualisiert werden, wenn die betreffenden Daten sich ndern!
  *)
  clAddField = class (clRecAbstr)
  public
    FieldName   : string;           //  z.B. 'add_2610200910094086103'
    DispName	: string;
    Typ        : eAddFieldType;
    CompIdx    : Integer;          //  Index der Editier-Komponente
    CompName   : string;           //  Name  der Editier-Komponente
    ContOld    : string;           //  Inhalt vor der nderung durch den Anwender
    ChoiceList : TStringList;      //  Gltige Eintrge in Auswahl-Feldern
    isCatchWord: boolean;          //  Ist eine ComboBox der VerschlagwortungsHilfe
    Optionen   : Integer;          //  enthlt den Optionenwert des Feldes

    constructor Create;
    destructor  Destroy;  override;

    (*  Zweck:
          Vegleicht sich 'self' mit dem Parameter auf Gleichheit
        Eingabe:
          'Rec': Ist der zu vergleichende Datensatz vom Typ 'clAddField'
        Rckgabe:
          true: Wenn der Name der Editier-Komponente gleich ist.
    *)
    function isEqual (Rec: TObject): Boolean;  override;
  end;


  clAddFieldTab = class (clRecTab)
  private
    constructor Create;
    destructor  Destroy;  override;

  public
    function  findComp (CompName: string): clAddField;
    function  findAdd (FieldName: string): clAddField;
    procedure add      (const Rec     : clAddField);   overload;

    function List: string;
  end;


  (*  Zweck:
        Enthlt Informationen zu allen Zusatzfeldern
      Achtung!
        Es mu geprft werden, ob diese Informationen wirklich an jeder Stelle
        aktualisiert werden, wenn die betreffenden Daten sich ndern!
  *)
  clStateField = class (clRecAbstr)
  public
  		DispName   : string;
      FieldName  : string;
      CompIdx    : Integer;          //  Index der Editier-Komponente
   	CompName   : string;           //  Name  der Editier-Komponente
      Comp 		  : TObject;			// Referenz auf die Komponente
    	ContOld    : string;
      AddfieldID : Integer;			// ID des verknpften Zusatzfeldes
      AddfieldName: string;			// Name des verknpften Zusatzufeldes
      AddFieldFunc: Integer;			// Typ der Verknpfung
      BitMask:		  Integer;			// Wertigkeit des Statusfeldes bei Berechnung des BitMaske
  												// 1.Status = 2, 2.Status = 4, 3.Status = 8 etc.
  		StateSet:	  Boolean;			// Status gestetzt
      StateNot:     Boolean;			// NotStatus (Rot) zur Suche
      ReadOnly:	  Boolean;
	  function isEqual (Rec: TObject): Boolean;  override;

  end;


  clStateFieldTab = class (clRecTab)
  private
    constructor Create;
    destructor  Destroy;  override;

  public
    function  findComp (CompName: string): clStateField;
    function  findAdd (FieldName: string): clStateField;
    procedure Add     (const Rec     : clStateField);   overload;

    function List: string;
  end;


  (*
      Enthlt Info zu einem Archiv

      Achtung: Singleton-Object!
        Im gesamten Programm gibt es nur ein einziges Exemplar
        der Klasse 'clArchiv'
  *)
  clArchiv = class
  private
    constructor Create;

  public
    AddFieldTab: clAddFieldTab;  //  Tabelle der Zusatzfelder

    (*  Zweck:
          initialisiert 'AddFieldCont'
        Eingabe:
          'Count': Ist die Anzahl einzurichtender Strings
    *)
    destructor Destroy;         override;

    (*  Zweck:
          Gibt das Singleton-Object dieser Klasse zurck!
    *)
    class function getInstance: clArchiv;

    (*
      Zweck:
        Findet anhand den Komponenten-Namens den Typ
      Eingabe:
        'CompName': Name der Komponente
      Rckgabe:
        Den Typ der Komponente.
        Sollte es den Komponent-Namen nicht geben, wird
        'zfld04_NichtDefiniert' zurckgegeben.
    *)
    function getType (CompName: string): eAddFieldType;
  end;


function TypToStr   (Typ  : eAddFieldType): string;
function StateToStr (State: TDataSetState): string;

(*
  ------------------------------------------------------------------------------
*)
implementation
(*
  ------------------------------------------------------------------------------
*)

uses
  SysUtils;

var
  gArchiv: clArchiv;



function TypToStr (Typ: eAddFieldType): string;
  begin
    case Typ of
      zfld00_Universal      : Result := 'zfld00_Universal';
      zfld01_Datum          : Result := 'zfld01_Datum';
      zfld02_Auswahl        : Result := 'zfld02_Auswahl';
      zfld03_MehrfachAuswahl: Result := 'zfld03_MehrfachAuswahl';
      zfld04_NichtDefiniert : Result := 'zfld04_NichtDefiniert';
      zfld05_Intervall      : Result := 'zfld05_Intervall';
      zfld06_GanzZahl       : Result := 'zfld06_GanzZahl';
      zfld07_FliessKomma    : Result := 'zfld07_FliessKomma';
      zfld08_Waehrung       : Result := 'zfld08_Waehrung';
    else
                           Result := 'einfach unmglich!';
    end;
  end;


function StateToStr (State: TDataSetState): string;
  begin
    case State of
      dsInactive    : Result := 'dsInactive';
      dsBrowse      : Result := 'dsBrowse';
      dsEdit        : Result := 'dsEdit';
      dsInsert      : Result := 'dsInsert';
      dsSetKey      : Result := 'dsSetKey';
      dsCalcFields  : Result := 'dsCalcFields';
      dsFilter      : Result := 'dsFilter';
      dsNewValue    : Result := 'dsNewValue';
      dsOldValue    : Result := 'dsOldValue';
      dsCurValue    : Result := 'dsCurValue';
      dsBlockRead   : Result := 'dsBlockRead';
      dsInternalCalc: Result := 'dsInternalCalc';
      dsOpening     : Result := 'dsOpening';
    else
    end;
  end;

(*
  ------------------------------------------------------------------------------
  clAddFieldTab
  ------------------------------------------------------------------------------
*)

constructor clAddFieldTab.Create;
  begin
    inherited;
  end;


destructor clAddFieldTab.Destroy;
  begin
    inherited Destroy;
  end;


function clAddFieldTab.findComp (CompName: string): clAddField;

  var
    lRec: clAddField;

  begin
    lRec          := clAddField.Create;
    lRec.CompName := CompName;

    Result := clAddField (find (lRec));
  end;

function clAddFieldTab.findAdd (FieldName: string): clAddField;

  var
    lRec: clAddField;

  begin
    lRec          := clAddField.Create;
    lRec.FieldName := FieldName;

    Result := clAddField (find (lRec));
  end;

procedure clAddFieldTab.Add (const Rec: clAddField);
  begin
    inherited Add (Rec);
  end;


function clAddFieldTab.List: string;

  var
    lRec: clAddField;
    lIdx: integer;

  begin
    Result := '';

    for lIdx := 0 to Count -1 do
    begin
      lRec   := clAddField (getRec (lIdx));

      Result := Result
              + Format ('"%s", "%s", "%s", "%s", "%s"'+#13+#10,
                        [          lRec.FieldName,
                         TypToStr (lRec.Typ     ),
                         IntToStr (lRec.CompIdx ),
                                   lRec.CompName,
                                   lRec.ContOld  ]);
    end;
  end;

(*
  ------------------------------------------------------------------------------
  clAddField 
  ------------------------------------------------------------------------------
*)

constructor clAddField.Create;
  begin
    inherited;

    ChoiceList  := TStringList.Create;
    isCatchWord := false;
  end;


destructor clAddField.Destroy;
  begin
    ChoiceList.Free;
  end;


function clAddField.isEqual (Rec: TObject): Boolean;

  var
    lRec: clAddField;

  begin
    lRec   := clAddField (Rec);
    if Self.CompName <> '' then
	    Result := Self.CompName = lRec.CompName
    else
       Result := Self.FieldName = lRec.FieldName
  end;



(*
  ------------------------------------------------------------------------------
  clStateFieldTab
  ------------------------------------------------------------------------------
*)

constructor clStateFieldTab.Create;
  begin
    inherited;
  end;


destructor clStateFieldTab.Destroy;
  begin
    inherited Destroy;
  end;


function clStateFieldTab.findComp (CompName: string): clStateField;

  var
    lRec: clStateField;

  begin
    lRec          := clStateField.Create;
    lRec.CompName := CompName;

    Result := clStateField (find (lRec));
  end;

function clStateFieldTab.findAdd (FieldName: string): clStateField;

  var
    lRec: clStateField;

  begin
    lRec          := clStateField.Create;
    lRec.FieldName := FieldName;

    Result := clStateField (find (lRec));
  end;

procedure clStateFieldTab.Add (const Rec: clStateField);
  begin
    inherited Add (Rec);
  end;


function clStateFieldTab.List: string;

  var
    lRec: clStateField;
    lIdx: integer;

  begin
    Result := '';

    for lIdx := 0 to Count -1 do
    begin
      lRec   := clStateField (getRec (lIdx));

      Result := Result
              + Format ('"%s", "%s", "%s", "%s""'+#13+#10,
                        [          lRec.FieldName,
                         IntToStr (lRec.BitMask ),
                         IntToStr (lRec.CompIdx ),
                                   lRec.CompName]);
    end;
  end;



(*
  ------------------------------------------------------------------------------
  clStateField
  ------------------------------------------------------------------------------
*)

  function clStateField.isEqual (Rec: TObject): Boolean;

  var
    lRec: clStateField;

  begin
    lRec   := clStateField(Rec);
    if Self.CompName <> '' then
	    Result := Self.CompName = lRec.CompName
    else
       Result := Self.FieldName = lRec.FieldName
  end;



(*
  ------------------------------------------------------------------------------
  clArchiv
  ------------------------------------------------------------------------------
*)

constructor clArchiv.Create;
  begin
    inherited Create;

    AddFieldTab := clAddFieldTab.Create;
  end;


destructor clArchiv.Destroy;
  begin
    AddFieldTab.Free;

    if gArchiv = Self then
      gArchiv := nil;

    inherited;
  end;


class function clArchiv.getInstance: clArchiv;
  begin
    if gArchiv = nil then
      gArchiv := clArchiv.Create;

    Result := gArchiv;
  end;


function clArchiv.getType (CompName: string): eAddFieldType;

  var
    xAddFld: clAddField;

  begin
    xAddFld := AddFieldTab.findComp (CompName);

    if xAddFld = nil then
      Result := zfld04_NichtDefiniert
    else
      Result := xAddFld.Typ;
  end;

initialization
  gArchiv := nil;

end.


