//-----------------------------------------------------------------------------
// Projekt:    Bitfarm-Archiv
// Entwickler: m.frey
// Erstellt:   2009-09-16
// Name:       u_ShLowUtils
// Typ:        Unit
// Zweck:      Diverse Kleinigkeiten um andere units zu "erleichtern"
//-----------------------------------------------------------------------------
// Regeln fr die uses-Liste:
// - ist klein zu halten und es drfen nur low-level verwendet werden.
// - Datenbankzugriffe sind erlaubt, jedoch drfen keine bestimmten Felder
//   oder Tabellen vorausgesetzt werden.
// - ebenfalls erlaubt sind alle Standard-VCL-Units
// - keine anderen units mit Formularen
//----------------------------------------------------------------------------}

unit u_ShLowUtils;

{###############################} INTERFACE {###############################}

{$WRITEABLECONST ON}


uses
  gnugettext, Windows, SysUtils, Classes, Dialogs, Controls, ShlObj, Activex, Registry,
  SvcMgr, Forms, ShellAPI, // SvcMgr muss vor Forms kommen
  MyAccess, jwaiphlpapi,jwaiptypes,jwawinsock2,
  Nb30, u_getmac_iphlpapi, u_ShInputBox, u_ShBase;

type
   // Hier werden globale Variablen gespeichert.
   // viele davon sind Systemvariablen, die beim Programmstart einmal
   // eingelesen werden. 
   TBfData = class
   public
      EnvTemp,   // %temp%
      EnvTempPd, // %temp% mit Pathdelimiter
      EnvSystemRoot,
      EnvSystemRootPd,
      EnvAppdata,
      EnvAppdataPd,
      EnvUsername,

      AppdataBitfarm,
      AppdataBitfarmPd,
      MyLoginIniName,

      ExeFile,
      ExeDir,
      GtDomain, // Domain fr gnugettext

      LogPath, // Verzeichnis fr das Logs
      LogName, // Dateiname (ohne Pfad) fr das Log. "<%d>" steht fr das Datum
      LogUser, // User fr den geloggt wird 
      LogPre,
      dummy: string;
      LogOn, DevDebug : Boolean;
      constructor Create;
   end;

const BD: TBfData = nil;

//Log
procedure LogEvnt(LogMsg: string);
procedure GgfLogEvnt(msg: string);
procedure LogEinzugEin;
procedure LogEinzugAus;
procedure GgfLogEvntBeg(msg: string);
procedure GgfLogEvntEnd(msg: string);
function BS(f:Boolean):string;

//System
function GetSystemVariable(const Name: string): string;
function GetVersionInfo: string;
function GetVolSerial(DriveLetter: string): string;
function GetMACAdress: string;
function GetAdapters: TLanaEnum;
procedure WindowsEreignisanzeige(Name, Meldung:string);
procedure nop;
function appversion(): String;
function GetFirstAdapterAdress:String;

//Crypt        GgfLogEvnt
function EnCrypt(InputStr, HexKeyStr: string): string;
function DeCrypt(HexInputStr, HexKeyStr: string): string;
function Registrierungscode(const sKS:string):string;

//Util
function StrlistsVergleichen(const sl1,sl2:TStringlist):Boolean;
function TrimLeftChar(const S: string; C:Char): string;
function AddWithComma(const S1,S2: string): string;
procedure AddListWhenNot(List:TStrings; s: string);

//Registry
procedure RegIfNotExistWriteString(Reg:TRegistry; Key, Value:string);
function RegReadBool(Reg: TRegistry; Key: string; DefValue: Boolean):Boolean;
function RegReadInteger(Reg: TRegistry; Key: string; DefValue: Integer):Integer;


//Unsortiert
function RefToCell(ARow, ACol: Integer): string;
function ParseSQLString(InputStr: string): string;
function ParseSemicolonUseBackslash(InputStr: string): string;
function ParseSemicolonWithoutBackslash(InputStr: string): string;
function ParseAddfields(InputStr: string): string;
function ParseSearchfield(InputStr: string): string;
function ParseFilename(S: string): string;
function ParseAlphanumeric(AValue: AnsiString):string;
function StrToHtml(s: string): string;
procedure AddfieldType0KeyPress(var Key: Char);

function FileAccessable(FileName: string): boolean;
function IsDirWriteable(const dir: string):Boolean;
function GetFileSizeStr(FileName: string): string;
function KeyPressed(key: integer): boolean;


function NogSelectDirectory(const Caption: string; const Root: WideString;
   var Directory: string; EditBox: boolean = False; ShowFiles: boolean = False;
   AllowCreateDirs: boolean = True): boolean;

type
   Twordandsemlist = record
      words: TStringList;
      sems: TStringList;
   end;
function StrToWords(const InputStr: string): TStringList;
function StrToWords2(const InputStr: string): TStringList;
function StrToWords3(const InputStr: string): Twordandsemlist;
function StrToWords4(const InputStr: string): TStringList;
function StrToWords5(const InputStr: string): TStringList;
function StrToWords6(const InputStr: string): TStringList;
function StrToWords7(const InputStr: string): TStringList;
function StrToWordswvl(const InputStr: string): TStringList;
function StrToWordswvl2(const InputStr: string): TStringList;
function StrToWords0(const InputStr: string): TStringList;
function StrToWordsTab(const InputStr: string): TStringList;
function StrToWordsCRLF(const InputStr: string): TStringList;
function StrToWordsLF(const InputStr: string): TStringList;
function WordsToStr(ts:TStrings):string;

function GetStringFromResource(Name: string): string;
function CleanFileName(InputStr: string): string;

function CopyFileT(const s1,s2:string):Boolean;
function CopyFileF(const S1,S2:string):Boolean;
function CopyFileLog(const s1,s2:string):Boolean;
function DeleteFileLog(const s1:string):Boolean;
function BfDeleteFile(const AFile: string): boolean;
function IsDirectoryEmpty(const directory : string) : boolean; //07.06.2011 b.takacs 3.5.1.19 [3069]
function EmptyDirectory(const ADir: string): boolean;

function ZiffernRechts(const S:string):string;
function ReadDir(Path, Mask: string; ShowPath: boolean): TStringlist;

procedure ErrorMsg(ErrMod, ErrMsg: String; ErrID: Integer; LogErr: boolean);

function S(const S: string):string;
procedure BfShowMessage(const Msg: string);
function BfMessageDlg(Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; Default_NO: boolean = False): Integer;
function BfMessageBox(Msg, Caption: string; Flags: Word; Handle: THandle = 0): Integer;
function BfInputBox(Msg, Caption: string; default: string = ''): string;

procedure GnugettextTextdomain(Domain, Path, Language: string);
function LangKuerzel:string;
procedure TranslateStrings(sl: TStrings;const TextDomain:string);
function RunWaitAndCaptureOutput(CommandLine:string; var Output:string; LogLevel:integer):DWord;
function StrToInt0(const S: string): Integer;
function ConvertStringToFileName(S:string):string;
function ElimRight(Name, Text:string):string;
function ConcatDelim(S1, Delim, S2:string):string;
function CalcNonExistingFilename(Path, Name, Ext: string): string; //3.5.0.38 [2364]
function EliminierenVonAnfuehrungszeichen(S:string):string; //3.5.0.67 [2445]/[2068]

procedure StringInDatei(const Dateiname:Ansistring; const Data:Ansistring);
procedure SpeicherInDatei(const Dateiname:Ansistring; const Buffer; Count: Longint);
function IstZahl(S:string):Boolean; //[2487]
function IsUserAdmin : boolean;
procedure FindAllFiles(var FileList: TStrings; RootFolder: string;
   Mask: string = '*.*'; Recurse: boolean = True);

procedure KillFile(FileName: string);
procedure ClearRights(var RightData : TRData);
(*
  Zweck:
    Konvertiert einen String vom Standard-Format
*)
//function toDbDateTime (DateTime: string): string;

(*
  Zweck:
    Ersetzt die Funktion 'indexOf' aus 'TStrings', denn
    diese originale Funktion findet nicht den leeren Eintrag ''.
*)
function indexOf (text: string;
                  list: TStrings): Integer;



{############################} IMPLEMENTATION {#############################}

uses u_ShDbUtils;

var csLog: RTL_CRITICAL_SECTION; //09-09-07 m.frey


function indexOf (text: string;
                  list: TStrings): Integer;
var
    xIdx   : Integer;
    xIdxMax: Integer;

begin
   Result  := -1;
   xIdxMax := list.Count;

   for xIdx := 0 to xIdxMax do
   begin
      if text = list [xIdx] then
      begin
         Result := xIdx;
         Break;
      end;
   end;
end;


//-----------------------------------------------------------------------------
constructor TBfData.Create;
begin
   EnvTemp        := GetSystemVariable('Temp');
   EnvTempPd      := IncludeTrailingPathDelimiter(EnvTemp);
   EnvSystemRoot  := GetSystemVariable('systemroot');
   EnvSystemRootPd:= IncludeTrailingPathDelimiter(EnvSystemRoot);
   EnvAppdata     := GetSystemVariable('appdata');
   EnvAppdataPd   := IncludeTrailingPathDelimiter(EnvAppdata);
   EnvUsername    := GetSystemVariable('username');

   AppdataBitfarm := EnvAppdata + '\bitfarm-archiv';
   AppdataBitfarmPd := IncludeTrailingPathDelimiter(AppdataBitfarm);

   MyLoginIniName := AppdataBitfarm + '\mylogin.ini';

   ExeFile := ParamStr(0);
   ExeDir := ExtractFilePath(ExeFile); // mit Delimiter hinten

   LogPath := EnvTemp; // vorbelegen
   LogName := ChangeFileExt(ExtractFileName(ExeFile), '.log');
end;


//-----------------------------------------------------------------------------
procedure LogEvnt(LogMsg: string);
//////////////////////////////////////////////////////////////////
///  10.04.2005 Erstellt LogEvnt by SV.                        ///
///                                                            ///
///  Diese Procedure schreibt LogMsg mit Datum in eine Datei   ///
///                                                            ///
//////////////////////////////////////////////////////////////////
var
   fLogFile: TextFile;
   sLogDate, Slognfile: string;
begin
   //if not BD.LogOn
   //then Exit;
   
   EnterCriticalSection(csLog); //09-09-07 m.frey
   try
      DateTimeToString(sLogDate, 'dd.mm.yyyy hh:nn:ss-zzz', now);
      DateTimeToString(Slognfile, 'yyyy-mm-dd', now);

      if BD.LogUser>''
      then Slognfile:=Slognfile+'_'+BD.LogUser;

      Slognfile:=StringReplace(BD.LogName, '<%d>', '_'+Slognfile, [rfReplaceAll, rfIgnoreCase]);
      Slognfile := IncludeTrailingPathDelimiter(BD.LogPath) + Slognfile;

      try
         AssignFile(fLogFile, Slognfile);
         if not FileExists(Slognfile) then
            ReWrite(fLogFile);
         Reset(fLogFile);
         Append(fLogFile);
         WriteLn(fLogFile, sLogDate + '    ' + BD.LogPre+LogMsg);
         Flush(fLogFile);
         CloseFile(fLogFile);
      except
         // on e: exception do ShowMessage('Schwerer Fehler:' + e.Message + #13 + #13 + 'Bitte Prfen Sie die Berechtigung auf die Datei ViewerV3.Log im Ordner ' + GetSystemVariable('systemroot'));
         on e: exception do
            WindowsEreignisanzeige(BD.ExeFile, 'Schwerer Fehler:' + e.Message + #13
                     + #13 + 'Bitte Prfen Sie die Berechtigung auf die Datei '+Slognfile);
      end;
      //OutputDebugString('LogEvnt }');
   finally
      LeaveCriticalSection(csLog); //09-09-07 m.frey
   end;
end;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Die Struktur "if mnu_Speicherinfo.Visible then logevnt('...');"
// gibt/gab es >400 mal in der unit
// damit kann das nun krzer gemacht werden.
procedure GgfLogEvnt(msg: string);
begin
   if BD.LogOn then
      logevnt(msg);
   if BD.DevDebug then
	   OutputDebugString(PChar(msg));
end;

procedure LogEinzugEin;
begin
   BD.LogPre:=BD.LogPre+'  ';
end;

procedure LogEinzugAus;
begin
   Delete(BD.LogPre,length(BD.LogPre)-1,2);
end;

procedure GgfLogEvntBeg(msg: string);
begin
   GgfLogEvnt(msg);
   LogEinzugEin;
end;

procedure GgfLogEvntEnd(msg: string);
begin
   LogEinzugAus;
   GgfLogEvnt(msg);
end;

function BS(f:Boolean):string;
begin
   if f
   then Result:='true'
   else Result:='false';
end;

//-----------------------------------------------------------------------------
function GetSystemVariable(const Name: string): string;
   //////////////////////////////////////////////////////////////////
   ///  02.05.2005 Erstellt GetSystemVariable by SV.              ///
   ///                                                            ///
   ///  Diese Funktion liest die Systemvariable in 'Name' aus     ///
   ///  und liefert ihren Wert als Delphi String                  ///
   ///                                                            ///
   ///  Format: String                                            ///
   ///                                                            ///
   ///  09-09-07 m.frey: nach unit u_BfUtils                      ///
   ///  09-09-16 m.frey: nach unit u_ShLowUtils                   ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
begin
   try
      Result:=Sysutils.GetEnvironmentVariable(Name);
   except
      on e: Exception do
         LogEvnt('9003    ' + _('Fehler') + ' in Modul: GetSystemVariable    ' + e.Message);
   end;
end;
//
//  Name:
//  Parameters:
//  Description:
//  Return value:
// []  b.takacs
//------------------------------------------------------------------------------
function appversion(): String;
var
  verblock:PVSFIXEDFILEINFO;
  versionMS,versionLS:cardinal;
  verlen:cardinal;
  rs:TResourceStream;
  m:TMemoryStream;
  p:pointer;
  s:cardinal;
  AppVersionString :string;
begin
  m:=TMemoryStream.Create;
  try
    rs:=TResourceStream.CreateFromID(HInstance,1,RT_VERSION);
    try
      m.CopyFrom(rs,rs.Size);
    finally
      rs.Free;
    end;
    m.Position:=0;
    if VerQueryValue(m.Memory,'\',pointer(verblock),verlen) then
      begin
        VersionMS:=verblock.dwFileVersionMS;
        VersionLS:=verblock.dwFileVersionLS;
        AppVersionString:= //Application.Title+' '+
          IntToStr(versionMS shr 16)+'.'+
          IntToStr(versionMS and $FFFF)+'.'+
          IntToStr(VersionLS shr 16)+'.'+
          IntToStr(VersionLS and $FFFF);
      end;
    (*
    if VerQueryValue(m.Memory,PChar('\\StringFileInfo\\'+
      IntToHex(GetThreadLocale,4)+IntToHex(GetACP,4)+'\\FileDescription'),p,s) or
        VerQueryValue(m.Memory,'\\StringFileInfo\\040904E4\\FileDescription',p,s) then //en-us
          AppVersionString:=PChar(p)+' '+AppVersionString;
      *)
  finally
    m.Free;
  end;
  Result:=  AppVersionString;
end;
//-----------------------------------------------------------------------------
function GetVersionInfo: string;
var
   VerInfoSize: DWord;
   VerInfo: Pointer;
   VerValueSize: DWord;
   VerValue: PVSFixedFileInfo;
   Dummy: DWord;
begin
   VerInfoSize := GetFileVersionInfoSize(PChar(ParamStr(0)), Dummy);
   GetMem(VerInfo, VerInfoSize);
   GetFileVersionInfo(PChar(ParamStr(0)), 0, VerInfoSize, VerInfo);
   VerQueryValue(VerInfo, '\', Pointer(VerValue), VerValueSize);
   with VerValue^ do
   begin
      result := IntTostr(dwFileVersionMS shr 16);
      result := result + '.' + IntTostr(dwFileVersionMS and $FFFF);
      result := result + '.' + IntTostr(dwFileVersionLS shr 16);
      result := result + '.' + IntTostr(dwFileVersionLS and $FFFF);
   end;
   FreeMem(VerInfo, VerInfoSize);
end;

//-----------------------------------------------------------------------------
function GetVolSerial(DriveLetter: string): string;
   //////////////////////////////////////////////////////////////////
   ///  04.05.2005 Erstellt by SV.                                ///
   ///                                                            ///
   ///  Liest die Seriennummer des Volumes DriveLetter aus        ///
   ///                                                            ///
   ///  Ausgabe: Volume Seriennummer als String                   ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
var
   wSerial: DWord;
   cMaxCompLen, cFSFlags: cardinal;
   aVolName, aFSName: array[0..255] of char;
   sZwResult: string;
   iCount: integer;
begin
   if GetVolumeInformation(PChar(DriveLetter), aVolName, 256, @wSerial,
      cMaxCompLen, cFSFlags, aFSName, 256) then
      sZwResult := IntToHex(wSerial, 4)
   else
      sZwresult := 'aa55aa55';
   iCount := Length(sZwResult);
   if iCount < 8 then
      Result := sZwResult + StringOfChar('A', 8 - iCount)
   else
   if iCount > 8 then
      Result := Copy(sZwResult, 1, 8)
   else
   if iCount = 8 then
      Result := sZwResult;
end;

//-----------------------------------------------------------------------------
function GetMACAdress: string;
   //////////////////////////////////////////////////////////////////
   ///  01.08.2005 Erstellt by SV. Orginal by Tony G              ///
   ///             www.tgsoft.ch                                  ///
   ///                                                            ///
   ///  Liest die MAC Adresse des ersten Netzwerkadapters aus     ///
   ///                                                            ///
   ///  Ausgabe: MAC Adresse als String                           ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
   // todo buggy should check for networkcard which doesnt change!
   // e.g. exclude usb- devices or check if there is only one device.

type
   TAdapterStatusA = record
      Adapt: TAdapterStatus;
      NameBuff: array[0..30] of TNameBuffer;
   end;
var
   NCB: TNCB;     //Network Control Block
   ReturnCode: char; //Retun Code
   AdapterStatus: TAdapterStatus;
   AdapterID: integer;
   L_Enum: TLanaEnum;
   sZwResult: string; // Zwischenergebnis
   iCount: integer;
   getmaciphlpapi: TGetmac;

begin

   Result := '';
   Exit;
//GgfLogEvnt('1') ;
   //Clear NCB Structure
   FillChar(NCB, SizeOf(NCB), 0);
//GgfLogEvnt('2');
   l_Enum := GetAdapters;
//GgfLogEvnt('3');
   AdapterID := byte(l_ENUM.lana[0]);
//GgfLogEvnt('4');

   //Programm NBC to fulfill a reset. It doublecheck if the Lana is present.
   //Can be removed, when used with GetAdapters.
   //NCB.ncb_command := char(NCBRESET);
   NCB.ncb_command := AnsiChar(NCBRESET);
//GgfLogEvnt('5');
   //NCB.ncb_lana_num := char(AdapterID);

   NCB.ncb_lana_num := AnsiChar(AdapterID);
//GgfLogEvnt('6');

   //Process NCB to NetBios
   ReturnCode := NetBios(@NCB);
//GgfLogEvnt('7');

   //Check if query was successfull, if returnCode = #0
   if ReturnCode <> #0 then
//   if not ((ReturnCode = #0) or (ReturnCode = '#')) then
   begin
      // Return the negative returncode
      // Error check would be "if ReturnCode[0]='-' Then..."

      //swithing for advanced solution

      Result := GetFirstAdapterAdress();

      if Result = '' then
      begin
         getmaciphlpapi := Tgetmac.Create;
         Result := getmaciphlpapi.getfirstmac;
         FreeAndNil(getmaciphlpapi);
      end;

      Exit;
   end;

   //Clear NCB Structure
   FillChar(NCB, SizeOf(NCB), 0);
//GgfLogEvnt('8');

   //Programm NBC to ask for the MAC associated with the LanaID
   NCB.ncb_command := char(NCBASTAT);
//GgfLogEvnt('9');
   NCB.ncb_lana_num := char(AdapterID);
//GgfLogEvnt('10');
   NCB.ncb_buffer := @AdapterStatus;
//GgfLogEvnt('11');
   NCB.ncb_length := SizeOf(AdapterStatus);
//GgfLogEvnt('12');
   StrCopy(NCB.ncb_callname, '* ');
//GgfLogEvnt('13');

   //Process NCB to NetBios
   ReturnCode := Netbios(@NCB);
//GgfLogEvnt('14');

   //Check if query was successfull, if returnCode = #0
   //if ReturnCode <> #0 then
   if not ((ReturnCode = #0) or (ReturnCode = '#')) then
   begin
//GgfLogEvnt('141');
      // Return the negative returncode
      Result := char(Ord(ReturnCode) * -1);
//GgfLogEvnt('142');
      Exit;
   end;

   //Return formated MAC
   sZwResult := Format('%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x',
      [Ord(AdapterStatus.Adapter_address[0]), Ord(AdapterStatus.Adapter_address[1]),
      Ord(AdapterStatus.Adapter_address[2]), Ord(AdapterStatus.Adapter_address[3]),
      Ord(AdapterStatus.Adapter_address[4]), Ord(AdapterStatus.Adapter_address[5])]);
//GgfLogEvnt('15');
   iCount := Length(sZwResult);
   if iCount < 12 then
      Result := sZwResult + StringOfChar('A', 12 - iCount)
   else
   if iCount > 12 then
      Result := Copy(sZwResult, 1, 12)
   else
   if iCount = 12 then
      Result := sZwResult;
//GgfLogEvnt('16');

end;

//-----------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////
///  Function GetAdapter                                              ///
///  =================                                                ///
///                                                                   ///
///  This function will return the total nummber of all lana's        ///
///  istalled and a unique the id of each lana.                       ///
///  E.g. on a system containing lana's 0, 2 and 3, a structure with  ///
///  length =3, lana[0]=0, lana[1]=2 and lana[2]=3 will be returned.
///
/// Diese Funktion gibt die gesamte Anzahl aller installierten Local Area network Adapter an
///
///                                                                   ///
/////////////////////////////////////////////////////////////////////////

function GetAdapters: TLanaEnum;
var
   NCB: TNCB;      //Network Control Block
   LanaEnum: TLanaEnum; //Enumeration of Lan Adapters
   ReturnCode: char;    //Retun Code

begin
   //Clear NCB Structure
   FillChar(NCB, SizeOf(NCB), 0);
   //Clear LanaEnum Stucture
   FillChar(LanaEnum, SizeOf(TLanaEnum), 0);

   //Programm NBC to ask for the number of Lana`s
   NCB.ncb_Command := char(NCBENUM);
   NCB.ncb_buffer := @LanaEnum;
   NCB.ncb_length := Sizeof(LanaEnum);

   //Ask NetBios
   ReturnCode := NetBios(@NCB);

   //Check if query was successfull, if returnCode = #0
   if ReturnCode <> #0 then
   begin
      LanaEnum.Length := char(0);
      LanaEnum.Lana[0] := ReturnCode;
   end;

   // Return valid LanaEnum Structure
   Result := LanaEnum;
end;


//-----------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////
// Neue GetAdapterFunktion zur Ermittlung der MAC-Adresse des
// ersten Netzweradapters
// f.schneider  13.10.2011
/////////////////////////////////////////////////////////////////////////

function GetFirstAdapterAdress:String;
  var
    dwFamily,
    dwFlags,
    dwBufLen,
    dwRetVal: DWORD;
    pAddresses: PIP_ADAPTER_ADDRESSES;
    i: Integer;
    tmp: String;

begin
  dwFamily := AF_UNSPEC;
  dwFlags := GAA_FLAG_INCLUDE_PREFIX;
  dwBufLen := SizeOf(IP_ADAPTER_ADDRESSES);
  GetMem(pAddresses,dwBufLen);
  if Assigned(pAddresses) then
    try
      if (GetAdaptersAddresses(dwFamily, dwFlags, NIL, pAddresses, @dwBufLen) = ERROR_BUFFER_OVERFLOW) then begin
        FreeMem(pAddresses);
        GetMem(pAddresses,dwBufLen);
        if Assigned(pAddresses) then begin
          dwRetVal := GetAdaptersAddresses(dwFamily, dwFlags, NIL, pAddresses, @dwBufLen);
          if (dwRetVal = NO_ERROR) then begin
// -----------------------------------------------------------------------------
            if  Assigned(pAddresses) then
            begin
              if (pAddresses^.PhysicalAddressLength <> 0) then begin
                tmp := '';
                for i := 0 to pAddresses^.PhysicalAddressLength -1 do begin
                  tmp := tmp +IntToHex(pAddresses^.PhysicalAddress[i],2);
                  if (i < Integer(pAddresses^.PhysicalAddressLength -1)) then
                    tmp := tmp +'-';
                end;
                Result :=tmp ;
              end;
            end;
// -----------------------------------------------------------------------------
          end
          else begin
            if (dwRetVal = ERROR_NO_DATA) then
            begin
              GgfLogEvnt('No adapter information available!');
              Result := '';
            end
            else
            begin
              GgfLogEvnt('SystemError= ' + string(dwRetVal));
              Result := '';
            end;
          end;
        end
        else
        begin
          GgfLogEvnt('Out of memory! (2)');
          Result := '';
        end;
      end;
    finally
      FreeMem(pAddresses);
    end
  else
  begin
    GgfLogEvnt('Out of memory! (1)');
    Result := '';
  end;

end;




//-----------------------------------------------------------------------------
// Exception in der Ereignisanzeigen von Windows 2000 vermerken    }
procedure WindowsEreignisanzeige(Name, Meldung:string);
begin
   with TEventLogger.Create(Name) do
   try
     LogMessage(Meldung, 1, 0, 0);
   finally
     Free;
   end;
end;

//-----------------------------------------------------------------------------
// m.frey: dummyfunction die man nehmen kann um einen Breakpoint setzen zu knnen
procedure nop;
begin end;



//-----------------------------------------------------------------------------
function EnCrypt(InputStr, HexKeyStr: string): string;
   //////////////////////////////////////////////////////////////////
   ///  10.04.2005 Erstellt EnCrypt by SV.                        ///
   ///                                                            ///
   ///  Diese Funktion erzeugt aus dem InputStr einen mit         ///
   ///  HexKeyStr verschlsselten HexString (einfaches XOR)       ///
   ///                                                            ///
   ///  Format: String aus Hexadezimalzahlen                      ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
var
   iCntIStr, iCntKStr: integer;
begin
   iCntIStr := 1;
   iCntKStr := 1;
   Result := '';
   while iCntIStr < (Length(InputStr) + 1) do
   begin
      try
         Result := Result + IntToHex(
            (Ord(InputStr[iCntIStr]) xor StrToInt('$' +
            copy(HexKeyStr, iCntKStr mod (Length(HexKeyStr) + 1), 2))), 2);
      except
         on e: Exception do
         begin
            LogEvnt('9001    ' + _('Fehler') + ' in Modul: EnCrypt    ' + e.Message);
            Result := '';
            exit;
         end;
      end;
      Inc(iCntIStr, 1);
      Inc(iCntKStr, 2);
   end;
end;

//-----------------------------------------------------------------------------
function DeCrypt(HexInputStr, HexKeyStr: string): string;
   //////////////////////////////////////////////////////////////////
   ///  10.04.2005 Erstellt DeCrypt by SV.                        ///
   ///                                                            ///
   ///  Diese Funktion erzeugt aus dem HexInputStr einen mit      ///
   ///  HexKeyStr entschlsselten String (einfaches XOR)          ///
   ///                                                            ///
   ///  Format: Entschlsselter String                            ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
var
   iCntIStr, iCntKStr: integer;
begin
   iCntIStr := 1;
   iCntKStr := 1;
   Result := '';
   while iCntIStr < (Length(HexInputStr) + 1) do
   begin
      try
         Result := Result + chr((StrToInt('$' + copy(HexInputStr, iCntIStr, 2)) xor
            StrToInt('$' + copy(HexKeyStr, iCntKStr mod (Length(HexKeyStr) + 1),
            2))));
      except
         on e: Exception do
         begin
            LogEvnt('9002    ' + _('Fehler') + ' in Modul: DeCrypt    ' + e.Message);
            Result := '';
            exit;
         end;
      end;
      Inc(iCntIStr, 2);
      Inc(iCntKStr, 2);
   end;
end;

//-----------------------------------------------------------------------------
function Registrierungscode(const sKS:string):string;
var sMAC, sVID: string;
begin
   (*sMAC := StringReplace(GetMacAdress, '-', '', [rfReplaceAll]);
   if length(sMac) < 12 then
      sMAC := '0000CB62EE98';*)
   sVID := GetVolSerial('c:\');
   if length(sVID) < 8 then
      sVID := '0c5E1602';
   
// f.schneider v3.5.1.50  wir verabschieden uns von der MAC Adresse
//   result := Encrypt(sVID + sMAC, sKS);

   result := Encrypt(sVID, sKS);
end;
(**)

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
procedure StringListLeereLoeschen(const sl:TStringlist);
var i:integer;
begin
   for i := sl.Count-1 downto 0 do
   if sl[i]='' then
     sl.Delete(i);
end;

//-----------------------------------------------------------------------------
// true wenn nicht gleich
function StrlistsVergleichen(const sl1,sl2:TStringlist):Boolean;
var i1,i2:integer;
begin
   sl1.Sorted:=true; sl2.Sorted:=true;
   i1:=0; i2:=0;
   sl1.Sorted:=false; sl2.Sorted:=false;
   result:=true; // davon ausgehen dass alle gleich sind.
   while (i1 < sl1.Count) and (i2 < sl2.Count) do
   begin
      if sl1[i1]=sl2[i2] then // strings sind gleich, dann lschen und weiter
         begin
            sl1[i1]:='';
            sl2[i2]:='';
            inc(i1);
            inc(i2);
            continue;
         end;
      result:=false; 
      if sl1[i1]<sl2[i2] then // in liste 1 ist lexikalisch kleiner
         begin
            inc(i1);
            continue;
         end
      else
         begin
            inc(i2);
            continue;
         end;
   end;
   StringListLeereLoeschen(sl1);
   StringListLeereLoeschen(sl2);
end;

//-----------------------------------------------------------------------------
function TrimLeftChar(const S: string; C:Char): string;
var
  I, L: Integer;
begin
  L := Length(S);
  I := 1;
  while (I <= L) and (S[I] = C) do Inc(I);
  Result := Copy(S, I, Maxint);
end;

//-----------------------------------------------------------------------------
function AddWithComma(const S1,S2: string): string;
begin
   Result := S1;
   if (S1 <> '') and (S2<>'') then
      Result := Result+',';
   Result := Result + S2;
end;

//-----------------------------------------------------------------------------
procedure AddListWhenNot(List:TStrings; s: string);
var Index: integer;
begin
   Index := List.Indexof(s);
   if Index >= 0 then Exit;
   List.Add(s);
end;

//-----------------------------------------------------------------------------
// TRegistry
//-----------------------------------------------------------------------------

procedure RegIfNotExistWriteString(Reg:TRegistry; Key, Value:string);
begin
   if not Reg.ValueExists(Key) then
      Reg.WriteString(Key, Value);
end;

//-----------------------------------------------------------------------------
function RegReadBool(Reg: TRegistry; Key: string; DefValue: Boolean):Boolean;
begin
   if Reg.ValueExists(Key) then
      Result := Reg.ReadBool(Key)
   else
      Result := DefValue;
end;

function RegReadInteger(Reg: TRegistry; Key: string; DefValue: Integer):Integer;
begin
   if Reg.ValueExists(Key) then
      Result := Reg.ReadInteger(Key)
   else
      Result := DefValue;
end;

//-----------------------------------------------------------------------------
function RefToCell(ARow, ACol: Integer): string;
var
  ACount, APos: Integer;
begin
// 	b.takacs 31.08.2010 [2686]

//   Result := Chr(Ord('A') + ACol - 1) + IntToStr(ARow);
//  Wenn ACol=27 (AA), kann nicht schreiben, fehlermeldung. Nur bis ACol=26 (Z) kann schreiben

  ACount := ACol div 26;
  APos := ACol mod 26;
  if APos = 0 then
  begin
    ACount := ACount - 1;
    APos := 26;
  end;

  if ACount = 0 then
    Result := Chr(Ord('A') + ACol - 1) + IntToStr(ARow);

  if ACount = 1 then
    Result := 'A' + Chr(Ord('A') + APos - 1) + IntToStr(ARow);

  if ACount > 1 then
    Result := Chr(Ord('A') + ACount - 1) + Chr(Ord('A') + APos - 1) + IntToStr(ARow);
end;

// Semikolon raus; \ und ' werden escaped ( \\ und \' )
function ParseSQLString(InputStr: string): string;
begin
   Result := StringReplace(InputStr, ';', '', [rfReplaceAll]);
   Result := StringReplace(Result, '\', '\\', [rfReplaceAll]);
   Result := StringReplace(Result, #39 , '\' + #39, [rfReplaceAll]);
end;

// ; wird entfernt, ' wird escaped zu \'
function ParseSemicolonUseBackslash(InputStr: string): string;
begin
   Result := StringReplace(InputStr, ';', '', [rfReplaceAll]);
   //change ' to \' for mysql
   Result := StringReplace(Result, #39 , '\' + #39, [rfReplaceAll]);
end;

// ; ' und \' werden entfernt
function ParseSemicolonWithoutBackslash(InputStr: string): string;
begin
   Result := StringReplace(InputStr, ';', '', [rfReplaceAll]);
   //be sure there is no \' already
   Result := StringReplace(Result, '\' + #39 + '', '' + #39 + '', [rfReplaceAll]);
   Result := StringReplace(Result, #39 , '', [rfReplaceAll]);
end;

// Sonderzeichen raus ' " \ ! ( ) * ? ` { | } , aber Semikolon darf bleiben
function ParseAddfields(InputStr: string): string;
begin
   Result := StringReplace(InputStr, #39, '', [rfReplaceAll]);
   Result := StringReplace(Result, #34, '', [rfReplaceAll]);
   Result := StringReplace(Result, '\', '', [rfReplaceAll]);
	Result := StringReplace(Result, '!', '', [rfReplaceAll]);
//   Result := StringReplace(Result, '(', '', [rfReplaceAll]);
//   Result := StringReplace(Result, ')', '', [rfReplaceAll]);
   Result := StringReplace(Result, '*', '', [rfReplaceAll]);
   Result := StringReplace(Result, '?', '', [rfReplaceAll]);
   Result := StringReplace(Result, '`', '', [rfReplaceAll]);
   Result := StringReplace(Result, '{', '', [rfReplaceAll]);
   Result := StringReplace(Result, '|', '', [rfReplaceAll]);
   Result := StringReplace(Result, '}', '', [rfReplaceAll]);
   Result := StringReplace(Result, #160, '', [rfReplaceAll]);
   while Pos('--', Result) > 0 do
	   Result := StringReplace(Result, '--', '-', [rfReplaceAll]); //MySQL Kommentarzeichen

end;

// Sonderzeichen raus ' \ ! ( ) * ? ` { | } , aber Semikolon und Anfhrungszeichen darf bleiben
// wegen der Suche nach Leer und nicht Leer
function ParseSearchfield(InputStr: string): string;
begin
   Result := StringReplace(InputStr, #39, '', [rfReplaceAll]);
   Result := StringReplace(Result, '\', '', [rfReplaceAll]);
	Result := StringReplace(Result, '!', '', [rfReplaceAll]);
//   Result := StringReplace(Result, '(', '', [rfReplaceAll]);
//   Result := StringReplace(Result, ')', '', [rfReplaceAll]);
   Result := StringReplace(Result, '*', '', [rfReplaceAll]);
   Result := StringReplace(Result, '?', '', [rfReplaceAll]);
   Result := StringReplace(Result, '`', '', [rfReplaceAll]);
   Result := StringReplace(Result, '{', '', [rfReplaceAll]);
   Result := StringReplace(Result, '|', '', [rfReplaceAll]);
   Result := StringReplace(Result, '}', '', [rfReplaceAll]);
   Result := StringReplace(Result, #160, '', [rfReplaceAll]);
   while Pos('--', Result) > 0 do
	   Result := StringReplace(Result, '--', '-', [rfReplaceAll]); //MySQL Kommentarzeichen

end;


procedure AddfieldType0KeyPress(var Key: Char);
begin
   if Key in [
      '!', 		// Trenner
      //'"', 	// Anfhrungszeichen, reserviert als Quote und auch SQL //3.5.0.46 [2414]
      //  '%', // wegen SQL mit like //3.5.0.71 wieder dazu
      #39, 		// Apostroph, wegen SQL
//      '(', 		// wegen SQL
//      ')', 		// wegen SQL
      '\', 		// wegen SQL
      '*', 		// vielleicht mal Platzhalter
      // ',', // Trenner //3.5.0.31 wegen Thomas, Herr Pi(e)tsch?
      '?', 		// vielleicht mal Platzhalter
      '`', 		// wegen SQL
      '{', 		// wegen - rein vorsichtshalber
      '|', 		// wegen - rein vorsichtshalber
      '}', 		// wegen - rein vorsichtshalber
      #160 		// geschtztes Leerzeichen
      ]
   then
   	Key := #0;
   if Key = #13 then    // die Windows-Bell unterdrcken
   	Key := #0;
end;


function ParseFilename(S: string): string;   // Windows unzulssige Zeichen
var
   i : Integer;
begin
   Result := '';
   for i := 1 to length(S) do
//      if not (S[i] in ['\','/',':',';','*','?','"','<','>','|','+','=']) then
      if not (S[i] in ['\','/',':','*','?','"','<','>','|']) then
         Result := Result + S[i];
end;

function ParseAlphanumeric(AValue: AnsiString):string;
var
  SrcPtr, DestPtr: PChar;
  i : Integer;
begin
  for i := 1 to Length(AValue) do
  begin
    if AValue[i] in ['a'..'z', 'A'..'Z', '0'..'9', '_'] then begin
      Result:= Result + AValue[i];
    end;
  end;

end;

function StrToHtml(s: string): string;
const
  EntityChars = '"&<>''';
  EntityStrings: array [1..5] of string
    = ('&quot;', '&amp;', '<', '>', '&apos;');
var
  i, j: integer;
  c : Char;
begin
  for i := Length(s) downto 1 do begin
    j := Pos(s[i], EntityChars);
    if j > 0 then begin
      Delete(s, i, 1);
      Insert(EntityStrings[j], s, i);
    end else
    if not (s[i] in [#32..#127]) then begin
    	c := s[i];
      Delete(s, i, 1);
      Insert('&#' + IntToStr(Ord(c)) + ';', s, i);
    end;
  end;
  Result := s;
end;




//-----------------------------------------------------------------------------
function FileAccessable(FileName: string): boolean;
   //////////////////////////////////////////////////////////////////
   ///  15.05.2005 Erstellt by SV.                                ///
   ///                                                            ///
   ///  Prft, ob auf eine Datei lesend zugegriffen werden kann    ///
   ///  oder nicht.                                               ///
   ///                                                            ///
   ///  Parameter: FileName    String mit Pfad und Dateiname      ///
   ///                                                            ///
   ///  Ausgabe:   True       bei Erfolg                          ///
   ///             False      bei Fehlschlag                      ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
var
   iFile: integer;
begin
   iFile := -1;
   result:= False; //22.09.2010 b.takacs
   try
      iFile := FileOpen(FileName, fmOpenRead);
      FileClose(iFile);

      if iFile >= 0 then
         Result := True
      else
         Result := False;
//   finally
	 except
      FileClose(iFile);
   end;
end;

function IsDirWriteable(const dir: string):Boolean;
var
  Handle:Integer;
begin
  try

    Handle:=filecreate(dir + '~bfatest');
    if (Handle) > 0 then
    begin
      IsDirWriteable:=True;
      FileClose(Handle);
      DeleteFile(dir + '~bfatest')
    end
    else
    begin
      IsDirWriteable:=False
    end;
  except
     IsDirWriteable:=False
  end;
end;



///////////////////////////////////////////////////////////////////////////////////////////////////////
// nachtraeglich kommentiert 2007-10 N.Gering
// Dateigroesse ermitteln
//
// Parameter: Filename   String mit Pfad und Dateiname
//
// Ausgabe: Dateigroesse
//
function GetFileSizeStr(FileName: string): string;
var
   hFile, iSize: cardinal;
begin
   hFile := CreateFile(PAnsiChar(FileName), 0, FILE_SHARE_READ, nil,
      OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
   if hFile <> INVALID_HANDLE_VALUE then
   begin
      iSize := GetFileSize(hFile, nil);
      CloseHandle(hFile);
      if iSize = $0 then
         Result := _('Lesefehler')
      else
      if iSize < $400 then
         Result := IntToStr(iSize) + ' Byte'
      else
      if iSize < $100000 then
         Result := IntToStr(Round(iSize / $400)) + ' KB'
      else
      if iSize < $40000000 then
         Result := IntToStr(Round(iSize / $100000)) + ' MB'
      else
      if iSize < $10000000000 then
         Result := IntToStr(Round(iSize / $40000000)) + ' GB'
      else
         Result := IntToStr(iSize) + ' Byte';
   end
   else
      Result := _('Lesefehler');

end;

//-----------------------------------------------------------------------------
function KeyPressed(key: integer): boolean;
   //////////////////////////////////////////////////////////////////
   ///  16.03.2006 Erstellt by SV.                                ///
   ///                                                            ///
   ///  Diese Funktion berprft ob die Taste 'key' gedrckt      ///
   ///  wird.                                                     ///
   ///                                                            ///
   ///  Ausgabe: True wenn Taste gedrckt                         ///
   ///           False wenn Taste nicht gedrckt                  ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
begin
   case GetKeyState(key) of
      -127, -128:
         Result := True;
   else
      Result := False;
   end;
end;


//-----------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//   Erweiterte Funktion zur Auswahl eines Verzeichnisses
//   Die Funktikon wurde um einen Button zum Erzeugen eines Verzeichnisses erweitert
//   NogSelectDirectory('Caption', 'c:\', dir, False, False, True);
//   lbl_PathtoDir.Caption := dir;
function NogSelectDirectory(const Caption: string; const Root: WideString;
   var Directory: string; EditBox: boolean = False; ShowFiles: boolean = False;
   AllowCreateDirs: boolean = True): boolean;

   function SelectDirCB(Wnd: HWND; uMsg: UINT; lParam, lpData: lParam): integer; stdcall;
   begin
      case uMsg of
         BFFM_INITIALIZED:
            SendMessage(Wnd, BFFM_SETSELECTION, Ord(True),
               integer(lpData));
      {BFFM_SELCHANGED:
      begin
        SHGetPathFromIDList(PItemIDList(lParam), @PathName);
      end;}
      end;
      Result := 0;
   end;
var
   WindowList: Pointer;
   BrowseInfo: TBrowseInfo;
   Buffer: PChar;
   RootItemIDList, ItemIDList: PItemIDList;
   ShellMalloc: IMalloc;
   IDesktopFolder: IShellFolder;
   Eaten, Flags: longword;
const
   BIF_USENEWUI = $0040;
   BIF_NOCREATEDIRS = $0200;
begin
   Result := False;
   if not DirectoryExists(Directory) then
      Directory := '';
   FillChar(BrowseInfo, SizeOf(BrowseInfo), 0);
   if (ShGetMalloc(ShellMalloc) = S_OK) and (ShellMalloc <> nil) then
   begin
      Buffer := ShellMalloc.Alloc(MAX_PATH);
      try
         RootItemIDList := nil;
         if Root <> '' then
         begin
            SHGetDesktopFolder(IDesktopFolder);
            IDesktopFolder.ParseDisplayName(Application.Handle, nil,
               POleStr(Root), Eaten, RootItemIDList, Flags);
         end;
         OleInitialize(nil);
         with BrowseInfo do
         begin
            hwndOwner := Application.Handle;
            pidlRoot := RootItemIDList;
            pszDisplayName := Buffer;
            lpszTitle := PChar(Caption);
            ulFlags := BIF_RETURNONLYFSDIRS or BIF_USENEWUI or
               BIF_EDITBOX * Ord(EditBox) or BIF_BROWSEINCLUDEFILES *
               Ord(ShowFiles) or BIF_NOCREATEDIRS * Ord(not AllowCreateDirs);
            lpfn := @SelectDirCB;
            if Directory <> '' then
               lParam := integer(PChar(Directory));
         end;
         WindowList := DisableTaskWindows(0);
         try
            ItemIDList := ShBrowseForFolder(BrowseInfo);
         finally
            EnableTaskWindows(WindowList);
         end;
         Result := ItemIDList <> nil;
         if Result then
         begin
            ShGetPathFromIDList(ItemIDList, Buffer);
            ShellMalloc.Free(ItemIDList);
            Directory := Buffer;
         end;
      finally
         ShellMalloc.Free(Buffer);
      end;
   end;
end;


(*
  Zweck:
    Markiert die Bereiche des Textes, die eine Zahl enthalten.
    Eine Zahl ist definiert als Teilstring, der
    1. zwischen Blanks steht, und aus
    2. einer Ziffernfolge,
    3. dem Komma oder Punkt,
    4. einer 2. Ziffernfolge
    besteht.
  Eingabe:
    'istZahl': Ist das Array, in das die Markierung eingetragen wird.
    'Text'   : Ist der zu markierende Text
*)
procedure determineNumberAreas (var   istZahl: array of Boolean;
                                const Text   : string          );

  type
    eZust = (zustBlank,            //  Leerzeichen vor der Zahl = Start
             zustErsteZiffer,
             zustZahlTeil01,       //  Der Teil der Zahl vor dem Komma
             zustKomma,            //  Komma oder Punkt, genau ein Zeichen
             zustZahlTeil02 );     //  Der Teil der Zahl nach dem Komma


  var
    xZust    : eZust;
    xCh      : Char;
    xIdx     : Integer;
    xIdxMax  : Integer;
    xIdxFirst: Integer;

  begin
    xZust     := zustErsteZiffer;  //  Die Zahl knnte mit dem 1. Zeichen beginnen
    xIdxMax   := Length (Text);
    xIdx      := 1;
    xIdxFirst := 1;


    while xIdx <= xIdxMax do
    begin
      xCh := Text [xIdx];

      case xZust of
      zustBlank:
        begin
          istZahl [xIdx] := False;

          if xCh = ' ' then
            xZust := zustErsteZiffer;

          Inc (xIdx);
        end;

      zustErsteZiffer:
        begin
          if xCh in ['0'..'9'] then
          begin
            xIdxFirst      := xIdx;
            istZahl [xIdx] := True;
            Inc     (xIdx);
            xZust          := zustZahlTeil01;
          end
          else
            xZust          := zustBlank;
        end;

      zustZahlTeil01:
        begin
          if xCh in ['0'..'9'] then
          begin
            istZahl [xIdx] := True;
            Inc     (xIdx);
          end
          else
          if    (xCh = ',')
             or (xCh = '.')
          then begin
            istZahl [xIdx] := True;
            Inc     (xIdx);
            xZust := zustKomma;
          end
          else
          begin
            xIdx  := xIdxFirst;
            xZust := zustBlank;
          end;
        end;

      zustKomma:
        begin
          if xCh in ['0'..'9'] then
          begin
            istZahl [xIdx] := True;
            Inc     (xIdx);

            if xIdx > xIdxMax then     //  Text-Ende erreicht
              xIdxFirst := xIdx;       //  Die Zahl ist fertig!

            xZust := zustZahlTeil02;
          end
          else
          begin
            xIdx  := xIdxFirst;
            xZust := zustBlank;
          end;
        end;

      zustZahlTeil02:
        begin
          if xCh in ['0'..'9'] then
          begin
            istZahl [xIdx] := True;
            Inc     (xIdx);

            if xIdx > xIdxMax then     //  Text-Ende erreicht
              xIdxFirst := xIdx;       //  Die Zahl ist fertig!
          end
          else
          if xCh = ' ' then
          begin
            xZust     := zustBlank;
            xIdxFirst := xIdx;         //  Die Zahl ist fertig!
          end
          else
          begin
            xIdx      := xIdxFirst;
            xZust     := zustBlank;
          end;
        end;
      end;
    end;

    //  nicht vollstndige Zahlen de-markieren
    //
    for xIdx := xIdxFirst to xIdxMax do
      istZahl [xIdx] := False;
  end;  //  determineNumberAreas


//-----------------------------------------------------------------------------
function StrToWords(const InputStr: string): TStringList;
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////
   ///  30.09.2006 modifiziert by NG.
   ///
   ///  Diese Function zerlegt einen String in einzelne Worte
   ///  bzw Phrasen und listet diese in einer Stringlist auf.
   ///                                                                                                                        //
   ///  Parameter: InputStr        String mit Suchbegriffen
   ///
   ///  Ausgabe: StringList mit einzelnen Worten und Phrasen
   ///
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////
const
   sTrennZeichen = ' .,!?';
var
   iCount, iWortStart, iWortEnde: integer;
   bPhrase: boolean;
   sOP, sWort, slStr: string;
begin
   Result := TStringList.Create;
   Result.Clear;
   iWortStart := 0;
   iWortEnde := 0;
   iCount := 0;
   bPhrase := False;
   sOP := '';
   // trim the input!
   slStr := trim(InputStr);
   while iCount <= Length(slStr) do
   begin
      if IsDelimiter('"', slStr, iCount) then
      begin
         if not bPhrase then
         begin
            bPhrase := True;
            iWortStart := iCount + 1;
            if IsDelimiter('+-*', slStr, iCount - 1) then
               sOp := slStr[iCount - 1]
            else
               sOp := '';
         end
         else
            bPhrase := False;
      end;
      if IsDelimiter(sTrennZeichen, slStr, iCount) and not bPhrase then
      begin
         if iWortStart <= iWortEnde then
         begin
            iWortEnde := iCount;
            sWort := Trim(StringReplace(Copy(slStr, iWortStart,
               iWortEnde - iWortStart), '"', '', [rfReplaceAll]));
            if ((Length(trim(sWort)) >= 1) and (not IsDelimiter('+-*', sWort, 1) or
               (Length(trim(sWort)) > 1))) then
               Result.Add(sOP + sWort);
            iWortStart := iWortEnde + 1;
         end;
      end;
      iWortEnde := iCount;
      Inc(iCount);
   end;
   if iWortStart <= iWortEnde then
      Result.Add(sOP + Trim(StringReplace(Copy(slStr, iWortStart, 1 +
         iWortEnde - iWortStart), '"', '', [rfReplaceAll])));
end;


//-----------------------------------------------------------------------------
function StrToWords2(const InputStr: string): TStringList;

   //////////////////////////////////////////////////////////////////
   ///  30.08.2005 Erstellt by SV.                                ///
   ///                                                            ///
   ///  Diese Function zerlegt einen String in einzelne Worte     ///
   ///  bzw Phrasen und listet diese in einer Stringlist auf.     ///
   ///                                                            ///
   ///  Parameter: InputStr        String mit Suchbegriffen       ///
   ///                                                            ///
   ///  Ausgabe: StringList mit einzelnen Worten und Phrasen      ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////

const
   sTrennZeichen = ' .,!?';

var
   iCount,
   iWortStart,
   iWortEnde : integer;
   bPhrase   : boolean;
   sWort     : string;
   xIstZahl  : array of Boolean;

begin
   Result := TStringList.Create;
   Result.Clear;

   if trim(InputStr) = '' then
      exit;
   iWortStart := 0;
   iWortEnde := 0;
   iCount := 0;
   bPhrase := False;

   SetLength            (xIstZahl, Length (InputStr) +1);
   determineNumberAreas (xIstZahl,         InputStr    );

   while iCount <= Length(InputStr) do
   begin
      if IsDelimiter('"', InputStr, iCount) then
      begin
         if not bPhrase then
         begin
            bPhrase := True;
            iWortStart := iCount + 1;
            //if IsDelimiter('+-*', InputStr, iCount - 1) then inc(iCount, 1);
         end
         else
            bPhrase := False;
      end;
      if        IsDelimiter(sTrennZeichen, InputStr, iCount)
        and not bPhrase
        and not xIstZahl [iCount]
      then begin
         if iWortStart <= iWortEnde then
         begin
            iWortEnde := iCount;
            sWort := Trim(StringReplace(Copy(InputStr, iWortStart,
               iWortEnde - iWortStart), '"', '', [rfReplaceAll]));
            if swort <> '' then
            begin
               if (Ord(sWort[1]) = $2B) or (Ord(sWort[1]) = $2D) then
                  sWort := Copy(sWort, 2, Length(sWort));
               Result.Add(sWort);
            end;
            //ShowMessage(sWort);
            iWortStart := iWortEnde + 1;
         end;
      end;
      iWortEnde := iCount;
      Inc(iCount);
   end;
   if iWortStart < iWortEnde then
      Result.Add(Trim(StringReplace(Copy(InputStr, iWortStart, 1 +
         iWortEnde - iWortStart), '"', '', [rfReplaceAll])));
end;


function StrToWords3 (const InputStr: string): Twordandsemlist;

(*   /////////////////////////////////////////////////////////////////////////////////////////////////////////////
   ///  In dieser Fassung werden Zahlen mit Nachkommastellen bercksichtigt
   ///  Solche Zahlen sind definiert als Zahlen, die ein Komma oder einen Punkt
   ///  enthalten mssen, und von Leerzeichen begrenzt sind.
   ///
   ///  Diese Function zerlegt einen String in einzelne Worte
   ///  bzw Phrasen und listet diese in einer Liste auf.
   ///                                                                                                                        //
   ///  Parameter: InputStr        String mit Suchbegriffen
   ///
   ///  Ausgabe: Liste
   ///    m.frey: Hier wird anstelle der TStringList die Twordandsemlist aus Ausgabe verwendet
   ///
   ///  f.schneider: in dieser Funktion werden " rausgeparst
   ///
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////
*)

const
   // v.3.5.0.89 f.schneider Was sollen all diese Trennzeichen bei der Suche ?
   //sTrennZeichen = ' .,!?';
   sTrennZeichen = ' ';

var
   iCount,
   iWortStart,
   iWortEnde : integer;
   bPhrase,
   bsem      : boolean;
   slwords,
   lslsem    : TStringList;
   sOP,
   sWort,
   slStr     : string;
   xIstZahl  : array of Boolean;

begin
   bsem       := false;
   iWortStart := 1;
   iWortEnde  := 0;
   iCount     := 0;
   bPhrase    := False;
   sOP        := '';
   slStr      := trim (InputStr);
   slwords    := TStringList.Create;
   lslsem     := TStringList.Create;

   slwords.Clear;
   SetLength            (xIstZahl, Length (slStr) +1);
   determineNumberAreas (xIstZahl,         slStr    );

   while iCount <= Length (slStr) do
   begin
      if IsDelimiter('"', slStr, iCount) then
      begin
         if not bPhrase then
         begin
            bPhrase := True;
            iWortStart := iCount + 1;
            if IsDelimiter('+-*', slStr, iCount - 1) then
               sOp := slStr[iCount - 1]
            else
               sOp := '';
            bsem := True;
         end
         else
            bPhrase := False;
      end;
      if        IsDelimiter(sTrennZeichen, slStr, iCount)
        and not bPhrase
        and not xIstZahl [iCount]
      then begin
         if iWortStart <= iWortEnde then
         begin
            iWortEnde := iCount;
            sWort := Trim(StringReplace(Copy(slStr, iWortStart,
               iWortEnde - iWortStart), '"', '', [rfReplaceAll]));
            if ((Length(trim(sWort)) >= 1) and (not IsDelimiter('+-*', sWort, 1) or
               (Length(trim(sWort)) > 1))) then
            begin
               slwords.Add(sOP + sWort);
               if bsem then
               begin
                  lslsem.add('y');
               end
               else
               begin
                  lslsem.add('n');
               end;
            end;
            bsem := False;
            iWortStart := iWortEnde + 1;
         end;
      end;
      iWortEnde := iCount;
      Inc(iCount);
   end;

   if iWortStart <= iWortEnde then
   begin
      slwords.Add(sOP + Trim(StringReplace(Copy(slStr, iWortStart, 1 +
         iWortEnde - iWortStart), '"', '', [rfReplaceAll])));
      if bsem then
      begin
         lslsem.add('y');
      end
      else
      begin
         lslsem.add('n');
      end;
   end;
   Result.words := slwords;
   Result.sems := lslsem;
end;


//-----------------------------------------------------------------------------
function StrToWords4(const InputStr: string): TStringList;

   /////////////////////////////////////////////////////////////////////////////////////////////////////////////
   ///  In dieser Fassung werden Zahlen mit Nachkommastellen bercksichtigt
   ///  Solche Zahlen sind definiert als Zahlen, die ein Komma oder einen Punkt
   ///  enthalten mssen, und von Leerzeichen begrenzt sind.
   ///
   ///  30.09.2006 modifiziert by NG.
   ///
   ///  Diese Function zerlegt einen String in einzelne Worte
   ///  bzw Phrasen und listet diese in einer Stringlist auf.
   ///                                                                                                                        //
   ///  Parameter: InputStr        String mit Suchbegriffen
   ///
   ///  Ausgabe: StringList mit einzelnen Worten und Phrasen
   ///
   ///  f.schneider: in dieser Funktion bleiben " erhalten
   ///
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////

const
   // v.3.5.0.89 f.schneider Was sollen all diese Trennzeichen bei der Suche ?
   //sTrennZeichen = ' .,!?';
   sTrennZeichen = ' ';

var
   iCount,
   iWortStart,
   iWortEnde,
   istatus   : integer;
   bPhrase   : boolean;
   sOP,
   sWort,
   slStr     : string;
   xIstZahl  : array of Boolean;

begin
   Result := TStringList.Create;
   Result.Clear;
   iWortStart := 1;
   iWortEnde := 0;
   iCount := 0;
   istatus := 0;
   bPhrase := False;
   sOP := '';
   // trim the input!
   slStr := trim(InputStr);

   SetLength            (xIstZahl, Length (slStr) +1);
   determineNumberAreas (xIstZahl,         slStr    );

   while iCount <= Length(slStr) do
   begin
      if IsDelimiter('"', slStr, iCount) then
      begin
         if not bPhrase then
         begin
            bPhrase := True;
            iWortStart := iCount;
            istatus := 1;
            if IsDelimiter('+-*', slStr, iCount - 1) then
               sOp := slStr[iCount - 1]
            else
               sOp := '';
         end
         else
         begin
            bPhrase := False;
            if istatus = 1 then
            begin
               istatus := 2;
            end;
         end;
      end;
      if        IsDelimiter(sTrennZeichen, slStr, iCount)
        and not bPhrase
        and not xIstZahl [iCount]
      then begin
         if iWortStart <= iWortEnde then
         begin
            iWortEnde := iCount;
            if istatus = 2 then
            begin
               sWort := Trim(Copy(slStr, iWortStart, iWortEnde - iWortStart));

            end
            else
            begin
               sWort := Trim(StringReplace(Copy(slStr, iWortStart,
                  iWortEnde - iWortStart), '"', '', [rfReplaceAll]));
            end;
            if ((Length(trim(sWort)) >= 1) and (not IsDelimiter('+-*', sWort, 1) or
               (Length(trim(sWort)) > 1))) then
               Result.Add(sOP + sWort);
            iWortStart := iWortEnde + 1;
         end;
      end;
      iWortEnde := iCount;
      Inc(iCount);
   end;
   if iWortStart <= iWortEnde then
   begin
      if istatus = 2 then
      begin
         Result.Add(sOP + Trim(Copy(slStr, iWortStart, 1 + iWortEnde - iWortStart)));
      end
      else
      begin
         Result.Add(sOP + Trim(StringReplace(Copy(slStr, iWortStart, 1 +
            iWortEnde - iWortStart), '"', '', [rfReplaceAll])));
      end;
   end;
end;

(* Die ersetzen wir mal durch eine korrigierte StrToWords5
Die wir eigentlich im Viewer nurnoch fr Global Favorites genommen
//-----------------------------------------------------------------------------
function StrToWords5(const InputStr: string): TStringList;
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////
   ///  30.09.2006 modifiziert by NG.
   ///
   ///  Diese Function zerlegt einen String in einzelne Worte
   ///  bzw Phrasen und listet diese in einer Stringlist auf.
   ///                                                                                                                        //
   ///  Parameter: InputStr        String mit Suchbegriffen
   ///
   ///  Ausgabe: StringList mit einzelnen Worten und Phrasen
   ///  Punkt nicht beachtet
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////

const
   sTrennZeichen = ',';
var
   iCount, iWortStart, iWortEnde, istatus: integer;
   bPhrase: boolean;
   sOP, sWort, slStr: string;
begin
   Result := TStringList.Create;
   Result.Clear;
   iWortStart := 0;
   iWortEnde := 0;
   iCount := 0;
   istatus := 0;
   bPhrase := False;
   sOP := '';
   // trim the input!
   slStr := trim(InputStr);
   while iCount <= Length(slStr) do
   begin
      if IsDelimiter('"', slStr, iCount) then
      begin
         if not bPhrase then
         begin
            bPhrase := True;
            iWortStart := iCount;
            istatus := 1;
            if IsDelimiter('+-*', slStr, iCount - 1) then
               sOp := slStr[iCount - 1]
            else
               sOp := '';
         end
         else
         begin
            bPhrase := False;
            if istatus = 1 then
            begin
               istatus := 2;
            end;
         end;
      end;
      if IsDelimiter(sTrennZeichen, slStr, iCount) and not bPhrase then
      begin
         if iWortStart <= iWortEnde then
         begin
            iWortEnde := iCount;
            if istatus = 2 then
            begin
               sWort := Trim(Copy(slStr, iWortStart, iWortEnde - iWortStart));

            end
            else
            begin
               sWort := Trim(StringReplace(Copy(slStr, iWortStart,
                  iWortEnde - iWortStart), '"', '', [rfReplaceAll]));
                  // In der Variante, die in Tdm_MainForm war, war hinter "iWortStart" noch "-1"
            end;
            if ((Length(trim(sWort)) >= 1) and (not IsDelimiter('+-*', sWort, 1) or
               (Length(trim(sWort)) > 1))) then
               Result.Add(sOP + sWort);
            iWortStart := iWortEnde + 1;
         end;
      end;
      iWortEnde := iCount;
      Inc(iCount);
   end;
   if iWortStart <= iWortEnde then
   begin
      if istatus = 2 then
      begin
         Result.Add(sOP + Trim(Copy(slStr, iWortStart, 1 + iWortEnde - iWortStart)));
      end
      else
      begin
         Result.Add(sOP + Trim(StringReplace(Copy(slStr, iWortStart, 1 +
            iWortEnde - iWortStart), '"', '', [rfReplaceAll])));
      end;
   end;
end;
*)
//-----------------------------------------------------------------------------
function StrToWords5(const InputStr: string): TStringList;
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////
   ///  30.09.2006 modifiziert by NG.
   ///
   ///  Diese Function zerlegt einen String in einzelne Worte
   ///  bzw Phrasen und listet diese in einer Stringlist auf.
   ///                                                                                                                        //
   ///  Parameter: InputStr        String mit Suchbegriffen
   ///
   ///  Ausgabe: StringList mit einzelnen Worten und Phrasen
   ///
   ///  20.07.2010 berarbeitet von f.schneider
   ///  der String wird nach Komma getrennt, Anfhrungszeichen entfernt,
   ///  Leerzeichen bleiben erhalten. Kommazahlen mssen daher in
   ///  Anfhrungszeichen gefasst werden, oder es wurde schon auf ; als Trennzeichen umgestellt
   ///
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////

const
   sTrennZeichen = ',';
var
   iCount, iWortStart, iWortEnde: integer;
   bPhrase: boolean;
   sOP, sWort, slStr: string;
begin
   Result := TStringList.Create;
   Result.Clear;
   iWortStart := 1;
   iWortEnde := 0;
   iCount := 0;
   bPhrase := False;
   sOP := '';
   // trim the input!
   slStr := trim(InputStr);
   while iCount <= Length(slStr) do
   begin
      if IsDelimiter('"', slStr, iCount) then
      begin
         if not bPhrase then
         begin
            bPhrase := True;
            iWortStart := iCount + 1;
            if IsDelimiter('+-*', slStr, iCount - 1) then
               sOp := slStr[iCount - 1]
            else
               sOp := '';
         end
         else
            bPhrase := False;
      end;
      if IsDelimiter(sTrennZeichen, slStr, iCount) and not bPhrase then
      begin
         if iWortStart <= iWortEnde then
         begin
           iWortEnde := iCount;
            sWort := Copy(slStr, iWortStart, iWortEnde - iWortStart);
            sWort := Trim(StringReplace(sWort, '"', '', [rfReplaceAll]));

            if ((Length(trim(sWort)) >= 1) and (not IsDelimiter('+-*', sWort, 1) or
               (Length(trim(sWort)) > 1))) then
               Result.Add(sOP + sWort);
            iWortStart := iWortEnde + 1;
         end;
      end;
       iWortEnde := iCount;
      Inc(iCount);

   end;
   if iWortStart <= iWortEnde then
      Result.Add(sOP + Trim(StringReplace(Copy(slStr, iWortStart, 1 +
         iWortEnde - iWortStart), '"', '', [rfReplaceAll])));
end;


//-----------------------------------------------------------------------------
function StrToWords6(const InputStr: string): TStringList;
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////
   ///  30.09.2006 modifiziert by NG.
   ///
   ///  Diese Function zerlegt einen String in einzelne Worte
   ///  bzw Phrasen und listet diese in einer Stringlist auf.
   ///                                                                                                                        //
   ///  Parameter: InputStr        String mit Suchbegriffen
   ///
   ///  Ausgabe: StringList mit einzelnen Worten und Phrasen
   ///
   ///  15.07.2010 berarbeitet von f.schneider
   ///  der String wird nach globalDelimiter getrennt, Anfhrungszeichen entfernt,
   ///  Leerzeichen bleiben erhalten. Kommazahlen mssen daher in
   ///  Anfhrungszeichen gefasst werden, oder es wurde schon auf ; als Trennzeichen umgestellt
   ///
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////

const
   sTrennZeichen = globalDelimiter;
var
   iCount, iWortStart, iWortEnde: integer;
   bPhrase: boolean;
   sOP, sWort, slStr: string;
begin
   Result := TStringList.Create;
   Result.Clear;
   iWortStart := 1;
   iWortEnde := 0;
   iCount := 0;
   bPhrase := False;
   sOP := '';
   // trim the input!
   slStr := trim(InputStr);
   while iCount <= Length(slStr) do
   begin
      if IsDelimiter('"', slStr, iCount) then
      begin
         if not bPhrase then
         begin
            bPhrase := True;
            iWortStart := iCount + 1;
            if IsDelimiter('+-*', slStr, iCount - 1) then
               sOp := slStr[iCount - 1]
            else
               sOp := '';
         end
         else
            bPhrase := False;
      end;
      if IsDelimiter(sTrennZeichen, slStr, iCount) and not bPhrase then
      begin
         if iWortStart <= iWortEnde then
         begin
           iWortEnde := iCount;
            sWort := Copy(slStr, iWortStart, iWortEnde - iWortStart);
            sWort := Trim(StringReplace(sWort, '"', '', [rfReplaceAll]));

            if ((Length(trim(sWort)) >= 1) and (not IsDelimiter('+-*', sWort, 1) or
               (Length(trim(sWort)) > 1))) then
               Result.Add(sOP + sWort);
            iWortStart := iWortEnde + 1;
         end;
      end;
       iWortEnde := iCount;
      Inc(iCount);

   end;
   if iWortStart <= iWortEnde then
      Result.Add(sOP + Trim(StringReplace(Copy(slStr, iWortStart, 1 +
         iWortEnde - iWortStart), '"', '', [rfReplaceAll])));
end;

//-----------------------------------------------------------------------------
function StrToWords7(const InputStr: string): TStringList;
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////
   ///  24.09.2009 by m.frey
   ///
   ///  Diese Function zerlegt einen String in einzelne Worte
   ///  und listet diese in einer Stringlist auf.
   ///
   /// nachdem die anderne Funktionen nicht ausreichend dokumentiert sind, blieb
   /// mir nichts anders brig als eine eigene neue zu machen.
   /// Die hier hat als Trennzeichen nur ein Komma. Es gibt kein Quote (Anfhrungszeichen)
   /// Sie ist dafr einfacher und schneller
   ///
   ///  Parameter: InputStr        String mit Suchbegriffen
   ///
   ///  Ausgabe: StringList mit einzelnen Worten und Phrasen
   ///
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////

const
   sTrennZeichen = globalDelimiter;
var
   iCount, iWortStart, iWortEnde: integer;  bPhrase: boolean;
   sWort, slStr: string;
begin
   Result := TStringList.Create;
   Result.Clear;
   iWortStart := 1;
   iWortEnde := 0;
   iCount := 0;
   bPhrase := False;
   // trim the input!
   slStr := trim(InputStr);
   while iCount <= Length(slStr) do
   begin
      if IsDelimiter(sTrennZeichen, slStr, iCount) and not bPhrase then
      begin
         if iWortStart <= iWortEnde then
         begin
            iWortEnde := iCount;
            sWort := Trim(Copy(slStr, iWortStart, iWortEnde - iWortStart));
            if (Length(sWort) >= 1) then
               Result.Add(sWort);
            iWortStart := iWortEnde + 1;
         end;
      end;
      iWortEnde := iCount;
      Inc(iCount);
   end;
   if iWortStart <= iWortEnde then
   begin
      sWort := Trim(Copy(slStr, iWortStart, maxint));
      Result.Add(sWort);
   end;
end;

//-----------------------------------------------------------------------------
function StrToWordswvl(const InputStr: string): TStringList;
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////
   ///  30.09.2006 modifiziert by NG.
   ///
   ///  Diese Function zerlegt einen String in einzelne Worte
   ///  bzw Phrasen und listet diese in einer Stringlist auf.
   ///                                                                                                                        //
   ///  Parameter: InputStr        String mit Suchbegriffen
   ///
   ///  Ausgabe: StringList mit einzelnen Worten und Phrasen
   ///
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////

const
   sTrennZeichen = ' ,!?';
var
   iCount, iWortStart, iWortEnde: integer;
   bPhrase: boolean;
   sOP, sWort, slStr: string;
begin
   Result := TStringList.Create;
   Result.Clear;
   iWortStart := 0;
   iWortEnde := 0;
   iCount := 0;
   bPhrase := False;
   sOP := '';
   // trim the input!
   slStr := trim(InputStr);
   while iCount <= Length(slStr) do
   begin
      if IsDelimiter('"', slStr, iCount) then
      begin
         if not bPhrase then
         begin
            bPhrase := True;
            iWortStart := iCount + 1;
            if IsDelimiter('+-*', slStr, iCount - 1) then
               sOp := slStr[iCount - 1]
            else
               sOp := '';
         end
         else
            bPhrase := False;
      end;
      if IsDelimiter(sTrennZeichen, slStr, iCount) and not bPhrase then
      begin
         if iWortStart <= iWortEnde then
         begin
            iWortEnde := iCount;
            sWort := Trim(StringReplace(Copy(slStr, iWortStart,
               iWortEnde - iWortStart), '"', '', [rfReplaceAll]));
            if ((Length(trim(sWort)) >= 1) and (not IsDelimiter('+-*', sWort, 1) or
               (Length(trim(sWort)) > 1))) then
               Result.Add(sOP + sWort);
            iWortStart := iWortEnde + 1;
         end;
      end;
      iWortEnde := iCount;
      Inc(iCount);
   end;
   if (iWortStart <= iWortEnde) and (iWortEnde <> 0) then
      Result.Add(sOP + Trim(StringReplace(Copy(slStr, iWortStart, 1 +
         iWortEnde - iWortStart), '"', '', [rfReplaceAll])));
end;

//-----------------------------------------------------------------------------
(*
function StrToWordswvl2(const InputStr: string): TStringList;
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////
   ///  30.09.2006 modifiziert by NG.
   ///
   ///  Diese Function zerlegt einen String in einzelne Worte
   ///  bzw Phrasen und listet diese in einer Stringlist auf.
   ///                                                                                                                        //
   ///  Parameter: InputStr        String mit Suchbegriffen
   ///
   ///  Ausgabe: StringList mit einzelnen Worten und Phrasen
   ///
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////

const
   sTrennZeichen = ',!?';
var
   iCount, iWortStart, iWortEnde: integer;
   bPhrase: boolean;
   sOP, sWort, slStr: string;
begin
   Result := TStringList.Create;
   Result.Clear;
   iWortStart := 1;
   iWortEnde := 0;
   iCount := 0;
   bPhrase := False;
   sOP := '';
   // trim the input!
   slStr := trim(InputStr);
   while iCount <= Length(slStr) do
   begin
      if IsDelimiter('"', slStr, iCount) then
      begin
         if not bPhrase then
         begin
            bPhrase := True;
            iWortStart := iCount + 1;
            if IsDelimiter('+-*', slStr, iCount - 1) then
               sOp := slStr[iCount - 1]
            else
               sOp := '';
         end
         else
            bPhrase := False;
      end;
      if IsDelimiter(sTrennZeichen, slStr, iCount) and not bPhrase then
      begin
         if iWortStart <= iWortEnde then
         begin
            iWortEnde := iCount;
            sWort := Trim(StringReplace(Copy(slStr, iWortStart,
               iWortEnde - iWortStart), '"', '', [rfReplaceAll]));
            if ((Length(trim(sWort)) >= 1) and (not IsDelimiter('+-*', sWort, 1) or
               (Length(trim(sWort)) > 1))) then
               Result.Add(sOP + sWort);
            iWortStart := iWortEnde + 1;
         end;
      end;
      iWortEnde := iCount;
      Inc(iCount);
   end;
   if (iWortStart <= iWortEnde) and (iWortEnde <> 0) then
      Result.Add(sOP + Trim(StringReplace(Copy(slStr, iWortStart, 1 +
         iWortEnde - iWortStart), '"', '', [rfReplaceAll])));
end;
(**)

//-----------------------------------------------------------------------------
{3.6<}
function StrToWordswvl2(const InputStr: string): TStringList;
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////
   ///  01.12.2009 modifiziert by m.frey
begin
   Result := StrToWords7( stringreplace( InputStr , ',', globalDelimiter, [rfReplaceAll]) );
end;
{>3.6}

//-----------------------------------------------------------------------------
function StrToWords0(const InputStr: string): TStringList;
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////
   ///  09.11.2009 by m.frey
   ///
   ///  Dies ist eine Dummy-Function. Sei zerlegt gar nicht, sondern weist
   ///  den String halt der Stringliste zu.
   ///
   ///  Parameter: InputStr        String mit Suchbegriffen
   ///
   ///  Ausgabe: StringList
   ///
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////

begin
   Result := TStringList.Create;
   Result.Text := InputStr;
end;

//-----------------------------------------------------------------------------
function StrToWordsTab(const InputStr: string): TStringList;
   /////////////////////////////////////////////////////////////////////////////////////////////////////////////
   ///  24.11.2009 by m.frey
   ///
   ///  Diese Function zerlegt einen String in einzelne Worte
   ///  und listet diese in einer Stringlist auf.
   ///
   /// Die Variante hier hat als Trennzeichen nur ein Tabulator #9
   /// Es gibt kein Quote (Anfhrungszeichen)
   ///
   ///  Parameter: InputStr        String mit Suchbegriffen
   ///
   ///  Ausgabe: StringList mit einzelnen Worten und Phrasen

const
   sTrennZeichen = #9;
var
   iCount, iWortStart, iWortEnde: integer;  bPhrase: boolean;
   sWort, slStr: string;
begin
   Result := TStringList.Create;
   Result.Clear;
   iWortStart := 1;
   iWortEnde := 0;
   iCount := 0;
   bPhrase := False;
   // trim the input!
   slStr := trim(InputStr);
   while iCount <= Length(slStr) do
   begin
      if IsDelimiter(sTrennZeichen, slStr, iCount) and not bPhrase then
      begin
         if iWortStart <= iWortEnde then
         begin
            iWortEnde := iCount;
            sWort := Trim(Copy(slStr, iWortStart, iWortEnde - iWortStart));
            if (Length(sWort) >= 1) then
               Result.Add(sWort);
            iWortStart := iWortEnde + 1;
         end;
      end;
      iWortEnde := iCount;
      Inc(iCount);
   end;
   if iWortStart <= iWortEnde then
   begin
      sWort := Trim(Copy(slStr, iWortStart, maxint));
      Result.Add(sWort);
   end;
end;


function StrToWordsCRLF(const InputStr: string): TStringList;
   //////////////////////////////////////////////////////////////////
   ///  24.11.2009 by m.frey
   ///
   ///  Diese Function zerlegt einen String in einzelne Worte
   ///  und listet diese in einer Stringlist auf.
   ///
   /// Die Variante hier hat als Trennzeichen nur ein Tabulator #9
   /// Es gibt kein Quote (Anfhrungszeichen)
   ///
   ///  Parameter: InputStr        String mit Suchbegriffen
   ///
   ///  Ausgabe: StringList mit einzelnen Worten und Phrasen

const
   sTrennZeichen = #13#10;
var
   iCount, iWortStart, iWortEnde: integer;  bPhrase: boolean;
   sWort, slStr: string;
begin
   Result := TStringList.Create;
   Result.Clear;
   iWortStart := 1;
   iWortEnde := 0;
   iCount := 0;
   bPhrase := False;
   // trim the input!
   slStr := trim(InputStr);
   while iCount <= Length(slStr) do
   begin
      if IsDelimiter(sTrennZeichen, slStr, iCount) and not bPhrase then
      begin
         if iWortStart <= iWortEnde then
         begin
            iWortEnde := iCount;
            sWort := Trim(Copy(slStr, iWortStart, iWortEnde - iWortStart));
            if (Length(sWort) >= 1) then
	            Result.Add(sWort);
            iWortStart := iWortEnde + 1;
         end;
      end;
      iWortEnde := iCount;
      Inc(iCount);
   end;
   if iWortStart <= iWortEnde then
   begin
      sWort := Trim(Copy(slStr, iWortStart, maxint));
      Result.Add(sWort);
   end;
end;

function StrToWordsLF(const InputStr: string): TStringList;
   //////////////////////////////////////////////////////////////////
   ///  24.11.2009 by m.frey
   ///
   ///  Diese Function zerlegt einen String in einzelne Worte
   ///  und listet diese in einer Stringlist auf.
   ///
   /// Die Variante hier hat als Trennzeichen nur ein LineFeed #10
   /// Es gibt kein Quote (Anfhrungszeichen)
   ///
   ///  Parameter: InputStr        String mit Suchbegriffen
   ///
   ///  Ausgabe: StringList mit einzelnen Worten und Phrasen

   ///  wird fr die Grenermittlung der Textannos bentigt, Leerzeilen bleiben daher bestehen
   ///  und es wird auch nicht getrimmt
   
const
   sTrennZeichen = #10;
var
   iCount, iWortStart, iWortEnde: integer;  bPhrase: boolean;
   sWort, slStr: string;
begin
   Result := TStringList.Create;
   Result.Clear;
   iWortStart := 1;
   iWortEnde := 0;
   iCount := 0;
   bPhrase := False;
   slStr := InputStr;
   while iCount <= Length(slStr) do
   begin
      if IsDelimiter(sTrennZeichen, slStr, iCount) and not bPhrase then
      begin
         if iWortStart <= iWortEnde then
         begin
            iWortEnde := iCount;
            sWort := Copy(slStr, iWortStart, iWortEnde - iWortStart);
            //if (Length(sWort) >= 1) then   //Leerzeilen mssen wir behalten
            Result.Add(sWort);
            iWortStart := iWortEnde + 1;
         end
         else
            Result.Add('');
      end;
      iWortEnde := iCount;
      Inc(iCount);
   end;
   if iWortStart <= iWortEnde then
   begin
      sWort := Copy(slStr, iWortStart, maxint);
      Result.Add(sWort);
   end;
end;


//-----------------------------------------------------------------------------
function WordsToStr(ts:TStrings):string;
var
  i : Integer;
  sep: string;

begin
  Result := '';
  for i:= 0 to ts.Count -1 do
  begin
    if Result <> '' then
      sep := globalDelimiter;
    if ts[i] <> '' then
    begin
//      if pos(',',ts[i]) > 0 then
//        ts[i]:= #34 + ts[i] + #34;
      Result:= Result + sep + ts[i];
    end;
  end;
//  ts.Free;
end;

//-----------------------------------------------------------------------------
function GetStringFromResource(Name: string): string;
   //////////////////////////////////////////////////////////////////
   ///  02.05.2005 Erstellt by SV.                                ///
   ///                                                            ///
   ///  Diese Funktion liest die Kompilierte Resource in 'Name'   ///
   ///  aus und liefert ihren Inhalt als Delphi String            ///
   ///                                                            ///
   ///  Ausgabe: String                                           ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
begin
   with TResourceStream.Create(HInstance, Name, RT_RCDATA) do
      try
         SetLength(Result, Size);
         Read(Result[1], Size);
      finally
         Free;
      end;
end;

//-----------------------------------------------------------------------------
function CleanFileName(InputStr: string): string;
var
   lsnamfile: string;
begin
   lsNamFile := stringreplace(InputStr, '*', '_', [rfReplaceAll]);
   lsNamFile := stringreplace(lsNamFile, '?', '_', [rfReplaceAll]);
//   lsNamFile := stringreplace(lsNamFile, '[', '_', [rfReplaceAll]);   // Herr Stein mchte auch die eckigen Klammern behalten
//   lsNamFile := stringreplace(lsNamFile, ']', '_', [rfReplaceAll]);   // auch das ist richtig [2098]
   lsNamFile := stringreplace(lsNamFile, '<', '_', [rfReplaceAll]);
   lsNamFile := stringreplace(lsNamFile, '>', '_', [rfReplaceAll]);
   lsNamFile := stringreplace(lsNamFile, '|', '_', [rfReplaceAll]);
   lsNamFile := stringreplace(lsNamFile, '+', '_', [rfReplaceAll]);
   lsNamFile := stringreplace(lsNamFile, '=', '_', [rfReplaceAll]);
   lsNamFile := stringreplace(lsNamFile, ':', '_', [rfReplaceAll]);
   lsNamFile := stringreplace(lsNamFile, ',', '_', [rfReplaceAll]);
   lsNamFile := stringreplace(lsNamFile, '\', '_', [rfReplaceAll]);
   lsNamFile := stringreplace(lsNamFile, '/', '_', [rfReplaceAll]);
   lsNamFile := stringreplace(lsNamFile, '"', '_', [rfReplaceAll]);
   //lsNamFile := stringreplace(lsNamFile, ' ', '_', [rfReplaceAll]);    // Herr Stein hat zu Recht bemngelt, dass Dokumente mit Leerzeichen im Namen beim lokal Speichern
   Result := lsNamFile;                                                  // die Leerzeichen durch Unterstriche ersetzt [1321]
end;

//-----------------------------------------------------------------------------
function CopyFileT(const S1,S2:string):Boolean;
begin
   result:=copyfile(pchar(s1), pchar(s2), true );
end;

//3.5.0.38 [2364]
function CopyFileF(const S1,S2:string):Boolean;
begin
   result:=copyfile(pchar(s1), pchar(s2), False );
end;

//-----------------------------------------------------------------------------
function CopyFileLog(const s1,s2:string):Boolean;
begin
   result:=copyfile(pchar(s1), pchar(s2), true );
   if not result then
   begin
      logevnt('Error copying ' + s1 + ' to ' + s2);
   end;
end;

//-----------------------------------------------------------------------------
function DeleteFileLog(const s1:string):Boolean;
begin
   result:=DeleteFile(pchar(s1) );
   if not result then
      begin
         logevnt(s1 + '" konnte nicht gelscht werden!');
      end;
end;

//-----------------------------------------------------------------------------
// if DeleteFile('c:\test.txt') then   //um eine Datei zu lschen
//   ...
// if DeleteFile('c:\test') then       //um ein Verzeichnis zu lschen
//   ...
// if DeleteFile('c:\test\*.*') then // Um alle Dateien eines Verzeichnisses zu lschen


function BFDeleteFile(const AFile: string): boolean;
var
   sh: SHFileOpStruct;
begin
  try
   ZeroMemory(@sh, sizeof(sh));
   with sh do
   begin
      Wnd := Application.Handle;
      wFunc := fo_Delete;
      pFrom := PChar(AFile + #0);
      fFlags := fof_Silent or fof_NoConfirmation;
   end;
   Result := SHFileOperation(sh) = 0;
  except
  	result:= false; //30.05.2011 b.takacs 3.5.1.16
  end;
end;



//-----------------------------------------------------------------------------
function ZiffernRechts(const S:string):string;
var i:integer;
begin
   Result:='';
   if S='' then Exit;
   i:=length(S);
   While (i>=1) and (S[i] in ['0'..'9']) do
      dec(i);
   Result := Copy(S,i+1,maxint);
end;

//-----------------------------------------------------------------------------
// Achtung: der Aufrufer muss die TStringlist wieder freigeben
function ReadDir(Path, Mask: string; ShowPath: boolean): TStringlist;
var SRec: TSearchRec;
begin
   //Path := IncludeTrailingBackslash(Path);
   Path := IncludeTrailingPathDelimiter(Path);
   Result := TStringList.Create;
   try
      if FindFirst(Path + Mask, not faDirectory, SRec) <> 0 then
         Exit;
      repeat
         case ShowPath of
            True:
               result.Add(Path + SRec.Name);
            False:
               result.Add(SRec.Name);
         end;
      until FindNext(SRec) <> 0;
   finally
      FindClose(SRec);
   end;
end;

//-----------------------------------------------------------------------------
// m.frey:
// Wandelt String um fr Umbrche. Die ist so kurz (wie auch die function "_")
// damit es lesbarer wird
function S(const S: string):string;
begin
   Result := StringReplace(S     , '\n', #13, [rfReplaceAll]);
   Result := StringReplace(Result, '<nl>', #13, [rfReplaceAll]);
end;

//-----------------------------------------------------------------------------
procedure BfShowMessage(const Msg: string);
begin
   //ShowMessage( S(Msg));
   BfMessageBox(Msg, 'bitfarm-Archiv V3',0);
   GgfLogEvnt(S(Msg));
end;

//-----------------------------------------------------------------------------
{function BfMessageDlg(Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint): Integer;
begin
   Msg := S(Msg);
   Result := MessageDlgPosHelp(Msg, DlgType, Buttons, HelpCtx, -1, -1, '');
end;
 }
// Variante mit bersetzung, siehe Viewer.Tdm_MainForm2.Frage
function BfMessageDlg(Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; Default_NO: boolean = False): Integer;
  var syes, sno, scancel, swarning, sconfirm, sinformation : string;
    Dialog: TForm;
begin
   Msg := S(Msg);

   syes := _('Ja');
   sno := _('Nein');
   scancel := _('Abbrechen');
   swarning := _('Warnung');
   sconfirm := _('Besttigung');
   sinformation := _('Informationen');
   //with CreateMessageDialog(Msg, DlgType, [mbYes, mbNo]) do
   Dialog := CreateMessageDialog(Msg, DlgType, Buttons);
   with Dialog do
   //with CreateMessageDialog(Msg, DlgType, Buttons) do
      try
         if Default_NO then
            ActiveControl := FindComponent('NO') as TWinControl;
         (FindComponent('NO') as TWinControl).SetTextBuf(PChar(sno));
         (FindComponent('YES') as TWinControl).SetTextBuf(PChar(syes));
         (FindComponent('CANCEL') as TWinControl).SetTextBuf(PChar(scancel));
         if DlgType = mtConfirmation then
	         Caption := sconfirm;
         if DlgType = mtWarning then
	         Caption := swarning;
         if DlgType = mtInformation then
         	Caption := sinformation;
         Dialog.Position := poMainFormCenter;   
         Result := Dialog.ShowModal;
      finally
         Close;
         Free;
      end

   //Result := MessageDlgPosHelp(Msg, DlgType, Buttons, HelpCtx, -1, -1, '');
end;




function BfMessageBox(Msg, Caption: string; Flags: Word; Handle: THandle = 0): Integer;
begin
   Msg := S(Msg);
   Caption := S(Caption);
   //Result := Application.MessageBox(PAnsiChar(Msg), PAnsiChar(Caption), Flags);
   Result := Windows.MessageBox(Handle,PAnsiChar(Msg), PAnsiChar(Caption), Flags or MB_SYSTEMMODAL or MB_SETFOREGROUND or MB_TOPMOST)
end;

function BfInputBox(Msg, Caption: string; default: string = ''): string;
begin
   Result := '';
	Msg := S(Msg);
   Caption := S(Caption);
   Application.CreateForm(Tfrm_InputBox, frm_InputBox);
   frm_InputBox.Caption := Caption;
   frm_InputBox.lbl1.Caption := Msg;
   frm_InputBox.edt1.Text := default;
   frm_InputBox.ShowModal;

   // wenn wir Abbrechen geben wir den defaultstring (Vorbelegung) zurck,
   // was gleichbedeutend mit keine nderung ist.
   if frm_InputBox.ModalResult = mrOK then
   	Result := frm_InputBox.edt1.Text
   else
   	Result:= default;
   frm_InputBox.Release;


end;

//-----------------------------------------------------------------------------
procedure ErrorMsg(ErrMod, ErrMsg: string; ErrID: integer; LogErr: boolean);
//////////////////////////////////////////////////////////////////
///  10.04.2005 Erstellt ErrorMsg by SV.                       ///
///                                                            ///
///  Diese Procedure blendet ein Meldungsfenster mit der       ///
///  Fehlermeldung ErrMsg von Modul ErrMod und Fehler ID       ///
///  ErrID ein. Optional kann dieser Event auch mit LogErr     ///
///  in einer Datei geloggt werden (LogEvnt)                   ///
///                                                            ///
//////////////////////////////////////////////////////////////////
begin
   BfMessageDlg(_('Fehler') + ' in Modul:' + ErrMod + #13 + 'Fehlercode:' +
      IntToStr(ErrID) + #13 + #13 + ErrMsg, mtError, [mbOK], 0);
   if LogErr then
      LogEvnt(IntToStr(ErrID) + '    ' + _('Fehler') + ' in Modul:' + ErrMod +
         '    ' + ErrMsg);
end;

//-----------------------------------------------------------------------------
procedure GnugettextTextdomain(Domain, Path, Language: string);
//var lsBind: string;
begin
   if Path='' then
      Path := IncludeTrailingPathDelimiter(extractfilepath(ExecutableFilename))
              + 'Viewer-Files';
   Path := IncludeTrailingPathDelimiter(Path)+ 'locale';

   {$ifdef DXGETTEXTDEBUG}
   DefaultInstance.DebugWriteln ('procedure GnugettextTextdomain Path='+Path);
   DefaultInstance.DebugWriteln ('procedure GnugettextTextdomain Language='+Language);
   {$endif}

   bindtextdomain(Domain, Path);
   bindtextdomain('Delphi7', Path);
   bindtextdomain('Shared', Path);

   //AddDomainForResourceString('Shared');


   if Language <> '' then
      UseLanguage(Language);

   textdomain(Domain);
   BD.GtDomain := Domain;
end;

function LangKuerzel:string;
var ls: string;
begin
   Result:='';
   ls := GetCurrentLanguage;
   if Copy(ls,0,2) ='en'
   then Result:='_en';
end;

//-----------------------------------------------------------------------------
// die gibt es schon in TGnuGettextInstance, ist dort aber protected
procedure TranslateStrings(sl: TStrings;const TextDomain:string);
var
  line: string;
  i: integer;
  s:TStringList;
begin
  if sl.Count > 0 then begin
    sl.BeginUpdate;
    try
      s:=TStringList.Create;
      try
        s.Assign (sl);
        for i:=0 to s.Count-1 do begin
          line:=s.Strings[i];
          if line<>'' then
            s.Strings[i]:=dgettext(TextDomain,line);
        end;
        sl.Assign(s);
      finally
        FreeAndNil (s);
      end;
    finally
      sl.EndUpdate;
    end;
  end;
end;

//-----------------------------------------------------------------------------
type ERedirectorError = Exception;

function RunWaitAndCaptureOutput(CommandLine:string; var Output:string; LogLevel:integer):DWord;
const BufSize=1024;
var
  buf:array[0..BufSize-1] of char;

  si:STARTUPINFO;
  sa:SECURITY_ATTRIBUTES;
  sd:SECURITY_DESCRIPTOR;               //security information for pipes
  pi:PROCESS_INFORMATION;

  // newstdin,
  newstdout,
  read_stdout:THandle;
  // write_stdin:THandle;  //pipe handles

  bytes_read:cardinal;
  bytes_available:cardinal;

  procedure ZeroBuffer;
  begin
    FillChar(Buf,SizeOf(Buf),0);
  end;

  procedure RaiseError(str:string);
  var n:DWord;
  begin
    n:=GetLastError;
    raise ERedirectorError.CreateFmt('%s: %d/0x%x - %s',[Str,n,n,SysErrorMessage(n)]);
  end;

  procedure GetData;
  begin 
     PeekNamedPipe(read_stdout,@buf,BufSize-1,@bytes_read,@bytes_available,NIL);

    //check to see if there is any data to read from stdout 
    if (bytes_read<>0) then 
      begin 
        ZeroBuffer; 
        if (bytes_available>BufSize-1) 
          then 
            while (bytes_read>=BufSize-1) do 
              begin 
                ReadFile(read_stdout,buf,BufSize-1,bytes_read,NIL);  //read the stdout pipe 
                Output:=Output+Buf; 
                ZeroBuffer; 
              end 
          else 
            begin 
              ReadFile(read_stdout,buf,BufSize-1,bytes_read,NIL); 
              Output:=Output+Buf; 
            end; 
     end; 
  end;
  procedure WriteOutput;
  begin
     write(Output); Output:='';
  end;
begin 
  Output:=''; 
  Result:=255; // indicate some error 

  //if IsNT
  if true
    then       //initialize security descriptor (Windows NT) 
      begin
        InitializeSecurityDescriptor(@sd,SECURITY_DESCRIPTOR_REVISION);
        SetSecurityDescriptorDacl(@sd,true,NIL,False); 
        sa.lpSecurityDescriptor:=@sd; 
      end 
    else sa.lpSecurityDescriptor:=NIL; 


  sa.nLength:=sizeof(SECURITY_ATTRIBUTES); 
  sa.bInheritHandle:=true;         //allow inheritable handles 
  (*
  if not(CreatePipe(newstdin,write_stdin,@sa,0))   //create stdin pipe
    then RaiseError('CreatePipe');
  *)
  if not(CreatePipe(read_stdout,newstdout,@sa,0))  //create stdout pipe
    then
      begin
        // CloseHandle(newstdin);
        // CloseHandle(write_stdin);
        RaiseError('CreatePipe');
      end;

  GetStartupInfoA(si);      //set startupinfo for the spawned process
  {
  The dwFlags member tells CreateProcess how to make the process. 
  STARTF_USESTDHANDLES validates the hStd* members. STARTF_USESHOWWINDOW 
  validates the wShowWindow member. 
  } 

  si.dwFlags:=STARTF_USESTDHANDLES or STARTF_USESHOWWINDOW; 
  si.wShowWindow:=SW_HIDE; 
  si.hStdOutput:=newstdout; 
  si.hStdError:=newstdout;     //set the new handles for the child process 
  // si.hStdInput:=newstdin; 

  //spawn the child process 
  if not 
       (CreateProcessA(NIL,PChar(CommandLine),NIL,NIL,TRUE,CREATE_NEW_CONSOLE,
                        NIL,NIL,si,pi))
    then 
      begin 
        // CloseHandle(newstdin); 
        CloseHandle(newstdout); 
        CloseHandle(read_stdout); 
        // CloseHandle(write_stdin); 
        RaiseError('CreateProcess');
      end; 
  ZeroBuffer;
  while True do
    begin
      GetExitCodeProcess(pi.hProcess,Result);      //while the process is running
      if (Result<>STILL_ACTIVE) then break;
      GetData;
      //Yield;  //frey
      sleep(200); //frey
      if LogLevel>0
      then WriteOutput;
    end;
  GetData;

  CloseHandle(pi.hThread);
  CloseHandle(pi.hProcess);
  // CloseHandle(newstdin);            //clean stuff up
  CloseHandle(newstdout);
  CloseHandle(read_stdout); 
  // CloseHandle(write_stdin); 
end;

//-----------------------------------------------------------------------------
function StrToInt0(const S: string): Integer;
begin
   try
      Result := StrToInt(S);
   except
      on e: EConvertError do
      begin
         Result := 0;
      end;
   end;
end;

//-----------------------------------------------------------------------------
//m.frey: vorlufig billig und lahm
function ConvertStringToFileName(S:string):string;
var i:integer;
begin
   Result := ParseFilename(S)
end;

//-----------------------------------------------------------------------------
function ElimRight(Name, Text:string):string;
var liName, liText: integer; 
begin
   liName := length(Name);
   liText := length(Text);
   if copy(Name, 1 + liName - liText, maxint) = Text then
      Result := copy(Name, 1, liName - liText)
   else
      Result := Name;
end;

//-----------------------------------------------------------------------------
function ConcatDelim(S1, Delim, S2:string):string;
begin
   Result := S1;
   if (S1 > '') and (S2 > '') then
      Result := S1 + Delim + S2
   else
      if S2 > '' then
         Result := S2;
end;

//-----------------------------------------------------------------------------
//3.5.0.38 [2364]
function ExistentFilesInDir(fsPath: string): integer;
var
   sr: TSearchRec;
   RowCount: integer;
begin
   RowCount := 0;
   Result := 0;
   if FindFirst(fsPath, faAnyFile, sr) = 0 then
   begin
      repeat
         RowCount := RowCount + 1;
      until FindNext(sr) <> 0;

      FindClose(sr);
      Result := RowCount;
   end;
end;
//-----------------------------------------------------------------------------
//3.5.0.38 [2364]
function CalcNonExistingFilename(Path, Name, Ext: string): string;
var linfiles: integer;   snamFile1:string; 
begin
   Result := Name;
   linfiles := ExistentFilesInDir(Path + '\' + Name + Ext);
   if linfiles <> 0 then
   begin
      snamFile1 := Name + '_' + IntToStr(linfiles);
      while FileExists(Path + '\' + snamFile1 + Ext) do
      begin
         Inc(linfiles);
         snamFile1 := Name + '_' + IntToStr(linfiles);
      end;
      Result := snamFile1;
   end;
end;

//-----------------------------------------------------------------------------
//3.5.0.67 [2445]/[2068]
// entfernt berflssige Anfhrungszeichen, d.h. alle auer wenn vorne und
// hinten eines ist
// auerdem wird gleich noch getrimmt
function EliminierenVonAnfuehrungszeichen(S:string):string;
var f:Boolean;
begin
   Result := trim(S);
   if S='"' then
      Result:=''
   else
      if length(S)>=2 then
      begin
         // vorne und hinten Anfhrungszeichen?
         f := (S[1] = #34) and (S[length(S)] = #34);

         // Alle weg
         Result := StringReplace(Result, '"', '', [rfReplaceall]);
         Result := trim(Result);

         // ggf. wieder vorne und hinten Anfhrungszeichen ran
         if f then
            Result := #34 + Result + #34;
      end;
end;

//-----------------------------------------------------------------------------
{3.6<}
procedure StringInDatei(const Dateiname:Ansistring; const Data:Ansistring);
begin
	if Data <> '' then    //18.07.2011 b.takacs 3.5.1.30 [3117]
	   SpeicherInDatei(Dateiname,Data[1],length(Data))
  else
	   SpeicherInDatei(Dateiname,Data,length(Data)); //18.07.2011 b.takacs 3.5.1.30 [3117]
end;
{>3.6}

//-----------------------------------------------------------------------------
procedure SpeicherInDatei(const Dateiname:Ansistring; const Buffer; Count: Longint);
var FS:TFileStream; // w:integer; f:file;
begin
   FS:=TFileStream.Create(Dateiname, fmCreate);
   try
      FS.Write(Buffer, Count);
   finally
      FS.Free;
   end;
end;

//-----------------------------------------------------------------------------
// true, wenn String nur aus Ziffern, Komma und/oder Punkt besteht
function IstZahl(S:string):Boolean; //[2487]
var i:integer;
begin
   Result := False;
   if S='' then
      Exit;
   for i := 1 to length(S) do
      if not (S[i] in ['0'..'9',',','.'])
      then Exit;
   Result := True;
end;

//-----------------------------------------------------------------------------
//returns true if a given directory is empty, false otherwise [3069]
function IsDirectoryEmpty(const directory : string) : boolean;
var
   searchRec :TSearchRec;
begin
  try
    result := (FindFirst(directory + '\*.*', faAnyFile, SearchRec) = 0) AND
              (FindNext(searchRec) = 0) AND
              (FindNext(searchRec) <> 0) ;
  finally
     FindClose(searchRec) ;
  end;
end;


function EmptyDirectory(const ADir: string): boolean;
var
   searchRec :TSearchRec;
begin
   Result := True;
   try
      if (FindFirst(ADir + '\*.*', faAnyfile - faDirectory, SearchRec) = 0) then
      begin
         repeat
            DeleteFile(ADir + '\' + SearchRec.Name);
         until (FindNext(SearchRec) <> 0);
         FindClose(SearchRec);
      end;
   except
       Result := False;
   end;
end;

function IsUserAdmin : boolean;
const CAdminSia : TSidIdentifierAuthority = (value: (0, 0, 0, 0, 0, 5));
var sid : PSid;
    ctm : function (token: dword; sid: pointer; var isMember: bool) : bool; stdcall;
    b1  : bool;
begin
  result := false;
  ctm := GetProcAddress(LoadLibrary('advapi32.dll'), 'CheckTokenMembership');
  if (@ctm <> nil) and AllocateAndInitializeSid(CAdminSia, 2, $20, $220, 0, 0, 0, 0, 0, 0, sid) then begin
    result := ctm(0, sid, b1) and b1;
    FreeSid(sid);
  end;
end;

procedure FindAllFiles(var FileList: TStrings; RootFolder: string;
   Mask: string = '*.*'; Recurse: boolean = True);
var
   SR: TSearchRec;
begin
   IncludeTrailingPathDelimiter(RootFolder);

   if Recurse then
      if FindFirst(RootFolder + '*.*', faAnyFile, SR) = 0 then
         try
            repeat
               if SR.Attr and faDirectory = faDirectory then
                  // --> ein Verzeichnis wurde gefunden
                  //   der Verzeichnisname steht in SR.Name
                  //   der vollstndige Verzeichnisname (inkl. darberliegender Pfade) ist
                  //       RootFolder + SR.Name
                  if (SR.Name <> '.') and (SR.Name <> '..') then
                     FindAllFiles(FileList, RootFolder + SR.Name, Mask, Recurse);
            until FindNext(SR) <> 0;
         finally
            FindClose(SR);
         end;
   if FindFirst(RootFolder + Mask, faAnyFile, SR) = 0 then
      try
         repeat
            if SR.Attr and faDirectory <> faDirectory then
            begin
               // --> eine Datei wurde gefunden
               //   der Dateiname steht in SR.Name
               //   der vollstndige Dateiname (inkl. Pfadangabe) ist
               //       RootFolder + SR.Name
               // sonst findet *.con auch .conf
               if lowercase(ExtractFileExt(SR.Name)) = lowercase(ExtractFileExt(mask)) then
	               FileList.Add(RootFolder + SR.Name);
            end;
         until FindNext(SR) <> 0;
      finally
         FindClose(SR);
      end;
end;

procedure KillFile(FileName: string);
var
   f: file;
begin
   GgfLogEvnt('KillFile:' + FileName);

   if FileExists(FileName) then
   begin
      assignfile(f, FileName);
{$I-}
      Erase(f);
{$I+}
   end
   else
   begin
      exit;
   end;
end;


procedure ClearRights(var RightData : TRData);
begin
		RightData.list := False;
      RightData.print := False;
      RightData.Read := False;
      RightData.link := False;
      RightData.modify := False;
      RightData.import := False;
      RightData.Write := False;
      RightData.history := False;
      RightData.mail := False;
      RightData.move := False;
      RightData.Aread := False;
      RightData.Awrite := False;
      RightData.AModify := False;
      RightData.ApRead := False;
      RightData.ApWrite := False;
      RightData.hstprotoread := False;
      RightData.suanno := False;
      RightData.docdel := False;
      RightData.docexport := False;
      RightData.revsave := False;

end;



//-----------------------------------------------------------------------------
initialization
   InitializeCriticalSection(csLog); //09-09-07 m.frey
   BD := TBfData.Create;

finalization
   FreeAndNil(BD);
   DeleteCriticalSection(csLog); //09-09-07 m.frey

end.



