unit u_ShSql;

(*
  ------------------------------------------------------------------------------
  Diese Unit kapselt den Zugriff auf SQL-Tabellen
  ------------------------------------------------------------------------------
*)


(*
  ==============================================================================
*)
interface
(*
  ==============================================================================
*)

uses
  StdCtrls, u_ShDefs, u_ShRecTab,
  MyAccess;



type
  (*
    Datensatz der SQL-Tabelle 'sys_gdoc_id'
  *)
  clSysGDocIDRec = class
  private
    mGDocID: Int64;
    mArcID : Int64;
    mDocID : Int64;

  public
    property GDocID: Int64 read mGDocID write mGDocID;
    property ArcID : Int64 read mArcID  write mArcID;
    property DocID : Int64 read mDocID  write mDocID;

    function  getSelectFieldNames: String;                virtual;
    function  getInsertFieldNames: String;                virtual;
  end;

  (*
    Funktionen fr die SQL-Tabelle 'sys_gdoc_id'
  *)
  sysGDocIDTab = class
  private
  public
    (*  Zweck:
          Legt in der SQL-Tabelle eine neue GDocID an
        Eingabe:
          'ArcID': Die ID des Archivs
          'DocID': Die ID des Dokuments
        Rckgabe:
          Den entstandenen Datensatz mit der neuen GDocID.
          'nil' bei einem Fehlschlag
    *)
    class function newGDocID (ArcID, DocID: Int64): clSysGDocIDRec;


    (*  Zweck:
          Lscht die GDocID aus der SQL-Tabelle
        Eingabe:
          'GDocID': Die GDocID des Dokuments
    *)
    class procedure deleteGDocID (GDocID: Int64);


    (*  Zweck:
          Holt die 'GDocID' aus der SQL-Tabelle
        Eingabe:
          'ArcID': Die ID des Archivs
          'DocID': Die ID des Dokuments
        Rckgabe:
          Den ausgefllten Datensatz.
          'nil' falls die GDocID nicht existiert.
    *)
    class function getGDocID (ArcID, DocID: Int64): clSysGDocIDRec;


    (*  Zweck:
          Holt die 'ArcID' und die 'DocID' aus der SQL-Tabelle
        Eingabe:
          'GDocID': Die GDocID des Dokuments
        Rckgabe:
          Den ausgefllten Datensatz.
          'nil' falls die 'ArcID' oder die 'DocID' nicht existiert.
    *)
    class function getArcID (GDocID: Int64): clSysGDocIDRec;

  end;

  (*
    Datensatz einer Konfigurations-Tabelle
    cfg_...
  *)
  clCfgRec = class
    private
      mCfg_ID   : Int64;
      mFeld_Name: string;  //  z.B. 'add_14042008115949'
      mDisp_Name: string;  //  z.B. 'Kunde'
      mTyp      : Integer;
      mWerte    : string;
      mMode     : Integer;
      mOptionen : Int64;
      mPos      : Integer;
      mChck_Prog: string;

      function  getTyp: eAddFieldType;
      procedure mapIntoRec (const Qry: TMyQuery);

      class function getSelectFieldNames: string;

    public
      property CfgID   : Int64         read mCfg_ID    write mCfg_ID;
      property FeldName: string        read mFeld_Name write mFeld_Name;
      property DispName: string        read mDisp_Name write mDisp_Name;
      property Typ     : eAddFieldType read getTyp;
      property Werte   : string        read mWerte     write mWerte;
      property Mode    : Integer       read mMode      write mMode;
      property Optionen: Int64         read mOptionen  write mOptionen;
      property Pos     : Integer       read mPos       write mPos;
      property ChckProg: string        read mChck_Prog write mChck_Prog;

      constructor Create;
      destructor  Destroy;  override;
    end;

  (*
    Konfigurations-Tabelle eines Archivs
    cfg_...
  *)
  clCfgTab = class (clRecTab)
    private
      mArcName  : string;  //  z.B. '10012008120238'
      mFieldName: string;  //  z.B. 'add_14042008115949'

      function makeSql: string;

    public
      constructor Create (const arcName: string);
      destructor  Destroy; override;

      function  getRec                 : clCfgRec; overload;
      function  getRec (RecNo: integer): clCfgRec; overload;

      procedure load;

      (*
        Zweck:
          Sorgt dafr, da der nchste Aufruf von 'load' nur den einen
          Datensatz ermittelt, der das Feld 'name' beschreibt.
        Eingabe:
          'name': Inhalt des SQL-Feldes 'Feld_Name' der cfg_-Tabelle
                  z.B. 'add_14042008115949'
                  Info zu diesem Feld wird gefunden.
      *)
      procedure setFeldName (name: string);
    end;

  (*
    Verkrzter Datensatz einer Archiv-Tabelle
    arc_...
    Dieser Datensatz enthlt nur den Inhalt eines einzigen Zusatfeldes.
  *)
  clArcAddRec = class
    private
      mContent: string;

    public
      property Content: string read mContent;

    end;

  (*
    Archiv-Tabelle mit verkrzten Datenstzen
    arc_...
    Diese Datenstze enthalten nur den Inhalt eines einzigen Zusatfeldes.
  *)
  clArcAddTab = class (clRecTab)
    private
      mArcName : string;
      mFldName : string;
      mLikeCont: string;
      mDocID   : integer;

      function  makeSqlSelect                : string;
      function  makeSqlUpdate (Value: string): string;
      procedure load;

    public
      (*
        Erzeugt (wie gewohnt) das Objekt,
        ldt aber zustzlich alle Feldinhalte.
        d.h.: Sobald das Objekt da ist, kann auf die Feld-Inhalte
        zugegriffen werden.
      *)
      constructor Create (ArcName : string;
                          FldName : string);           overload;
      constructor Create (ArcName : string;
                          FldName : string;
                          LikeCont: string);           overload;  //  gesuchter Inhalt mit like %
      constructor Create (ArcName : string;
                          FldName : string;
                          DocID   : integer);          overload;
      destructor  Destroy;  override;

      function  getRec                 : clArcAddRec;  overload;
      function  getRec (RecNo: integer): clArcAddRec;  overload;

      (*
        Zweck:
          Ergnzt die Auswahlliste einer ComboBox um die Inhalte dieser
          Tabelle.
        Hinweis:
          Nur noch nicht vorhandene Inhalte werden hinzugefgt.
      *)
      procedure   fillComboBox (Box: TComboBox);

      (*
        Zweck:
          Der neue Wert des SQL-Feldes 'FldName' (siehe 'Create')
          wird in der Datenbank gesichert
        Eingabe:
          'Value': Ist der zu speichernde Wert
      *)
      procedure updateValue (Value: string);

      (*
        Zweck:
          Ruft 'updateValue' auf, und hat somit den gleichen Zweck.
        Eingabe:
          'Value': Ist der zu speichernde Wert im Datums-Format 'dd.mm.yyyy'
                   Wird konvertiert ins Format 'yyyy-mm-dd'
      *)
      procedure updateDate (Value: string);
    end;


  (*
    Datensatz einer Archiv-Tabelle
    arc_...
    fr jedes Archiv eine Tabelle
    fr jedes Dokument eine Zeile
  *)
  clArcRec = class
    private
      mDoc_ID   : Int64;
      mDisp_Name: string;  //  Dokument-Name
      mDoc_Loc  : string;  //  Datei-Pfad. TIFF von revisionssicheren Dokumenten
      mExt_Loc  : string;  //  Datei-Pfad. Bei nderungen oder Notizen
      mOri_Loc  : string;  //  Datei-Pfad. Original-Dokumente
      mUsr_Name : string;
      mUsr_ID   : int64;
      mTStamp   : TDateTime;
      mDoc_Txt  : string;
      mDoc_State: integer;
      mOcr_QF   : int64;
      mGDoc_ID  : int64;
      mOptionen : integer;
      mArc_ID   : integer;

      class function getSelectFieldNames: string;

    public
      property DocID   : Int64         read mDoc_ID    write mDoc_ID;
      property DispName: string        read mDisp_Name write mDisp_Name;
      property DocLoc  : string        read mDoc_Loc   write mDoc_Loc;
      property ExtLoc  : string        read mExt_Loc   write mExt_Loc;
      property OriLoc  : string        read mOri_Loc   write mOri_Loc;
      property UsrName : string        read mUsr_Name  write mUsr_Name;
      property UsrID   : int64         read mUsr_ID    write mUsr_ID;
      property TStamp  : TDateTime     read mTStamp    write mTStamp;
      property DocTxt  : string        read mDoc_Txt   write mDoc_Txt;
      property DocState: integer       read mDoc_State write mDoc_State;
      property OcrQF   : int64         read mOcr_QF    write mOcr_QF;
      property GDocID  : int64         read mGDoc_ID   write mGDoc_ID;
      property Optionen: integer       read mOptionen  write mOptionen;
      property Arc_ID  : integer			read mArc_ID    write mArc_ID;

      constructor Create;
      destructor  Destroy;  override;

      procedure   mapIntoRec (const Qry: TMyQuery; arcid : integer);
    end;

  (*
    Archiv-Tabelle
    arc_...
    fr jedes Archiv eine Tabelle
    fr jedes Dokument eine Zeile
  *)
  clArcTab = class (clRecTab)
    private
      mArcID   : int64;
      mArcName : string;

      function makeSql: string;

    public
      property ArcID  : int64  read mArcID;
      property ArcName: string read mArcName;

      constructor Create; overload;
      constructor Create (      arcID  : integer;
                          const arcName: string );  overload;


      destructor  Destroy; override;

      function  getRec                 : clArcRec; overload;
      function  getRec (RecNo: integer): clArcRec; overload;

      procedure load;

      procedure add (const Rec: clArcRec);
    end;


(*
  ==============================================================================
*)
implementation
(*
  ==============================================================================
*)

uses
  //  Delphi
  //
  Classes, SysUtils,

  // Shared
  //
  u_ShDbUtils, u_ShLowUtils;

var
  gCmd  : TMyCommand;
  gQuery: TMyQuery;


procedure prepareCommand;
  begin
    if gCmd = nil then
      gCmd := DbCreateCommand;
  end;


procedure prepareQuery;
  begin
    if gQuery = nil then
      gQuery := DbCreateQuery;
  end;

(*
  ------------------------------------------------------------------------------
  clSysGDocIDRec
  ------------------------------------------------------------------------------
*)

function clSysGDocIDRec.getSelectFieldNames: String;
  begin
    Result := 'gdoc_id, arc_id, doc_id';
  end;


function clSysGDocIDRec.getInsertFieldNames: String;
  begin
    Result := 'arc_id, doc_id';
  end;


class function sysGDocIDTab.newGDocID (ArcID, DocID: Int64): clSysGDocIDRec;

  var
    xSQL: string;

  begin
    Result       := clSysGDocIDRec.Create;
    Result.ArcID := ArcID;
    Result.DocID := DocID;

    prepareCommand;

     xSQL := 'insert sys_gdocid (' + Result.getInsertFieldNames + ')'
          + ' values ('
          + IntToStr (ArcID) + ', '
          + IntToStr (DocID) + ')';

    gCmd.SQL.Text := xSQL;

    try
      gCmd.Execute;

      Result.GDocID := gCmd.InsertID;

    except
      on e: Exception do
      begin
        Result.Free;
        Result := nil;
      end;
    end;
  end;  //  sysGDocIDTab.newGDocID


class procedure sysGDocIDTab.deleteGDocID (GDocID: Int64);

  var
    xSQL: string;

  begin
    prepareCommand;
    xSQL := 'delete from sys_gdocid '
          + 'where gdoc_id = ' + IntToStr (GDocID);

    gCmd.SQL.Text := xSQL;

    try
      gCmd.Execute;

    except
      on e: Exception do
      begin
      end;
    end;
  end;  //  sysGDocIDTab.deleteGDocID


class function sysGDocIDTab.getGDocID (ArcID, DocID: Int64): clSysGDocIDRec;

  var
    xSQL: string;
    xErr: Boolean;

  begin
    Result       := clSysGDocIDRec.Create;
    Result.ArcID := ArcID;
    Result.DocID := DocID;

    xErr := True;

    xSQL := 'Select Sql_No_cache ' + Result.getSelectFieldNames
           +' from sys_gdocid'
           +' where arc_id = '     + IntToStr (ArcID)
           +' and doc_id = '       + IntToStr (DocID);

    prepareQuery;

    if DbDoSkySQL (gQuery, xSQL) then
    begin
      if gQuery.RecordCount = 1 then
      begin
        xErr := False;

        Result.GDocID := gQuery.FieldByName ('gdoc_id').AsInteger;
      end
    end;

    gQuery.Close;

    if xErr then
    begin
      Result.Free;
      Result := nil;
    end;
  end;  //  sysGDocIDTab.getGDocID


class function sysGDocIDTab.getArcID (GDocID: Int64): clSysGDocIDRec;

  var
    xSQL: string;
    xErr: Boolean;

  begin
    Result        := clSysGDocIDRec.Create;
    Result.GDocID := GDocID;

    xErr := True;
    xSQL := 'Select Sql_No_cache ' + Result.getSelectFieldNames + ' sys_gdocid'
           +' where gdoc_id = '     + IntToStr (GDocID);


    prepareQuery;

    if DbDoSkySQL (gQuery, xSQL) then
    begin
      if gQuery.RecordCount = 1 then
      begin
        xErr := False;

        Result.ArcID := gQuery.FieldByName ('arc_id').AsInteger;
        Result.DocID := gQuery.FieldByName ('doc_id').AsInteger;
      end
    end;

    gQuery.Close;

    if xErr then
    begin
      Result.Free;
      Result := nil;
    end;

    gQuery.Close;
  end;  //  sysGDocIDTab.getArcID

(*
  ------------------------------------------------------------------------------
  clArcAddTab
  ------------------------------------------------------------------------------
*)

constructor clArcAddTab.Create (ArcName: string;
                                FldName: string);
  begin
    inherited Create;

    mArcName  := ArcName;
    mFldName  := FldName;
    mLikeCont := '';
    mDocID    := -1;

    load;
  end;


constructor clArcAddTab.Create (ArcName : string;
                                FldName : string;
                                LikeCont: string);
  begin
    inherited Create;

    mArcName  := ArcName;
    mFldName  := FldName;
    mLikeCont := LikeCont;
    mDocID    := -1;

    load;
  end;

constructor clArcAddTab.Create (ArcName : string;
                                FldName : string;
                                DocID   : integer);
  begin
    inherited Create;

    mArcName  := ArcName;
    mFldName  := FldName;
    mLikeCont := '';
    mDocID    := DocID;

    load;
  end;


destructor clArcAddTab.Destroy;
  begin
    inherited Destroy;
  end;


function clArcAddTab.getRec: clArcAddRec;
  begin
    Result := clArcAddRec (inherited getRec ());
  end;


function clArcAddTab.getRec (RecNo: integer): clArcAddRec;
  begin
    Result := clArcAddRec (inherited getRec (RecNo));
  end;


function clArcAddTab.makeSqlSelect: string;
  begin
    Result := 'Select Sql_No_cache distinct ' + mFldName + ' from arc_' + mArcName;

    if mLikeCont > '' then
    begin
      Result := Result + ' where ' + mFldName + ' like ''%' + mLikeCont + '%''';
    end
    else
    if mDocID >= 0 then
    begin
      Result := Result + ' where DOC_ID = ' + IntToStr (mDocID);
    end;

    Result := Result
            + ' order by '
            + mFldName;
  end;


function clArcAddTab.makeSqlUpdate (Value: string): string;
  begin
    Result := 'update  arc_' + mArcName + ' set '
            + mFldName
            + ' = '
            + Value
            + ' where DOC_ID = '
            + IntToStr (mDocID);
  end;


procedure clArcAddTab.load;

  var
    xRec : clArcAddRec;
    xSQL : string;
    xSize: Integer;
    xIdx : Integer;

  begin
    xSQL := makeSqlSelect;

    prepareQuery;
    clear;

    if DbDoSkySQL (gQuery, xSQL) then
    begin
      xSize := gQuery.RecordCount;
      for xIdx := 0 to xSize-1 do
      begin
        xRec          := clArcAddRec.Create;
        xRec.mContent := gQuery.FieldByName (mFldName).asString;

        add (xRec);

        gQuery.Next;
      end;
    end;
    gQuery.Close;
  end;  //  clArcAddTab.load


procedure clArcAddTab.updateValue (Value: string);

  var
    xSQL: string;

  begin
    xSQL := makeSqlUpdate (Value);

    prepareCommand;

    gCmd.SQL.Text := xSQL;

    try
      gCmd.Execute;
    except
      on e: Exception do
      begin
      end;
    end;
  end;


procedure clArcAddTab.updateDate (Value: string);

  var
    xDate: TDateTime;

  begin
    xDate := StrToDate (Value);

    updateValue (FormatDateTime ('yyyy-mm-dd', xDate));
  end;


procedure clArcAddTab.fillComboBox (Box: TComboBox);

  var
    xRec   : clArcAddRec;
    xIdx   : Integer;
    xIdxMax: Integer;
    xLstIdx: Integer;

  begin
    xIdxMax := Self.Count -1;

    for xIdx := 0 to xIdxMax do
    begin
      xRec    := getRec (xIdx);

      xLstIdx := indexOf (xRec.Content, Box.Items);

      if xLstIdx < 0 then
        Box.Items.Add (xRec.Content);
    end;
  end;

(*
  ------------------------------------------------------------------------------
  clCfgRec
  ------------------------------------------------------------------------------
*)

constructor clCfgRec.Create;
  begin
    inherited Create;
  end;


destructor clCfgRec.Destroy;
  begin
    inherited Destroy;
  end;


function clCfgRec.getTyp: eAddFieldType;
  begin
    Result := eAddFieldType (mTyp);
  end;


class function clCfgRec.getSelectFieldNames: string;
  begin
    Result := 'Cfg_ID, Feld_Name, Disp_Name, Typ, Werte, Mode, Optionen, '
            + 'Pos, Chck_Prog';
  end;


procedure clCfgRec.mapIntoRec (const Qry: TMyQuery);
  begin
    mCfg_ID    := Qry.FieldByName ('Cfg_ID'   ).asInteger;
    mFeld_Name := Qry.FieldByName ('Feld_Name').asString;
    mDisp_Name := Qry.FieldByName ('Disp_Name').asString;
    mTyp       := Qry.FieldByName ('Typ'      ).asInteger;
    mWerte     := Qry.FieldByName ('Werte'    ).asString;
    mMode      := Qry.FieldByName ('Mode'     ).asInteger;
    mOptionen  := Qry.FieldByName ('Optionen' ).asInteger;
    mPos       := Qry.FieldByName ('Pos'      ).asInteger;
    mChck_Prog := Qry.FieldByName ('Chck_Prog').asString;
  end;

(*
  ------------------------------------------------------------------------------
  clCfgTab
  ------------------------------------------------------------------------------
*)

constructor clCfgTab.Create (const arcName: string);
  begin
    inherited Create;

    mArcName   := arcName;
    mFieldName := '';
  end;


destructor clCfgTab.Destroy;
  begin
    inherited Destroy;
  end;


function clCfgTab.getRec: clCfgRec;
  begin
    Result := clCfgRec (inherited getRec);
  end;


function clCfgTab.getRec (RecNo: integer): clCfgRec;
  begin
    Result := clCfgRec (inherited getRec (RecNo));
  end;


function clCfgTab.makeSql: string;
  begin
    Result := 'Select Sql_No_cache '
            + clCfgRec.getSelectFieldNames
            + ' from cfg_'
            + mArcName;

    if mFieldName > '' then
      Result := Result
              + ' where Feld_Name = '''
              + mFieldName
              + '''';
  end;


procedure clCfgTab.setFeldName (name: string);
  begin
    mFieldName := name;
  end;


procedure clCfgTab.load;

  var
    xRec : clCfgRec;
    xSQL : string;
    xSize: Integer;
    xIdx : Integer;

  begin
    xSQL := makeSql;

    prepareQuery;
    clear;

    if DbDoSkySQL (gQuery, xSQL) then
    begin
      xSize := gQuery.RecordCount;

      for xIdx := 0 to xSize-1 do
      begin
        xRec := clCfgRec.Create;

        xRec.mapIntoRec (gQuery);
        add             (xRec  );

        gQuery.Next;
      end;
    end;

    gQuery.Close;
  end;  //  clCfgTab.load

(*
  ------------------------------------------------------------------------------
  clArcRec
  ------------------------------------------------------------------------------
*)

constructor clArcRec.Create;
  begin
    inherited;
  end;


destructor  clArcRec.Destroy;
  begin
    inherited;
  end;


class function clArcRec.getSelectFieldNames: string;
  begin
    //  mu um die Zusatzfelder erweitert werden
    //
    Result := 'doc_id, disp_name, doc_loc, ext_loc, ori_loc, usr_name, '
             +'usr_id, TStamp, doc_txt, doc_state, ocr_qf, gdoc_id, Optionen';
  end;


procedure clArcRec.mapIntoRec (const Qry: TMyQuery; arcid: integer);
  begin
    mDoc_ID    := Qry.FieldByName ('Doc_ID'   ).asInteger;
    mDisp_Name := Qry.FieldByName ('Disp_Name').asString;
    mDoc_Loc   := Qry.FieldByName ('Doc_Loc'  ).asString;
    mExt_Loc   := Qry.FieldByName ('Ext_Loc'  ).asString;
    mOri_Loc   := Qry.FieldByName ('Ori_Loc'  ).asString;
    mUsr_Name  := Qry.FieldByName ('Usr_Name' ).asString;
    mUsr_ID    := Qry.FieldByName ('Usr_ID'   ).asInteger;
    mTStamp    := Qry.FieldByName ('TStamp'   ).AsDateTime;
    mDoc_Txt   := Qry.FieldByName ('Doc_Txt'  ).asString;
    mDoc_State := Qry.FieldByName ('Doc_State').asInteger;
    mOcr_QF    := Qry.FieldByName ('Ocr_QF'   ).asInteger;
    mGDoc_ID   := Qry.FieldByName ('GDoc_ID'  ).asInteger;
    mOptionen  := Qry.FieldByName ('Optionen' ).asInteger;
    mArc_ID    := arcid;
  end;

(*
  ------------------------------------------------------------------------------
  clArcTab
  ------------------------------------------------------------------------------
*)

constructor clArcTab.Create;
  begin
    inherited Create;

    mArcID   := -1;
    mArcName := '';
  end;


constructor clArcTab.Create (      arcID  : integer;
                             const arcName: string );
  begin
    inherited Create;

    mArcID   := arcID;
    mArcName := arcName;
  end;


destructor  clArcTab.Destroy;
  begin
    inherited;
  end;


function clArcTab.getRec: clArcRec;
  begin
    Result := clArcRec (inherited getRec);
  end;


function  clArcTab.getRec (RecNo: integer): clArcRec;
  begin
    Result := clArcRec (inherited getRec (RecNo));
  end;


function clArcTab.makeSql: string;
  begin
  end;


procedure clArcTab.load;
  begin
    raise Exception.Create ('"clArcTab.load;" ist noch nicht implementiert!');
  end;


procedure clArcTab.add (const Rec: clArcRec);
  begin
    inherited add (Rec);
  end;


initialization
  gCmd   := nil;
  gQuery := nil;

end.


