unit u_ShStatefields;

interface

uses  Classes, SysUtils, Graphics, TBX, TBXExtItems, MyAccess, u_ShDefs, u_ShLowUtils , u_ShBase, u_ShDbUtils;


procedure Register;



type bfaStatefields = class(TTBXToolbar)
   private
      fThreeStates : Boolean;
      fStatefieldTab : clStatefieldTab;
      fState : Integer;	// Status als Bitmaske
      fNotState : Integer;  // Not-Status als Bitmaske fr die Suche
      fSearchState: Integer;  // Suchstatus Bitmaske
      fOldState : Integer; // Status vor einer nderung
      fModified : Boolean;
      fIgnoreRightsAtImport : Boolean;
      fUserID : Integer;    // UserID kann Initial gesetzt werden, oder der Funktion ShowStatefields bergeben werden
      							 // ist nichts gesetzt, wird ohne Abfrage der Rechte alle Statusfelder angezeigt.
      fArcid : Integer;     // das aktuell geladene Archiv;
	public
      fclTeal, fclLime, fclRed : TColor;

      AfterStateClick: procedure(Statefield: clStateField) of object;

      constructor Create(AOwner: TComponent); override;
		procedure CreateStatefields(iArcid: Integer; iUsrid : Integer = -1);
      procedure OnStateClick(Sender: TObject);
      procedure ClearStatefields;
      procedure ClearState;

      procedure ShowState(iState : Integer);
      property State: integer read fState;
      property NotState : Integer read fNotState;
      property ThreeStates: boolean read fThreeStates write fThreeStates;
      property IgnoreRightsAtImport : Boolean read fIgnoreRightsAtImport write fIgnoreRightsAtImport;
      property UserID : Integer read fUserID write fUserID;
      property Modified : Boolean read fModified write fModified;
end;


implementation


//------------------------------------------------------------------------------
procedure Register;
begin
  RegisterComponents('bitfarm - Components', [bfaStatefields]);

end;



constructor bfaStatefields.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   fclTeal := clTeal;
   fclLime := clLime;
   fclRed := clRed;
   fUserID := -1;
   fArcid := -1;
end;


procedure bfaStatefields.CreateStatefields(iArcid: Integer; iUsrid : Integer = -1);
var
   my_q, my_q2: TMyQuery;
   tbl_name : string;
   testitem: TTBXColorItem;
   Statefield : clStateField;
   bHasRights : Boolean;
begin
   my_q := DbCreateQuery;
   my_q2 := DbCreateQuery;

   try
		if iArcid = fArcid then
         Exit
      else
         ClearStatefields;

      tbl_name := DbTableColumnGetString('select arc_tblname from sys_arc where arc_id = ' + inttostr(iArcid));
      if tbl_name = '' then
      begin
   		ClearStatefields;
	      Exit;
      end;

      if not DbTableExist('sts_' + tbl_name) then
      begin
   		ClearStatefields;
	      Exit;
      end;

      DbDoSkySQL(my_q, 'select * from sts_' + tbl_name + ' order by sts_id');
      if my_q.RecordCount = 0 then
      begin
   		ClearStatefields;
	      Exit;
      end;

      if fStatefieldTab = nil then
         fStatefieldTab := clStateFieldTab.create;
      Self.Visible := True;

      if iUsrID = -1 then
      	iUsrID := fUserID;

      if iUsrID <> -1 then
      	DbDoSkySQL(my_q2, 'select * from sys_stsxs where arc_id='+ IntToStr(iArcid) +' and usr_id = ' + inttostr(iUsrid));

    	bHasRights := True;

      while not my_q.Eof do
      begin
         Statefield := clStateField.Create;
         testitem := TTBXColorItem.Create(Self);
         Statefield.DispName := my_q.FieldByName('sts_name').AsString;
         testitem.Caption := Statefield.DispName;
         Statefield.CompName := 'State_' + IntToStr(my_q.RecNo);
         testitem.Name :=  Statefield.CompName;
         testitem.OnClick := OnStateClick;
         testitem.Color := fclTeal;
         Statefield.BitMask := 1 shl my_q.RecNo;
         Statefield.Comp := testitem;
         testitem.FontSettings.Color := cColorNoRightStat;

         if my_q2.RecordCount > 0 then
         begin
            bHasRights := False;
            my_q2.First;
            while not my_q2.Eof do
            begin
               If my_q2.FieldByName('sts_id').AsInteger = my_q.FieldByName('sts_id').AsInteger then
               begin
                  bHasRights := True;
                  Break;
               end;
               my_q2.Next;
            end;
         end;

         if bHasRights then
         begin
         	Statefield.ReadOnly := False;
            testitem.FontSettings.Color := clNone;
            testitem.Enabled := True;
         end
         else
         begin
            Statefield.ReadOnly := True;
         	testitem.FontSettings.Color := cColorNoRightStat;
            if not fThreeStates then
            begin
               testitem.Enabled := False;
               testitem.Visible := False;
            end;

         end;

         fStatefieldTab.Add(Statefield);
         Self.Items.Add(testitem);
         Statefield := nil;
         my_q.Next;
      end;
      fArcid := iArcid;

   finally
       FreeAndNil(my_q);
       FreeAndNil(my_q2);
   end;

end;

procedure bfaStatefields.ShowState(iState : Integer);
var
   Statefield : clStateField;
begin
    if not Assigned(fStatefieldTab) then
    	Exit;

    fStatefieldTab.First;
    while not fStatefieldTab.EOF do
    begin
      TObject(Statefield) := fStatefieldTab.getRec;
      if Statefield.BitMask and iState = Statefield.BitMask then
      begin
         Statefield.StateSet := True;
         fState := fState or Statefield.BitMask;
         TTBXColorItem(Statefield.Comp).Color := fclLime;
      end;
    	fStatefieldTab.Next;
    end;
    fOldState := iState;
end;


procedure bfaStatefields.ClearStatefields;
begin
   if Assigned(fStatefieldTab) then
   	fStatefieldTab := nil;
   fState := 0;
   fSearchState := 0;
   fNotState := 0;
   fOldState := 0;
   fModified := False;
   fArcid := -1;
   Self.Items.Clear;
   Self.Visible := False;
end;

procedure bfaStatefields.ClearState;
var
   Statefield : clStateField;
begin
   fState := 0;
   fSearchState := 0;
   fNotState := 0;
   fOldState := 0;
   fModified := False;

   if not Assigned(fStatefieldTab) then
   	Exit;

	fStatefieldTab.First;
   while not fStatefieldTab.EOF do
   begin
      TObject(Statefield) := fStatefieldTab.getRec;
      TTBXColorItem(Statefield.Comp).Color := fclTeal;
		fStatefieldTab.Next;
   end;
end;

procedure bfaStatefields.OnStateClick(Sender: TObject);
var
   Statefield : clStateField;
begin
   if not (Sender is TTBXColorItem) //m.Frey - zur Sicherheit
   then
      Exit;

   Statefield := fStatefieldTab.findComp(TTBXColorItem(Sender).Name);

   if TTBXColorItem(Sender).Color = fclTeal then
   begin
      TTBXColorItem(Sender).Color := fclLime;
      Statefield.StateSet := True;
      Statefield.StateNot := False;
      if (not Statefield.ReadOnly) or fIgnoreRightsAtImport then
	      fState := fState or Statefield.BitMask;
      fSearchState := fSearchState or Statefield.BitMask;
      fNotState := fNotState and not Statefield.BitMask;
   end
   else if TTBXColorItem(Sender).Color = fclLime then
   begin
      if fThreeStates then
      begin
         TTBXColorItem(Sender).Color := fclRed;
         Statefield.StateSet := False;
         Statefield.StateNot := True;
         if (not Statefield.ReadOnly) or fIgnoreRightsAtImport then
	         fState := fState and not Statefield.BitMask;
         fSearchState := fSearchState and not Statefield.BitMask;
	      fNotState := fNotState or Statefield.BitMask;
      end
	   else
      begin
         TTBXColorItem(Sender).Color := fclTeal;
         Statefield.StateSet := False;
         Statefield.StateNot := False;
         if (not Statefield.ReadOnly) or fIgnoreRightsAtImport then
	         fState := fState and not Statefield.BitMask;
         fSearchState := fSearchState and not Statefield.BitMask;
	      fNotState := fNotState and not Statefield.BitMask;
      end;
   end
   else if TTBXColorItem(Sender).Color = fclRed then
   begin
   	TTBXColorItem(Sender).Color := fclTeal;
      Statefield.StateSet := False;
      Statefield.StateNot := False;
      if (not Statefield.ReadOnly) or fIgnoreRightsAtImport then
      	fState := fState and not Statefield.BitMask;
      fSearchState := fSearchState and not Statefield.BitMask;
	   fNotState := fNotState and not Statefield.BitMask;
   end;

   fModified := fState <> fOldState;

   if Assigned(AfterStateClick) then
       AfterStateClick(Statefield);

end;


end.
