unit u_ShViewer;

interface

uses u_ShBase, u_ShSocketConnection, Classes, u_ShDocument, ievect, Forms,
		 XMLDoc, XMLIntf, Dialogs, u_ShRequestHandler, SysUtils, u_ShLowUtils;

type
	TViewer = class(TObject)
//  private
  	fConnectionType: byte;
    fSocketType: byte;
    fUserID: integer;
    fUserName: string;
    fServerHost: string;
    fServerPort: integer;
    fServerReadTimeout: integer;
    fDir: string;
    fCertFile: string;
    fArcPath, fCopyPath : string;
    procedure CreateStreams;

	published
   	property ConnectionType: byte read fConnectionType;      // Normal(0) Connection or Socket(1) Connection
  	property SocketType: byte read fConnectiontype;      // NoSSL(0) Connection or SSL(1) Connection
    property UserID: integer read fUserID write fUserID;
    property UserName: string read fUserName write fUserName;
    property ServerHost: string read fServerHost write fServerHost;
    property ServerPort: integer read fServerPort write fServerPort;
    property ServerReadTimeout: integer read fServerReadTimeout write fServerReadTimeout;
    property Dir: string read fDir write fDir;
    property CertFile: string read fCertFile write fCertFile;
    property ArcPath: string read fArcPath write fArcPath;
    property CopyPath: string read fCopyPath write fCopyPath;
  public
    MainSocketConnection: TSocketConnection;
    Document: TDocument;
    FStreamTIF: TMemoryStream;
    FStreamIEV: TMemoryStream;
    FStreamORI: TMemoryStream;
    FStreamLetter: TMemorystream;

    ReqHdl: TRequestHandler;
    ReqHdlThr: TRequestHandler;

    constructor Create(ConnType: byte); virtual;
    procedure CreateSocketConnection(SocketType: byte);

    destructor Destroy;

    function Login: boolean;

  end;

implementation

//
//  Name: Create
//  Parameters:
//  Description: Normal Create if there is no Socket Connection
//  Return value:
// []  b.takacs
//------------------------------------------------------------------------------
constructor TViewer.Create(ConnType: byte);
begin
	GgfLogEvntBeg('Create Normal Viewer');

	inherited Create;

   fConnectionType:= ConnType;

   Document:= TDocument.Create;
	GgfLogEvntEnd('Create Normal Viewer');
end;
//
//  Name: CreateSocket
//  Parameters:
//  Description:  Create with default parameters. This Create is used if the
// 								Connection is a Socket Connection
//  Return value:
// []  b.takacs
//------------------------------------------------------------------------------
procedure TViewer.CreateSocketConnection(SocketType: byte);
begin
	GgfLogEvntBeg('Create Socket Viewer');

  if SocketType = sctpNoSSL then
    fSocketType:= sctpNoSSL
  else
  	fSocketType:= sctpSSL;

  MainSocketConnection:= TSocketConnection.Create(fSocketType, fServerHost, fServerPort, fServerReadTimeout, fDir, fCertFile);

  CreateStreams;

  Document.VoransichtSocketConnection:= TSocketConnection.Create(fSocketType, fServerHost, fServerPort, fServerReadTimeout, fDir, fCertFile);
  Document.IEVSocketConnection:= TSocketConnection.Create(fSocketType, fServerHost, fServerPort, fServerReadTimeout, fDir, fCertFile);
  Document.OrifileSocketConnection := TSocketConnection.Create(fSocketType, fServerHost, fServerPort, fServerReadTimeout, fDir, fCertFile);


  ReqHdl:= TRequestHandler.Create;
  ReqHdlThr:= TRequestHandler.Create;

	GgfLogEvntEnd('Create Socket Viewer');

end;
//
//  Name:
//  Parameters:
//  Description:
//  Return value:
// []  b.takacs
//------------------------------------------------------------------------------
destructor TViewer.Destroy;
begin
	GgfLogEvntBeg('Destroy Viewer');
  if fConnectionType = vctpSocket then
  begin
    FStreamTIF.Free;
    FStreamIEV.Free;
    FStreamORI.Free;
    FStreamLetter.Free;
    Document.Free;
    MainSocketConnection.Free;
    ReqHdl.Free;
    ReqHdlThr.Free;
  end
  else
  begin
    Document.Free;
  end;
	GgfLogEvntEnd('Destroy Viewer');
end;
//
//  Name:
//  Parameters:
//  Description:
//  Return value:
// []  b.takacs
//------------------------------------------------------------------------------
procedure TViewer.CreateStreams;
begin
	FStreamTIF := TMemorystream.Create;
	FStreamIEV := TMemorystream.Create;
  FStreamORI := TMemorystream.Create;
  FStreamLetter := TMemorystream.Create;
end;
//
//  Name:
//  Parameters:
//  Description:
//  Return value:
// []  b.takacs
//------------------------------------------------------------------------------
function TViewer.Login: boolean;
var
	str: string;
begin
	GgfLogEvntBeg('Viewer.Login');
  try
    ReqHdl.user:= fUserID;
    ReqHdl.reqType:= strReqAnmeldung;
    try
      MainSocketConnection.SendXMLToServer(ReqHdl.createXMLString);
    except
      result:= false;
    end;
    try
      str:= MainSocketConnection.ReceiveXML;
    except
    end;
    try
      ReqHdl.readXMLString(str);
    except
    end;
  finally
    if ReqHdl.reqType = strRespAuth then
      result:= true
    else
      result:= false;
  end;
	GgfLogEvntEnd('Viewer.Login');
end;
//

end.
