unit Collision;

interface

uses
   vectors,  // bentigt fr Vektorrechnung
   Math;     // bentigt fr min / max funktionen

type
   // Array mit den Vertices des Polygons
   TV2fArray = array of TVector2f;
   // Polygon Klasse
   TPolygon = class
   private
      fposition: TVector2f;                             // Position
      fvertices: TV2fArray;                             // Vertices (Objektkoordinaten)
      function GetVertex(n: integer): TVector2f;        // Liefert die Objektkoordinaten
      function GetVertexAbs(n: integer): TVector2f;
      // Liefert die absoluten Koordinaten
      procedure SetVertex(n: integer; Value: TVector2f);// Setzt die Objektkoordinaten
      function GetCount: integer;                       // Liefert length(fvertices)
   public
      procedure AddVertex(v: TVector2f);                // Fgt ein Vertex hinzu
      procedure AddVertexAbs(v: TVector2f);
      // Fgt ein Vertex mit Weltkoordinaten hinzu
      procedure RemoveVertex(n: integer);               // Entfernt ein Vertex
      property position: TVector2f Read fposition Write fposition;             // Position
      property vertices[n: integer]: TVector2f Read GetVertex Write SetVertex;
      // Vertex Koordinaten
      property vertices_abs[n: integer]: TVector2f Read GetVertexAbs;
      // Vertex Weltkoordinaten
      property Count: integer Read GetCount;
      // siehe GetCount
   end;

   TPolygonArray = array of TPolygon;
   // Kreis Klasse
   TCircle = class
   private
      fposition: TVector2f;
      fradius: extended;
   public
      property position: TVector2f Read fposition Write fposition;
      property radius: extended Read fradius Write fradius;
   end;

// Export Kollisionserkennung
function PolyPolyIntersect(A, B: TPolygon; var MTD: TVector2f): boolean;
function CircleCircleIntersect(A, B: TCircle): boolean;
function CirclePolyIntersect(P: TPolygon; C: TCircle): boolean;

implementation

procedure TPolygon.AddVertexAbs(v: TVector2f);
begin
   setlength(fvertices, length(fvertices) + 1);
   fvertices[high(fvertices)] := v2f_sub(v, position);
end;

procedure TPolygon.AddVertex(v: TVector2f);
begin
   setlength(fvertices, length(fvertices) + 1);
   fvertices[high(fvertices)] := v;
end;

procedure TPolygon.RemoveVertex(n: integer);
var
   i: integer;
begin
   for i := n to high(fvertices) - 1 do
      fvertices[i] := fvertices[i + 1];
   setlength(fvertices, length(fvertices) - 1);
end;

function TPolygon.GetVertex(n: integer): TVector2f;
begin
   Result := fvertices[n];
end;

function TPolygon.GetVertexAbs(n: integer): TVector2f;
begin
   Result := v2f_add(fvertices[n], fposition);
end;

procedure TPolygon.SetVertex(n: integer; Value: TVector2f);
begin
   fvertices[n] := Value;
end;

function TPolygon.GetCount: integer;
begin
   Result := length(fvertices);
end;

function CreateAxis(P: TPolygon): TV2fArray;
var
   i, l: integer;
   tmp: TVector2f;
begin
   for i := 0 to (P.Count - 1) do
   begin
      l := i + 1;
      if l > (P.Count - 1) then
         l := 0;
      // Berechnung der Seitenflche
      tmp := v2f_sub(P.vertices[l], P.vertices[i]);
      // Berechnet die Normale der Seitenflche
      setlength(Result, length(Result) + 1);
      Result[high(Result)] := v2f_normalize(to_v2f(-tmp.y, tmp.x));
   end;
end;

procedure ProjectOntoAxis(P: TPolygon; proj: TVector2f; var pmin, pmax: extended);
var
   i: integer;
   dp: extended;
begin
   // Projeziert den ersten Wert
   pmin := v2f_dotproduct(P.vertices[0], proj);
   pmax := pmin;
   // Findet den kleinsten und grten projezierten Wert fr die Gerade fr P
   for i := 1 to (P.Count - 1) do
   begin
      // projezieren
      dp := v2f_dotproduct(P.vertices[i], proj);
      if dp < pmin then
         pmin := dp;
      if dp > pmax then
         pmax := dp;
   end;
end;

function CollisionCheck(A, B: TPolygon; var axis: TVector2f;
   voffset: TVector2f): boolean;
var
   foffset, amin, amax, bmin, bmax, d1, d2, depth: extended;
begin
   ProjectOntoAxis(A, axis, amin, amax);
   ProjectOntoAxis(B, axis, bmin, bmax);
   foffset := v2f_dotproduct(voffset, axis);
   amin := amin + foffset;
   amax := amax + foffset;
   d1 := amin - bmax;
   d2 := bmin - amax;
   // Wenn es keine berschneidung gibt, abbrechen -> keine Kollision
   if (d1 > 0) or (d2 > 0) then
   begin
      Result := False;
      exit;
   end;
   // Ansonsten den Verschiebungsvektor bestimmen
   depth := max(d1, d2);
   axis := v2f_scale(axis, abs(depth));
   Result := True;
end;

function PolyPolyIntersect(A, B: TPolygon; var MTD: TVector2f): boolean;
var
   axis: TV2fArray;
   voffset: TVector2f;
   i: integer;
begin
   MTD := to_v2f(0, 0);
   // Offset berechnen
   voffset := v2f_sub(A.position, B.position);
   // Alle Achsen fr A
   axis := CreateAxis(A);
   // Alle Achsen fr B
   axis := CreateAxis(B);
   // Projezieren der Polygone
   for i := 0 to high(axis) do
      if CollisionCheck(A, B, axis[i], voffset) = False then
      begin
         Result := False;
         exit;
      end;
   // MTD bestimmen
   MTD := axis[0];
   for i := 1 to high(axis) do
      if v2f_length(axis[i]) < v2f_length(MTD) then
         MTD := axis[i];
   if v2f_dotproduct(voffset, MTD) < 0 then
      MTD := v2f_scale(MTD, -1);
   // Kollision
   Result := True;
end;

function CircleCircleIntersect(A, B: TCircle): boolean;
begin
   Result := (sqr(A.position.x - B.position.x) + sqr(A.position.y - B.position.y)) <
      sqr(A.radius + B.radius);
end;

function CirclePolyIntersect(P: TPolygon; C: TCircle): boolean;
var
   i, j, l: integer;
   tmp, proj, voffset: TVector2f;
   dp, pmin, pmax, cmin, cmax, d1, d2, foffset: extended;
begin
   // Offset berechnen
   voffset := v2f_sub(P.position, C.position);
   // P - alle Projektionsgeraden ermitteln und projezieren
   for i := 0 to (P.Count - 1) do
   begin
      l := i + 1;
      if l > (P.Count - 1) then
         l := 0;
      // Berechnung der Seitenflche
      tmp := v2f_sub(P.vertices[l], P.vertices[i]);
      // Berechnet die Normale der Seitenflche
      proj := v2f_normalize(to_v2f(-tmp.y, tmp.x));
      // Projeziert den ersten Wert
      pmin := v2f_dotproduct(P.vertices[0], proj);
      pmax := pmin;
      // Findet den kleinsten und grten projezierten Wert fr die Gerade fr A
      for j := 1 to (P.Count - 1) do
      begin
         // projezieren
         dp := v2f_dotproduct(P.vertices[j], proj);
         if dp < pmin then
            pmin := dp;
         if dp > pmax then
            pmax := dp;
      end;
      cmax := v2f_dotproduct(v2f_scale(proj, C.radius), proj);
      cmin := -cmax;
      // 1D Kollision
      foffset := v2f_dotproduct(voffset, proj);
      pmin := pmin + foffset;
      pmax := pmax + foffset;
      d1 := pmin - cmax;
      d2 := cmin - pmax;
      // Wenn es keine berschneidung gibt, abbrechen -> keine Kollision
      if (d1 > 0) or (d2 > 0) then
      begin
         Result := False;
         exit;
      end;
      //C - Alle Projektionsgeraden ermitteln und projezieren
      proj := v2f_normalize(v2f_sub(C.position, P.vertices_abs[i]));
      // s.o.
      pmin := v2f_dotproduct(P.vertices[0], proj);
      pmax := pmin;
      for j := 1 to (P.Count - 1) do
      begin
         dp := v2f_dotproduct(P.vertices[j], proj);
         if dp < pmin then
            pmin := dp;
         if dp > pmax then
            pmax := dp;
      end;
      cmax := v2f_dotproduct(v2f_scale(proj, C.radius), proj);
      cmin := -cmax;
      foffset := v2f_dotproduct(voffset, proj);
      pmin := pmin + foffset;
      pmax := pmax + foffset;
      d1 := pmin - cmax;
      d2 := cmin - pmax;
      if (d1 > 0) or (d2 > 0) then
      begin
         Result := False;
         exit;
      end;
   end;
   Result := True;
end;

end.
