unit u_ShLowUtils;
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Utils fr bitfarm
// m.frey 09-09-08 creates
// diverse Kleinigkeiten um andere units zu "erleichtern"
// die uses-Liste ist klein zu halten und es drfen nur low-level verwendet werden.

interface

{$WRITEABLECONST ON}

uses
   Windows, SysUtils,
   Nb30, gnugettext,
   U_getmac_iphlpapi;

type
   TBfData = class
   public
      Appdata,
      AppdataBitfarm,
      MyloginIniName,
      SystemVariableTemp,   // %temp%
      SystemVariableTempPd, // %temp% mit Pathdelimiter
      dummy: string;
      constructor Create;
   end;

const BD: TBfData = nil;

function GetSystemVariable(const Name: string): string;
procedure LogEvnt(LogMsg: string);

function EnCrypt(InputStr, HexKeyStr: string): string;
function DeCrypt(HexInputStr, HexKeyStr: string): string;
function GetVolSerial(DriveLetter: string): string;
function GetMACAdress: string;
function GetAdapters: TLanaEnum;

function Registrierungscode(const sKS: string): string;


implementation


var csLog: RTL_CRITICAL_SECTION; //09-09-07 m.frey


//-----------------------------------------------------------------------------
function GetSystemVariable(const Name: string): string;
   //////////////////////////////////////////////////////////////////
   ///  02.05.2005 Erstellt GetSystemVariable by SV.              ///
   ///                                                            ///
   ///  Diese Funktion liest die Systemvariable in 'Name' aus     ///
   ///  und liefert ihren Wert als Delphi String                  ///
   ///                                                            ///
   ///  Format: String                                            ///
   ///                                                            ///
   ///  09-09-07 m.frey: nach unit u_ShLowUtils                      ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
begin
   try
      //m.frey SetLength(Result, 255);
      //m.frey SetLength(Result, GetEnvironmentVariable(PChar(Name), PChar(Result), 255));
      Result := Sysutils.GetEnvironmentVariable(Name);
   except
      on e: Exception do
         LogEvnt('9003    ' + _('Fehler') + ' in Modul: GetSystemVariable    ' + e.Message);
   end;
end;


//-----------------------------------------------------------------------------
procedure LogEvnt(LogMsg: string);
//////////////////////////////////////////////////////////////////
///  10.04.2005 Erstellt LogEvnt by SV.                        ///
///                                                            ///
///  Diese Procedure schreibt LogMsg mit Datum in eine Datei   ///
///                                                            ///
//////////////////////////////////////////////////////////////////
var
   fLogFile: TextFile;
   sLogDate, Slognfile, sLogPathFile: string;
begin
   EnterCriticalSection(csLog); //09-09-07 m.frey
   try
      //OutputDebugString('LogEvnt {');
      // get for timerevelant actions the server time
      {
        dm_db.my_q_datetime.SQL.text:='select date_format(now(),"%d.%m.%Y %T") as date';
        dm_db.my_q_datetime.execute;
        sGTmpName  :=dm_db.my_q_datetime.fieldbyname('date').asstring;
      }
      DateTimeToString(sLogDate, 'dd.mm.yyyy hh:nn:ss', now);
      DateTimeToString(Slognfile, 'yyyy-mm-dd', now);
      try
         sLogPathFile := BD.SystemVariableTemp + '\ViewerV3' + Slognfile + '.log';

         AssignFile(fLogFile, sLogPathFile);
         if not FileExists(sLogPathFile) then
            ReWrite(fLogFile);
         Reset(fLogFile);
         Append(fLogFile);
         WriteLn(fLogFile, sLogDate + '    ' + LogMsg);
         Flush(fLogFile);
         CloseFile(fLogFile);
      except
         // on e: exception do ShowMessage('Schwerer Fehler:' + e.Message + #13 + #13 + 'Bitte Prfen Sie die Berechtigung auf die Datei ViewerV3.Log im Ordner ' + GetSystemVariable('systemroot'));
      end;
      //OutputDebugString('LogEvnt }');
   finally
      LeaveCriticalSection(csLog); //09-09-07 m.frey
   end;
end;


//-----------------------------------------------------------------------------
constructor TBfData.Create;
begin
   Appdata := PChar(GetSystemVariable('APPDATA'));
   AppdataBitfarm := Appdata + '\bitfarm-archiv';
   MyloginIniName := AppdataBitfarm + '\mylogin.ini';
   SystemVariableTemp := GetSystemVariable('Temp');
   SystemVariableTempPd := IncludeTrailingPathDelimiter(SystemVariableTemp);
end;


//-----------------------------------------------------------------------------
function EnCrypt(InputStr, HexKeyStr: string): string;
   //////////////////////////////////////////////////////////////////
   ///  10.04.2005 Erstellt EnCrypt by SV.                        ///
   ///                                                            ///
   ///  Diese Funktion erzeugt aus dem InputStr einen mit         ///
   ///  HexKeyStr verschlsselten HexString (einfaches XOR)       ///
   ///                                                            ///
   ///  Format: String aus Hexadezimalzahlen                      ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
var
   iCntIStr, iCntKStr: integer;
begin
   iCntIStr := 1;
   iCntKStr := 1;
   Result := '';
   while iCntIStr < (Length(InputStr) + 1) do
   begin
      try
         Result := Result + IntToHex(
            (Ord(InputStr[iCntIStr]) xor StrToInt('$' +
            copy(HexKeyStr, iCntKStr mod (Length(HexKeyStr) + 1), 2))), 2);
      except
         on e: Exception do
         begin
            LogEvnt('9001    ' + _('Fehler') + ' in Modul: EnCrypt    ' + e.Message);
            Result := '';
            exit;
         end;
      end;
      Inc(iCntIStr, 1);
      Inc(iCntKStr, 2);
   end;
end;

//-----------------------------------------------------------------------------
function DeCrypt(HexInputStr, HexKeyStr: string): string;
   //////////////////////////////////////////////////////////////////
   ///  10.04.2005 Erstellt DeCrypt by SV.                        ///
   ///                                                            ///
   ///  Diese Funktion erzeugt aus dem HexInputStr einen mit      ///
   ///  HexKeyStr entschlsselten String (einfaches XOR)          ///
   ///                                                            ///
   ///  Format: Entschlsselter String                            ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
var
   iCntIStr, iCntKStr: integer;
begin
   iCntIStr := 1;
   iCntKStr := 1;
   Result := '';
   while iCntIStr < (Length(HexInputStr) + 1) do
   begin
      try
         Result := Result + chr((StrToInt('$' + copy(HexInputStr, iCntIStr, 2)) xor
            StrToInt('$' + copy(HexKeyStr, iCntKStr mod (Length(HexKeyStr) + 1),
            2))));
      except
         on e: Exception do
         begin
            LogEvnt('9002    ' + _('Fehler') + ' in Modul: DeCrypt    ' + e.Message);
            Result := '';
            exit;
         end;
      end;
      Inc(iCntIStr, 2);
      Inc(iCntKStr, 2);
   end;
end;


//-----------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////
///  Function GetAdapter                                              ///
///  =================                                                ///
///                                                                   ///
///  This function will return the total nummber of all lana's        ///
///  istalled and a unique the id of each lana.                       ///
///  E.g. on a system containing lana's 0, 2 and 3, a structure with  ///
///  length =3, lana[0]=0, lana[1]=2 and lana[2]=3 will be returned.
///
/// Diese Funktion gibt die gesamte Anzahl aller installierten Local Area network Adapter an
///
///                                                                   ///
/////////////////////////////////////////////////////////////////////////

function GetAdapters: TLanaEnum;
var
   NCB: TNCB;      //Network Control Block
   LanaEnum: TLanaEnum; //Enumeration of Lan Adapters
   ReturnCode: char;    //Retun Code

begin
   //Clear NCB Structure
   FillChar(NCB, SizeOf(NCB), 0);
   //Clear LanaEnum Stucture
   FillChar(LanaEnum, SizeOf(TLanaEnum), 0);

   //Programm NBC to ask for the number of Lana`s
   NCB.ncb_Command := char(NCBENUM);
   NCB.ncb_buffer := @LanaEnum;
   NCB.ncb_length := Sizeof(LanaEnum);

   //Ask NetBios
   ReturnCode := NetBios(@NCB);

   //Check if query was successfull, if returnCode = #0
   if ReturnCode <> #0 then
   begin
      LanaEnum.Length := char(0);
      LanaEnum.Lana[0] := ReturnCode;
   end;

   // Return valid LanaEnum Structure
   Result := LanaEnum;
end;

(**)
//-----------------------------------------------------------------------------
function GetMACAdress: string;
   //////////////////////////////////////////////////////////////////
   ///  01.08.2005 Erstellt by SV. Orginal by Tony G              ///
   ///             www.tgsoft.ch                                  ///
   ///                                                            ///
   ///  Liest die MAC Adresse des ersten Netzwerkadapters aus     ///
   ///                                                            ///
   ///  Ausgabe: MAC Adresse als String                           ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
   // todo buggy should check for networkcard which doesnt change!
   // e.g. exclude usb- devices or check if there is only one device.

type
   TAdapterStatusA = record
      Adapt: TAdapterStatus;
      NameBuff: array[0..30] of TNameBuffer;
   end;
var
   NCB: TNCB;     //Network Control Block
   ReturnCode: char; //Retun Code
   AdapterStatus: TAdapterStatus;
   AdapterID: integer;
   L_Enum: TLanaEnum;
   sZwResult: string; // Zwischenergebnis
   iCount: integer;
   getmaciphlpapi: TGetmac;

begin
   //Clear NCB Structure
   FillChar(NCB, SizeOf(NCB), 0);
   l_Enum := GetAdapters;
   AdapterID := byte(l_ENUM.lana[0]);

   //Programm NBC to fulfill a reset. It doublecheck if the Lana is present.
   //Can be removed, when used with GetAdapters.
   NCB.ncb_command := char(NCBRESET);
   NCB.ncb_lana_num := char(AdapterID);

   //Process NCB to NetBios
   ReturnCode := NetBios(@NCB);

   //Check if query was successfull, if returnCode = #0
   if ReturnCode <> #0 then
   begin
      // Return the negative returncode
      // Error check would be "if ReturnCode[0]='-' Then..."

      //swithing for advanced solution
      getmaciphlpapi := Tgetmac.Create;
      Result := getmaciphlpapi.getfirstmac;

      FreeAndNil(getmaciphlpapi);
      //     Result := char(Ord(ReturnCode) * -1);
      Exit;
   end;

   //Clear NCB Structure
   FillChar(NCB, SizeOf(NCB), 0);

   //Programm NBC to ask for the MAC associated with the LanaID
   NCB.ncb_command := char(NCBASTAT);
   NCB.ncb_lana_num := char(AdapterID);
   NCB.ncb_buffer := @AdapterStatus;
   NCB.ncb_length := SizeOf(AdapterStatus);
   StrCopy(NCB.ncb_callname, '* ');

   //Process NCB to NetBios
   ReturnCode := Netbios(@NCB);

   //Check if query was successfull, if returnCode = #0
   if ReturnCode <> #0 then
   begin
      // Return the negative returncode
      Result := char(Ord(ReturnCode) * -1);
      Exit;
   end;

   //Return formated MAC
   sZwResult := Format('%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x',
      [Ord(AdapterStatus.Adapter_address[0]), Ord(AdapterStatus.Adapter_address[1]),
      Ord(AdapterStatus.Adapter_address[2]), Ord(AdapterStatus.Adapter_address[3]),
      Ord(AdapterStatus.Adapter_address[4]), Ord(AdapterStatus.Adapter_address[5])]);
   iCount := Length(sZwResult);
   if iCount < 12 then
      Result := sZwResult + StringOfChar('A', 12 - iCount)
   else
   if iCount > 12 then
      Result := Copy(sZwResult, 1, 12)
   else
   if iCount = 12 then
      Result := sZwResult;
end;
(**)

//-----------------------------------------------------------------------------
function GetVolSerial(DriveLetter: string): string;
   //////////////////////////////////////////////////////////////////
   ///  04.05.2005 Erstellt by SV.                                ///
   ///                                                            ///
   ///  Liest die Seriennummer des Volumes DriveLetter aus        ///
   ///                                                            ///
   ///  Ausgabe: Volume Seriennummer als String                   ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
var
   wSerial: DWord;
   cMaxCompLen, cFSFlags: cardinal;
   aVolName, aFSName: array[0..255] of char;
   sZwResult: string;
   iCount: integer;
begin
   if GetVolumeInformation(PChar(DriveLetter), aVolName, 256, @wSerial,
      cMaxCompLen, cFSFlags, aFSName, 256) then
      sZwResult := IntToHex(wSerial, 4)
   else
      sZwresult := 'aa55aa55';
   iCount := Length(sZwResult);
   if iCount < 8 then
      Result := sZwResult + StringOfChar('A', 8 - iCount)
   else
   if iCount > 8 then
      Result := Copy(sZwResult, 1, 8)
   else
   if iCount = 8 then
      Result := sZwResult;
end;

//-----------------------------------------------------------------------------
function Registrierungscode(const sKS: string): string;
var sMAC, sVID, NoRegCode: string;
begin
   sMAC := StringReplace(GetMacAdress, '-', '', [rfReplaceAll]);
   if length(sMac) < 12 then
      sMAC := '0000CB62EE98';
   sVID := GetVolSerial('c:\');
   if length(sVID) < 8 then
      sVID := '0c5E1602';
   result := Encrypt(sVID + sMAC, sKS);
end;
(**)

//-----------------------------------------------------------------------------

initialization
   InitializeCriticalSection(csLog); //09-09-07 m.frey
   BD := TBfData.Create;

finalization
   FreeAndNil(BD);
   DeleteCriticalSection(csLog); //09-09-07 m.frey

end.
