unit u_ObjInfo;

interface

uses
   Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
   Dialogs, Buttons, StdCtrls, ievect, gnugettext;

type
   Tfrm_AnnoInfo = class (TForm)
      grbx_Alg: TGroupBox;
      lbl_AName: TLabel;
      lbl_ATyp: TLabel;
      lbl_AID: TLabel;
      lbl_ANameTxt: TLabel;
      lbl_ATypTxt: TLabel;
      lbl_AIDTxt: TLabel;
      grbx_Messw: TGroupBox;
      lbl_MFlaeche: TLabel;
      lbl_MUmfang: TLabel;
      lbl_MLeft: TLabel;
      lbl_MWidth: TLabel;
      lbl_MFlaecheTxt: TLabel;
      lbl_MUmfangTxt: TLabel;
      lbl_MLeftTxt: TLabel;
      lbl_MTop: TLabel;
      lbl_MWidthTxt: TLabel;
      lbl_MHeight: TLabel;
      lbl_MTopTxt: TLabel;
      lbl_MHeightTxt: TLabel;
      sbtn_Prev: TSpeedButton;
      sbtn_Next: TSpeedButton;
      cbx_MMesTyp: TComboBox;
      lbl_MEinheit: TLabel;
      procedure FormShow(Sender: TObject);
      procedure cbx_MMesTypChange(Sender: TObject);
      procedure sbtn_PrevClick(Sender: TObject);
      procedure sbtn_NextClick(Sender: TObject);
      procedure FormCreate(Sender: TObject);
   protected
      procedure CreateParams(var Params: TCreateParams); override;
      { Private-Deklarationen }
   public
      { Public-Deklarationen }
      iGSelObjIdx: integer;
      function LoadObjInfo(idx: integer): boolean;
   end;

var
   frm_AnnoInfo: Tfrm_AnnoInfo;

implementation

{$R *.dfm}

uses u_DocEditor;

function ElUmfang(a, b: double): double;
   //////////////////////////////////////////////////////////////////
   ///  09.07.2005 Erstellt by SV.                                ///
   ///                                                            ///
   ///  Diese Funktion errechnet den Umfang einer Ellipse oder    ///
   ///  Kreises [a = b]                                           ///
   ///                                                            ///
   ///  Parameter: a           Lnge des langen Radius            ///
   ///             b           Lnge des kurzen Radius            ///
   ///                                                            ///
   ///  Rckgabe:  Umfang der Ellipse                             ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
var
   eps, eq, s, t, h: double;
   k: integer;
begin
   eps := 1E-16;
   if (a < b) then
   begin
      h := a;
      a := b;
      b := h;
   end;
   eq := 1 - b * b / (a * a); // quadrierte numerische Exzentrizitaet
   s := 0;
   k := 2;
   t := 1 / 4 * eq;
   while (t >= eps) do
   begin
      s := s + t;
      t := t * eq * (k - 1) * (k + 1) / ((k + 2) * (k + 2));
      k := k + 2;
   end;
   Result := 2 * pi * a * (1 - s);
end;

function ElFlaeche(a, b: double): double;
   //////////////////////////////////////////////////////////////////
   ///  09.07.2005 Erstellt by SV.                                ///
   ///                                                            ///
   ///  Diese Funktion errechnet die Flche einer Ellipse oder    ///
   ///  eines Kreises [a = b]                                     ///
   ///                                                            ///
   ///  Parameter: a           Lnge des langen Radius            ///
   ///             b           Lnge der kurzen Radius            ///
   ///                                                            ///
   ///  Rckgabe:  Flche der Ellipse                             ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
begin
   Result := PI * a * b;
end;

function ReUmfang(a, b: double): double;
   //////////////////////////////////////////////////////////////////
   ///  09.07.2005 Erstellt by SV.                                ///
   ///                                                            ///
   ///  Diese Funktion errechnet den Umfang eines Rechtecks       ///
   ///                                                            ///
   ///  Parameter: a           Lnge der Seite a                  ///
   ///             b           Lnge der Seite b                  ///
   ///                                                            ///
   ///  Rckgabe:  Umfang des Rechtecks                           ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
begin
   Result := 2 * a + 2 * b;
end;

function ReFlaeche(a, b: double): double;
   //////////////////////////////////////////////////////////////////
   ///  09.07.2005 Erstellt by SV.                                ///
   ///                                                            ///
   ///  Diese Funktion errechnet die Flche eines Rechtecks.      ///
   ///                                                            ///
   ///  Parameter: a           Lnge der Seite a                  ///
   ///             b           Lnge der Seite b                  ///
   ///                                                            ///
   ///  Rckgabe:  Flche des Rechtecks                           ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
begin
   Result := a * b;
end;

function LiLaenge(a, b: double): double;
   //////////////////////////////////////////////////////////////////
   ///  09.07.2005 Erstellt by SV.                                ///
   ///                                                            ///
   ///  Diese Funktion errechnet die Lnge einer Linie nach Py-   ///
   ///  tagoras [l = Wurzel aus a * b]                          ///
   ///                                                            ///
   ///  Parameter: a           Lnge der Seite a                  ///
   ///             b           Lnge der Seite b                  ///
   ///                                                            ///
   ///  Rckgabe:  Lnge der Linie                                ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
begin
   Result := sqrt(a * a + b * b);
end;

function Tfrm_AnnoInfo.LoadObjInfo(idx: integer): boolean;
   //////////////////////////////////////////////////////////////////
   ///  09.07.2005 Erstellt by SV.                                ///
   ///                                                            ///
   ///  Diese Funktion errechnet die Geometrischen Eigenschaften  ///
   ///  des selektierten Objektes [idx] und zeigt diese an.       ///
   ///                                                            ///
   ///  Parameter: idx           Selektiertes Objekt [idx]        ///
   ///             -1            Objekt zurck                    ///
   ///             -2            Objekt vor                       ///
   ///                                                            ///
   ///  Rckgabe:  True bei Erfolg                                ///
   ///             False bei Fehlschlag                           ///
   ///                                                            ///
   //////////////////////////////////////////////////////////////////
var
   iHobj: integer;
   fArea, fLen: double;
   sObjTyp, sMesTyp: string;
begin
   Result := True;
   try
      with frm_DocEdit.fra_DocEditor.img_DocView do
      begin
         fArea := 0;
         fLen := 0;
         if SelObjectsCount < 0 then
            exit;
         if idx = -1 then
            idx := iGSelObjIdx - 1; // Vorheriges selektiertes Objekt
         if idx = -2 then
            idx := iGSelObjIdx + 1; // Nchstes selektiertes Objekt
         if (idx > (SelObjectsCount - 1)) or (idx < 0) then
            idx := 0; // Prfung auf Plausibilitt und Korrektur
         iGSelObjIdx := idx; // Globale Variable mit aktuellem Objektindex fllen
         iHobj := SelObjects[idx];
         lbl_ANameTxt.Caption := ObjName[iHobj];
         case ObjKind[iHobj] of
            iekNONE:
            begin
               sObjTyp := _('kein');
               fArea := 0;
               fLen := 0;
            end;
            iekLINE:
            begin
               sObjTyp := _('einfache Linie / Pfeil');
               fArea := 0;
               fLen := LiLaenge(MeasureCoefX * ObjWidth[iHObj],
                  MeasureCoefY * ObjHeight[iHObj]);
            end;
            iekBOX:
            begin
               sObjTyp := _('Rechteck');
               fArea := ReFlaeche(MeasureCoefX * ObjWidth[iHObj],
                  MeasureCoefY * ObjHeight[iHObj]);
               fLen := ReUmfang(MeasureCoefX * ObjWidth[iHObj],
                  MeasureCoefY * ObjHeight[iHObj]);
            end;
            iekELLIPSE:
            begin
               sObjTyp := _('Ellipse');
               fArea := ElFlaeche((MeasureCoefX * ObjWidth[iHObj]) / 2,
                  (MeasureCoefY * ObjHeight[iHObj]) / 2);
               fLen := ElUmfang((MeasureCoefX * ObjWidth[iHObj]) / 2,
                  (MeasureCoefY * ObjHeight[iHObj]) / 2);
            end;
            iekARC:
            begin
               sObjTyp := _('Bogen');
               fArea := 0;
               fLen := 0;
            end;
            iekBITMAP:
            begin
               sObjTyp := _('Bitmap / Grafik');
               fArea := ReFlaeche(MeasureCoefX * ObjWidth[iHObj],
                  MeasureCoefY * ObjHeight[iHObj]);
               fLen := ReUmfang(MeasureCoefX * ObjWidth[iHObj],
                  MeasureCoefY * ObjHeight[iHObj]);
            end;
            iekTEXT:
            begin
               sObjTyp := _('Text');
               fArea := ReFlaeche(MeasureCoefX * ObjWidth[iHObj],
                  MeasureCoefY * ObjHeight[iHObj]);
               fLen := ReUmfang(MeasureCoefX * ObjWidth[iHObj],
                  MeasureCoefY * ObjHeight[iHObj]);
            end;
            iekRULER:
            begin
               sObjTyp := _('Lineal / Bemassung');
               fArea := 0;
               fLen := LiLaenge(MeasureCoefX * ObjWidth[iHObj],
                  MeasureCoefY * ObjHeight[iHObj]);
            end;
            iekPOLYLINE:
            begin
               sObjTyp := _('Freihand Linie / Poligon');
               fArea := GetPolyLineArea(iHobj);
               fLen := GetPolyLineLen(iHObj);
            end;
            iekANGLE:
            begin
               sObjTyp := _('Winkel');
               fArea := 0;
               fLen := 0;
            end;
            iekMEMO:
            begin
               sObjTyp := _('Notiz');
               fArea := ReFlaeche(MeasureCoefX * ObjWidth[iHObj],
                  MeasureCoefY * ObjHeight[iHObj]);
               fLen := ReUmfang(MeasureCoefX * ObjWidth[iHObj],
                  MeasureCoefY * ObjHeight[iHObj]);
            end;
            iekLINELABEL:
            begin
               sObjTyp := _('Beschriftung');
               fArea := 0;
               fLen := LiLaenge(MeasureCoefX * ObjWidth[iHObj],
                  MeasureCoefY * ObjHeight[iHObj]);
            end;
         end;
         lbl_ATypTxt.Caption := sObjTyp;
         lbl_AIDTxt.Caption := IntToStr(ObjId[iHObj]);
         case MUnit of
            ieuPIXELS:
            begin
               sMesTyp := ' px';
               cbx_MMesTyp.ItemIndex := 0;
            end;
            ieuINCHES:
            begin
               sMesTyp := ' In';
               cbx_MMesTyp.ItemIndex := 1;
            end;
            ieuKM:
            begin
               sMesTyp := ' km';
               cbx_MMesTyp.ItemIndex := 5;
            end;
            ieuMETERS:
            begin
               sMesTyp := ' m';
               cbx_MMesTyp.ItemIndex := 4;
            end;
            ieuCENTIMETERS:
            begin
               sMesTyp := ' cm';
               cbx_MMesTyp.ItemIndex := 3;
            end;
            ieuMILLIMETERS:
            begin
               sMesTyp := ' mm';
               cbx_MMesTyp.ItemIndex := 2;
            end;
            ieuMICRONS:
            begin
               sMesTyp := ' m';
               cbx_MMesTyp.ItemIndex := 6;
            end;
            ieuNANOMETERS:
            begin
               sMesTyp := ' nm';
               cbx_MMesTyp.ItemIndex := 7;
            end;
         end;
         lbl_MFlaecheTxt.Caption := FloatToStr(fArea) + sMesTyp + '';
         lbl_MUmfangTxt.Caption := FloatToStr(fLen) + sMesTyp;
         lbl_MTopTxt.Caption := FloatToStr(MeasureCoefY * ObjTop[iHobj]) + sMesTyp;
         lbl_MLeftTxt.Caption := FloatToStr(MeasureCoefX * ObjLeft[iHobj]) + sMesTyp;
         lbl_MWidthTxt.Caption := FloatToStr(MeasureCoefX * ObjWidth[iHobj]) + sMesTyp;
         lbl_MHeightTxt.Caption := FloatToStr(MeasureCoefY * ObjHeight[iHobj]) + sMesTyp;
      end;
   except
      // nix
      Result := False;
   end;
end;

procedure Tfrm_AnnoInfo.FormShow(Sender: TObject);
begin
   LoadObjInfo(0);
end;

procedure Tfrm_AnnoInfo.cbx_MMesTypChange(Sender: TObject);
begin
   with frm_DocEdit.fra_DocEditor.img_DocView do
   begin
      case cbx_MMesTyp.ItemIndex of
         0:
            MUnit := ieuPIXELS;
         1:
            MUnit := ieuINCHES;
         2:
            MUnit := ieuMILLIMETERS;
         3:
            MUnit := ieuCENTIMETERS;
         4:
            MUnit := ieuMETERS;
         5:
            MUnit := ieuKM;
         6:
            MUnit := ieuMICRONS;
         7:
            MUnit := ieuNANOMETERS;
      end;
   end;
   LoadObjInfo(iGSelObjIdx);
end;

procedure Tfrm_AnnoInfo.sbtn_PrevClick(Sender: TObject);
begin
   LoadObjInfo(-1);
end;

procedure Tfrm_AnnoInfo.sbtn_NextClick(Sender: TObject);
begin
   LoadObjInfo(-2);
end;

procedure Tfrm_AnnoInfo.CreateParams(var Params: TCreateParams);
begin
   // so that mainform doesnt throw frm_docedit to background
   inherited;
   Params.ExStyle := Params.ExStyle; // or WS_EX_APPWINDOW;
   Params.WndParent := GetDesktopWindow;
end;

procedure Tfrm_AnnoInfo.FormCreate(Sender: TObject);
begin
   TranslateComponent(self, 'Viewerv3');
end;

end.
