//-----------------------------------------------------------------------------
// Projekt:    Bitfarm-Archiv Administrator
// Entwickler: Frey
// Erstellt:   2009-10-19
// Name:       u_SvnVersion
// Typ:        Unit
// Zweck:      Anzeige und Auswahlmglichkeit von Versionen eines Dokumentes
//-----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------}
unit u_SvnVersion;

{###############################} INTERFACE {###############################}

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, TBXDkPanels, StdCtrls, ExtCtrls, Math,
  gnugettext,
  u_ShBase;

type
  Tfrm_SvnVersion = class(TForm)
    pnlTop: TPanel;
    ScrollBoxLeft: TScrollBox;
    MemoInfo: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ButtonMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ButtonClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private-Deklarationen }
  public
    SvnRevision:int64;
    SvnCaption:string;
    procedure CalcSvnVersion(Id:TArcDocId);
  protected
    AktTag:integer;
    fslCaptions, fslHints, fslInfos, fslTags: TStringlist;
    procedure MemoBestimmen;
    procedure ButtonsCreate;
    procedure SetzeInfo(i:integer);
  end;

var
  frm_SvnVersion: Tfrm_SvnVersion;

{############################} IMPLEMENTATION {#############################}

{$R *.dfm}

uses {} u_dm_ViewerMain, u_dm_DB;

//-----------------------------------------------------------------------------
procedure Tfrm_SvnVersion.FormCreate(Sender: TObject);
begin
   fslCaptions:=TStringlist.Create;
   fslHints   :=TStringlist.Create;
   fslInfos   :=TStringlist.Create;
   fslTags    :=TStringlist.Create;
   AktTag:=-1;
end;

//-----------------------------------------------------------------------------
procedure Tfrm_SvnVersion.FormDestroy(Sender: TObject);
begin
   FreeAndNil(fslTags);
   FreeAndNil(fslInfos);
   FreeAndNil(fslHints);
   FreeAndNil(fslCaptions);
end;

//-----------------------------------------------------------------------------
// versucht die Breite des Memos anhand des Textes etwas schn zu bestimmen
procedure Tfrm_SvnVersion.MemoBestimmen;
var i,k,iBreite,iTmp,iAnzInfos:integer; s,sZeile:string; sl:TStringList;
    iMaxInfoLen : integer; // Die Lnge des lngsten Infotextes
    iMaxInfoLine : integer; // Die lngste Zeile
    iMaxInfoLines : integer; // Die grte Anzahl an Zeilen
begin
   Canvas.Font := MemoInfo.Font;
   sl:=TStringList.Create;
   try
      iMaxInfoLen := 0;
      iMaxInfoLine := 0;
      iMaxInfoLines := 0;
      iAnzInfos := fslCaptions.Count;
      for i := 0 to iAnzInfos-1 do
      begin
         s := fslInfos[i];
         sl.Text:=s;
         iMaxInfoLen := Max(iMaxInfoLen, length(s));

         for k := 0 to sl.Count-1 do
            begin
               sZeile := sl[k];
               iBreite := Canvas.TextWidth(sZeile);
               iMaxInfoLine:=Max(iMaxInfoLine,iBreite);
            end;
         iMaxInfoLines := Max(iMaxInfoLines, sl.Count);
      end;
   finally
      sl.Free;
   end;
   if iMaxInfoLen=0 then
      iBreite := 0
   else
      begin
         //iBreite := Min(Max(50,iMaxInfoLine),300);
         iTmp:=GetSystemmetrics(sm_cyhscroll);
         iBreite := iMaxInfoLine + iTmp + 8; // bischen noch frn Rand
         if iBreite > 300
         then iBreite := 300;
      end;
   ClientWidth := ScrollBoxLeft.Width + iBreite;
end;

//-----------------------------------------------------------------------------
procedure Tfrm_SvnVersion.ButtonsCreate;
var i,a:integer;  Button: TTBXButton;
begin
   while ScrollBoxLeft.ComponentCount>0 do
      ScrollBoxLeft.Components[0].Free;

   for i := 0 to fslCaptions.Count-1 +1 do
   begin
      Button := TTBXButton.Create(ScrollBoxleft);
      with Button do
      begin
         Width := ScrollBoxLeft.Width;
         if i<fslCaptions.Count then
            Caption := fslCaptions[i]
         else
            Caption := _('Abbrechen');
         Height:=21;
         Top:= i * Height;
         Tag := i;
         OnMouseMove := ButtonMouseMove;
         OnClick := ButtonClick;
         Parent := ScrollBoxleft;
      end;
   end;
   a:=fslCaptions.Count;
   if a<3 then a:=3;
   if a>20 then a:=20;
   ClientHeight := (a+1) * 21;

   SetzeInfo(0);
end;

//-----------------------------------------------------------------------------
procedure Tfrm_SvnVersion.CalcSvnVersion(Id:TArcDocId);
var
   sltmp, lsSvnVersion, lsSvnDate, lsCaption: string;
   iRev, iCount2: integer;
begin
   iCount2 := 0;
   // Alle SVN-Eintrge aus der DB zu dem Dokument abfragen
   sltmp := 'select * from sys_svn ' +
      ' where arc_id = ' + IntToStr(Id.ArcId) + ' and doc_id = ' +
      IntToStr(Id.DocId) + ' order by svn_version desc';
   dm_DB.my_q_Tmp.SQL.text := sltmp;
   dm_DB.my_q_Tmp.execute;
   // Wenn was drin, dann mal alles auslesen
   if dm_DB.my_q_Tmp.recordcount > 0 then
   begin
      with dm_DB.my_q_Tmp do
      begin
         while not EOF do
         begin
            lsSvnVersion :=FieldByName('svn_version').AsString;
            lsSvnDate := FieldByName('svn_date').AsString;
            if ((dm_MainForm2.pum_OriFile.Items.Find(lsSvnVersion) = nil)) then
            begin
               //lsCaption := '(' + lsSvnDate + ') - Version ' + lsSvnVersion;
               if iCount2=0 then
                  lsCaption := '(' + lsSvnDate + ') - '+_('<Aktuell>')
               else
                  lsCaption := '(' + lsSvnDate + ') - Version ' + lsSvnVersion;
               iRev := FieldByName('svn_revision').AsInteger;
               dm_MainForm2.pum_OriFile.Items[iCount2].Tag := iRev;
               dm_MainForm2.pum_OriFile.Items[iCount2].hint := FieldByName('svn_link').AsString;

               fslCaptions.Add(lsCaption);
               fslHints.Add(FieldByName('svn_link').AsString);
               fslInfos.Add(FieldByName('svn_info').AsString);
               fslTags.Add( IntToStr(iRev) );

               iCount2 := iCount2 + 1;
            end;
            dm_DB.my_q_Tmp.next;
         end;
      end;
   end;
   MemoBestimmen;
   ButtonsCreate;
end;

//-----------------------------------------------------------------------------
procedure Tfrm_SvnVersion.ButtonMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
   SetzeInfo(TComponent(Sender).Tag)
end;

//-----------------------------------------------------------------------------
procedure  Tfrm_SvnVersion.SetzeInfo(i:integer);
var lsInfo:string;
begin
   if AktTag=i
   then Exit;

   if i<fslCaptions.Count then
      lsInfo := fslInfos[i]
   else
      lsInfo := _('Fenster schlieen');
   MemoInfo.Text := lsInfo;
   AktTag:=i
end;

//-----------------------------------------------------------------------------
procedure Tfrm_SvnVersion.ButtonClick(Sender: TObject);
var i:integer;
begin
   i := TComponent(Sender).Tag;
   if i<fslCaptions.Count then
      begin
         SvnRevision:=StrToInt(fslTags[i]);
         SvnCaption:=fslCaptions[i];
         Modalresult:=mrOk;
      end
   else
      begin
         Modalresult:=mrCancel;
      end;
end;

//-----------------------------------------------------------------------------
procedure Tfrm_SvnVersion.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
   if Key=vk_Escape
   then Modalresult:=mrCancel;
end;

//-----------------------------------------------------------------------------
end.
