unit u_ilTwain;

interface

uses
   Windows, Messages, SysUtils, Classes, Controls, Dllsp96, Dll96v1, ILSaveImageThread,
   Dialogs;

type
   pTW_VERSION = ^TW_VERSION;
   TW_VERSION = packed Record
      MajorNum: Word;                     //* Major revision number of the software.
      MinorNum: Word;                     //* Incremental revision number of the software.
      Language: Word;                     //* e.g. TWLG_SWISSFRENCH
      Country: Word;                     //* e.g. TWCY_SWITZERLAND
      Info: Array[0..33] of char;         //* e.g. "Beta release"
   end;

   pTW_IDENTITY = ^TW_IDENTITY;
   TW_IDENTITY = packed record
      Id: ULONG;                          //* Unique number.  In Windows, application hWnd
      Version: TW_VERSION;                //* Identifies the piece of code
      ProtocolMajor: Word;                //* Application and DS must set to TWON_PROTOCOLMAJOR
      ProtocolMinor: Word;                //* Application and DS must set to TWON_PROTOCOLMINOR
      SupportedGroups: ULONG;             //* Bit field OR combination of DG_ constants
      Manufacturer: Array[0..33] of char; //* Manufacturer name, e.g. "Hewlett-Packard"
      ProductFamily: Array[0..33] of char;//* Product family name, e.g. "ScanJet"
      ProductName: Array[0..33] of char;  //* Product name, e.g. "ScanJet Plus"
   end;

function ilTwOpen(const SourceName: String): boolean;
function ilTwClose: boolean;
function ilTwStatus: Integer;
function ilTwSelectSource: boolean;
function ilTwSetPages(const ScanPageCount: Integer): boolean;
function ilTwSetPixelFormat(const TwPixelFormat: Integer): boolean;
function ilTwSetBrightness(const Brightness: Integer): boolean;
function ilTwSetContrast(const Contrast: Integer): boolean;
function ilTwSetThreshold(const Threshold: Integer): boolean;
function ilTwSetFeederEnabled(const Enable: boolean): boolean;
function ilTwSetAutoFeed(const Enable: boolean): boolean;
function ilTwSetDuplex(const Enable: boolean): boolean;
function ilTwSetXYResolution(const X, Y: Integer): boolean;
function ilTwSetPaperSize(const PaperSize: Integer): boolean;
function ilTwSetOrientation(const Orientation: Integer): boolean;
function ilTwSetRotation(const Rotation: Integer): boolean;
function ilTwSetUnit(const Units: Integer): boolean;
function ilTwSetTranMode(const Mode: Integer): boolean;
function ilTwEnableSource(const EnableUI: boolean): boolean;
function ilTwStartScan(Sender: TObject): boolean;
function ilTwDisableSource: boolean;
function ilTwDibCall(HDIB: Thandle; d_object: longint): Smallint; CDECL; export;
function ilTwCallBackDispatch(HDIB: Thandle): Smallint;
procedure ilSaveThrdThreadProgress(Sender: TObject; Progress: Smallint;
   Count: Integer; var CancelProcess: Boolean);

var  hTwObj: THandle;  //TwainObjectHandle
   hTwDLL: Longint;  //TwainDLLHandle
   hWindow: HWND;    //Handle
   sFileName: String;
   ilSaveThrd: ILSaveImageThrd;

implementation

function ilTwOpen(const SourceName: String): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      result := False;
      exit;
   end;
   try
      if SourceName = '' then
         hTwObj := illow_open_twain(hWindow)
      else hTwObj := illow_open_twain_set_source(hWindow, SourceName);

      if hTwObj = 0 then    // wenn Handle 0 dann gab es einen Fehler beim
      begin                 // Laden des Twain-Treibers (Source)
         result := False;
         exit;
      end;
      hTwDLL := illow_getdllhandle_twain(hTwObj);
   except
      result := false;
   end;
end;

function ilTwClose: boolean;
var iResult: Integer;
begin
   try
      if hTwObj <> 0 then
         iResult := illow_close_twain(hTwObj);
      if hTwDLL <> 0 then
         FreeLibrary(hTwDLL);  // Crash - Twain Locked prevention
      if iResult = 1 then
         result := True else result := False;
   except
      result := false;
   end;
end;

function ilTwStatus: Integer;
begin
   result := 0;
   try
      result := illow_getstatus_twain(hTwObj)
   except
      result := -1;
   end;
end;

function ilTwSelectSource: boolean;
begin
   result := True;
   try
      SelectTwainSource(hWindow);
   except
      result := False;
   end;
end;

function ilTwSetPages(const ScanPageCount: Integer): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setpages_twain(hTwObj, ScanPageCount);
      except
         result := False;
      end;
   end;
end;

function ilTwSetScanArea(const Left, Top, Width, Height: Single): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setscanArea_twain(hTwObj, Left, Top, Width, Height);
      except
         result := False;
      end;
   end;
end;

function ilTwSetPixelFormat(const TwPixelFormat: Integer): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setpixeltype_twain(hTwObj, TwPixelFormat);
      except
         result := False;
      end;
   end;
end;

function ilTwSetBrightness(const Brightness: Integer): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setcap_fix32_as_long_twain(hTwObj, ICAP_BRIGHTNESS, Brightness, False);
      except
         result := False;
      end;
   end;
end;

function ilTwSetContrast(const Contrast: Integer): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setcap_fix32_as_long_twain(hTwObj, ICAP_CONTRAST, Contrast, False);
      except
         result := False;
      end;
   end;
end;

function ilTwSetThreshold(const Threshold: Integer): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setcap_fix32_as_long_twain(hTwObj, ICAP_THRESHOLD, Threshold, False);
      except
         result := False;
      end;
   end;
end;

function ilTwSetFeederEnabled(const Enable: boolean): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setcap_bool_twain(hTwObj, CAP_FEEDERENABLED, Enable, False);
      except
         result := False;
      end;
   end;
end;

function ilTwSetAutoFeed(const Enable: boolean): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setcap_bool_twain(hTwObj, CAP_AUTOFEED, Enable, False);
      except
         result := False;
      end;
   end;
end;

function ilTwSetDuplex(const Enable: boolean): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setcap_bool_twain(hTwObj, CAP_DUPLEXENABLED, Enable, False);
      except
         result := False;
      end;
   end;
end;

function ilTwSetXYResolution(const X, Y: Integer): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setcap_fix32_as_long_twain(hTwObj, ICAP_YRESOLUTION, Y, False);
         illow_setcap_fix32_as_long_twain(hTwObj, ICAP_XRESOLUTION, X, False);
      except
         result := False;
      end;
   end;
end;

function ilTwSetPaperSize(const PaperSize: Integer): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setcap_uint16_twain(hTwObj, ICAP_SUPPORTEDSIZES, PaperSize, False);
      except
         result := False;
      end;
   end;
end;

function ilTwSetOrientation(const Orientation: Integer): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setcap_uint16_twain(hTwObj, ICAP_ORIENTATION, Orientation, False);
      except
         result := False;
      end;
   end;
end;

function ilTwSetRotation(const Rotation: Integer): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setcap_uint16_twain(hTwObj, ICAP_ROTATION, Rotation, False);
      except
         result := False;
      end;
   end;
end;

function ilTwSetUnit(const Units: Integer): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setcap_uint16_twain(hTwObj, ICAP_UNITS, Units, False);
      except
         result := False;
      end;
   end;
end;

function ilTwSetTranMode(const Mode: Integer): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_setcap_uint16_twain(hTwObj, ICAP_XFERMECH, Mode, False);
      except
         result := False;
      end;
   end;
end;

function ilTwEnableSource(const EnableUI: boolean): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_enablesource_twain(hTwObj, EnableUI);
      except
         result := False;
      end;
   end;
end;

function ilTwStartScan(Sender: TObject): boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_scan_twain(hTwObj, LongInt(nil), nil, ilTwDibCall);
      except
         result := False;
      end;
   end;
end;

function ilTwDisableSource: boolean;
begin
   result := True;
   if hTwObj <> 0 then
   begin
      try
         illow_disablesource_twain(hTwObj);
      except
         result := False;
      end;
   end;
end;

function ilTwDibCall(HDIB: Thandle; d_object: longint): Smallint; CDECL; export;
begin
   //Dispatch the callback to the calling class "d_object"
   result := ilTwCallBackDispatch(HDib);
end;

function ilTwCallBackDispatch(HDIB: Thandle): Smallint;
begin
   if (HDIB <> 0) and (GlobalSize(HDIB) > 0) then
   begin
      ilSaveThrd.SaveImageToFile(sFileName, HDIB, sColorTrue, img_Tiff, sfaxccitt4, tiffAppend,
         1, 100, 25, 0);
      BringWindowToTop(hWindow);
      Sleep(0);
   end;
   result := 1;
end;

procedure ilSaveThrdThreadProgress(Sender: TObject; Progress: Smallint;
   Count: Integer; var CancelProcess: Boolean);
begin
   // Your Code Here;
end;

///// Ab hier die Funktionen noch kopieren


end.
