unit u_bfaArchivierungsdienst;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, SvcMgr, Dialogs,
  ShellAPI, TlHelp32, JvTimer;

type
  TbfaArchiveSrv = class(TService)
    ServiceTimer: TJvTimer;
    procedure ServiceExecute(Sender: TService);
    procedure ServiceCreate(Sender: TObject);
    procedure ServiceDestroy(Sender: TObject);
    procedure ServiceStart(Sender: TService; var Started: Boolean);
    procedure ServiceStop(Sender: TService; var Stopped: Boolean);
    procedure OnTimer(Sender: TObject);
    procedure ServicePause(Sender: TService; var Paused: Boolean);
    procedure ServiceContinue(Sender: TService; var Continued: Boolean);
  private
    { Private-Deklarationen }
  public
    function GetServiceController: TServiceController; override;
    { Public-Deklarationen }
    procedure WriteLog(LogMsg: string);
  end;

  
  function IsExeRunning(const AExeName: string): boolean;
  
var
  bfaArchiveSrv: TbfaArchiveSrv;
  LogFile: TStringList;
  gsServiceDir, LogFileName, gsBitfarmDir: string;
  giTimerInterval:Integer;
  
implementation



{$R *.DFM}


procedure ServiceController(CtrlCode: DWord); stdcall;
begin
  bfaArchiveSrv.Controller(CtrlCode);
end;

function TbfaArchiveSrv.GetServiceController: TServiceController;
begin
  Result := ServiceController;
end;


procedure TbfaArchiveSrv.ServiceContinue(Sender: TService;
  var Continued: Boolean);
begin
  ServiceTimer.Enabled:= True;
  WriteLog('Dienst wird fortgesetzt');  
end;

procedure TbfaArchiveSrv.ServiceCreate(Sender: TObject);
begin
  LogFile:=TStringList.Create;
  ServiceTimer.Enabled:=False;
end;


procedure TbfaArchiveSrv.ServiceDestroy(Sender: TObject);
begin
  LogFile.Free;
end;

procedure TbfaArchiveSrv.ServiceExecute(Sender: TService);
begin

  ServiceTimer.Interval:= giTimerInterval * 1000;
  ServiceTimer.Enabled:=True;
  while not Terminated do
  begin
    ServiceThread.ProcessRequests(False);
    Sleep(1000);
    
    //WriteLog(DateTimeToStr(Now)+' Timer gestartet');

  end;
end;


procedure TbfaArchiveSrv.ServicePause(Sender: TService; var Paused: Boolean);
begin
  ServiceTimer.Enabled:= False;
  WriteLog('Dienst angehalten');
end;

procedure TbfaArchiveSrv.ServiceStart(Sender: TService; var Started: Boolean);
Var
i: Integer;
bArchivierung: Boolean;
begin
  ServiceTimer.Enabled:= False;
  gsServiceDir:= ExtractFilePath(ParamStr(0));
  giTimerInterval:= 3;     // alle 3sec
  
  if bfaArchiveSrv.ParamCount >= 2 then
  begin
    for i := 1 to bfaArchiveSrv.ParamCount - 1 do
    begin
      if Pos('-p:',bfaArchiveSrv.Param[i]) <> 0 then
        gsBitfarmDir:=Copy( bfaArchiveSrv.Param[i],4,Length(  bfaArchiveSrv.Param[i])-3);
      if Pos('-t:',bfaArchiveSrv.Param[i]) <> 0 then
        giTimerInterval:=StrToInt(Copy( bfaArchiveSrv.Param[i],4,Length(  bfaArchiveSrv.Param[i])-3));
    end;
  end;

  if gsBitfarmDir = '' then
    gsBitfarmDir:= gsServiceDir;

  if not FileExists(gsBitfarmDir + 'Archivierung.vbs') then
    bArchivierung:= False
  else
    bArchivierung:= True;
  if not FileExists(gsBitfarmDir + 'Archivierung.bat') then
    bArchivierung:= False
  else
    bArchivierung:= True;
  WriteLog(DateTimeToStr(Now)+' Service gestartet');
  WriteLog(gsServiceDir);
  WriteLog('Parameter:' + IntToStr(bfaArchiveSrv.ParamCount));
  for i := 0 to bfaArchiveSrv.ParamCount - 1 do
    WriteLog(DateTimeToStr(Now)+' Param:'+ bfaArchiveSrv.Param[i]);
  WriteLog('Dienstverzeichnis:' + gsServiceDir);
  WriteLog('Archivierungs-Pfad:' + gsBitfarmDir);
  WriteLog('TimerInterval(sec):' + IntToStr(giTimerInterval));

  if not bArchivierung then
  begin
    WriteLog('Fehler: Archivierung.vbs konnte nicht gefunden werden');
    WriteLog('Der Dienst wird terminiert...');
    LogMessage('Fehler: Archivierung.vbs konnte nicht gefunden werden'+#13+#10+'Der Dienst wird terminiert...'+#13+#10+'bfapop3-Pfad:' + gsBitfarmDir,1,0,4 );
  end;

  if not bArchivierung then
     bfaArchiveSrv.DoStop;

end;



procedure TbfaArchiveSrv.ServiceStop(Sender: TService; var Stopped: Boolean);
begin
  ServiceTimer.Enabled:= False;
  WriteLog('bitfarm-Archiv Archivierungsdienst beendet...');

end;

procedure TbfaArchiveSrv.OnTimer(Sender: TObject);
begin

  
  //if not IsExeRunning('bfapop3.exe') then
  //begin
  //  WriteLog('Archivierung.vbs wird gestartet');
    ShellExecute(0,'open',PChar(gsBitfarmDir + 'Archivierung.bat'),PChar(''), PChar(0),SW_MINIMIZE)
  //end
  //else
  //  WriteLog('eine Instanz des bfaPop3-Connector ist noch aktiv');
end;

procedure TbfaArchiveSrv.WriteLog(LogMsg: string);
//////////////////////////////////////////////////////////////////
///  10.04.2005 Erstellt LogEvnt by SV.                        ///
///                                                            ///
///  Diese Procedure schreibt LogMsg mit Datum in eine Datei   ///
///                                                            ///
//////////////////////////////////////////////////////////////////
var
   fLogFile: TextFile;
   sLogDate, Slognfile,Slognpath: string;
begin

      try
        DateTimeToString(sLogDate, 'dd.mm.yyyy hh:nn:ss-zzz', now);
        DateTimeToString(Slognfile, 'yyyy-mm-dd', now);
        Slognpath:=ExtractFilePath(ParamStr(0));
        if not DirectoryExists(Slognpath) then
          CreateDir(Slognfile);

        Slognfile:=Slognpath + 'bitfarm-Archiv Archivierungsdienst_'+Slognfile+'.log';

        try
           AssignFile(fLogFile, Slognfile);
           if not FileExists(Slognfile) then
              ReWrite(fLogFile);
           Reset(fLogFile);
           Append(fLogFile);
           WriteLn(fLogFile, sLogDate + '    ' +LogMsg);
           Flush(fLogFile);
           CloseFile(fLogFile);
        except
           
           on e: exception do
              LogMessage('Fehler:' + e.Message + #13
                       + #10 + 'Bitte Prfen Sie die Berechtigung auf die Datei '+Slognfile,0,0,4);
        end;
        
      finally

      end;


end;

function IsExeRunning(const AExeName: string): boolean;
var
   h: THandle;
   p: TProcessEntry32;
begin
   p.dwSize := SizeOf(p);
   h := CreateToolHelp32Snapshot(TH32CS_SnapProcess, 0);
   try
      Process32First(h, p);
      repeat
         Result := AnsiUpperCase(AExeName) = AnsiUpperCase(p.szExeFile);
      until Result or (not Process32Next(h, p));
   finally
      CloseHandle(h);
   end;
end;


end.
