﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.IO;
using System.Runtime.InteropServices;
using Puma.Net;
using Puma.Net.Utilities;

namespace ConsoleApplication1
{
    class Program
    {
        static int Main(string[] args)
        {

            var sparam = "";
            var sInputpfad = "";
            var sOutputpfad = "";
            var sOut = "";
            var sLanguage = "German";
            var bGetLanguages = false;
            var bOCR = false;
            var bOneColumn = false;
            var bUseTextformat = false;
            var bStdout = false;
            var bHelp = false;
            foreach (string s in args)
            {
                sparam = sparam + s + " ";
                if (s.Length > 2)
                {
                    if (s.Substring(0, 3) == "-i:")
                        sInputpfad = s.Substring(3, s.Length - 3);
                    if (s.Substring(0, 3) == "-o:")
                        sOutputpfad = s.Substring(3, s.Length - 3);
                    if (s == "-getlanguages")
                        bGetLanguages = true;
                    if (s == "-usetextformat")
                        bUseTextformat = true;
                    if (s == "-onecolumn")
                        bOneColumn = true;
                    if (s.Substring(0, 3) == "-l:")
                        sLanguage = s.Substring(3, s.Length - 3);
                    if (s == "-console")
                        bStdout = true;
                    if (s == "-help")
                        bHelp = true;
                }
            }

            if (bStdout == false)
            {

                Console.WriteLine("bitfarm-Archiv bfaORC - Tool v3.4.0.1");
                Console.WriteLine("basiert auf die Cuneiform Recongnition Engine und Puma.Net");
                Console.WriteLine("(siehe Lizenzdatei)");
                Console.WriteLine("Aufruf: bfarOcr.exe -i:<Bilddatei>{-o:<Outputpfad>}{-l:<Language>}");
                Console.WriteLine("weitere Optionen:");
                Console.WriteLine("{-getlanguages}  erstellt eine Liste der installierten Sprachen");
                Console.WriteLine("{-onecolumn}  liest das Dokument einspaltig");
                Console.WriteLine("{-usetextformat}  versucht den Formataufbau beizubehalten");
                Console.WriteLine("{-help}  zeigt diese Hilfe an");
                Console.WriteLine("{-console}  Ausgabe direkt auf StdOut, es wird kein Textfile geschrieben");
                Console.WriteLine("mögliche Bilddateitypen: .bmp .gif .jpg .png .tif");
                Console.WriteLine("Standard Outputpfad ist Bilddatei.txt, Ausgabeformat ist Ansi");
                Console.WriteLine("Standard Sprache ist deutsch");
            }
            if (bHelp)
                return 0;


            if (sparam != "")
            {
                if (bStdout == false)
                    Console.WriteLine("Übergebene Parameter:" + sparam);
            }
            else
            {
                if (bStdout == false)
                    Console.WriteLine("Es wurden keine Parameter übergeben");
                return 1;
            }

            if (bGetLanguages == false)
            {
                if (sInputpfad == "" && bGetLanguages == false)
                {
                    if (bStdout == false)
                        Console.WriteLine("Es wurden keine Bilddatei übergeben");
                    return 2;
                }

                if (File.Exists(sInputpfad) == false)
                {
                    if (bStdout == false)
                        Console.WriteLine("Bilddatei nicht gefunden");
                    return 3;
                }

                FileInfo fInfo = new FileInfo(@sInputpfad);
                var fext = fInfo.Extension;
                var bAccept = false;
                string[] arrExt = new String[7] { ".bmp", ".jpg", ".jpeg", ".gif", ".png", ".tif", ".tiff" };

                foreach (string ext in arrExt)
                {
                    if (fext == ext)
                    {
                        bAccept = true;
                        break;
                    }
                }

                if (bAccept == false)
                {
                    if (bStdout == false)
                        Console.WriteLine("Bilddateiformat nicht unterstützt");
                    return 4;
                }

                if (sOutputpfad == "")
                {
                    sOutputpfad = Path.GetDirectoryName(sInputpfad) + "\\" + Path.GetFileNameWithoutExtension(sInputpfad) + ".txt";
                }
            }
            else
            {
                sInputpfad = null;
            }

            try
            {
                var pumapage = new PumaPage();
               
                using(pumapage)
                {
                    if (bGetLanguages)
                    {
                        List<string> lstLanguages = new List<string>();
                        lstLanguages.AddRange(LanguageStrings.GetSupportedLanguages(pumapage));
                        foreach (string lang in lstLanguages)
                        {
                            if (bStdout == false)
                                Console.WriteLine(lang);
                        }
                    }
                    else
                    {
                        pumapage.LoadImage(sInputpfad);
                        pumapage.FileFormat = PumaFileFormat.TxtAnsi;
                        pumapage.EnableSpeller = false;
                        //pumapage.Language = PumaLanguage.German;
                        pumapage.Language = LanguageStrings.GetSupportedLanguage(pumapage, sLanguage);
                        pumapage.AutoRotateImage = true;
                        pumapage.OneColumnMode = bOneColumn;
                        pumapage.UseTextFormating = bUseTextformat;

                        try
                        {
                            if (bStdout == false)
                                pumapage.RecognizeToFile(sOutputpfad);
                            else
                            {
                                sOut = pumapage.RecognizeToString();
                                Console.WriteLine(sOut);

                            }

                            
                        }
                        catch (Exception ex)
                        {
                            if (bStdout == false)
                                Console.WriteLine(
                                    "Allgemeiner Fehler bei der OCR: " +
                                    ex.Message);
                            return 7;
                        }
                        bOCR = true;
                    }
                }
            }
            catch (COMException ex)
            {
                if (bStdout == false)
                    Console.WriteLine(
                        "Error while loading Puma COM server. Check that it's registered. Inner error message: " +
                        ex.Message);
                return 5;
            }
            catch (Exception ex)
            {
                if (bStdout == false)
                    Console.WriteLine(
                        "Error while starting. Inner error message: " +
                        ex.Message);
                return 6;
            }


            if (bOCR)
            {
                if (bStdout == false)
                {
                    Console.WriteLine("OCR erfolgreich durchgeführt.");
                    Console.WriteLine("Ergebnis als " + sOutputpfad + " gespeichert.");
                }
                return 0;
            }
            else
            {
                if (bStdout == false)
                    Console.WriteLine("Allgemeiner Fehler bei der Programmausführung.");
                return 255;
            }
        }

    }
}
