unit u_bfaSpooldienst;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, SvcMgr, Dialogs,
  ShellAPI, TlHelp32, JvTimer, IniFiles;

type
  TbfaSpoolSrv = class(TService)
    ServiceTimer: TJvTimer;
    procedure ServiceExecute(Sender: TService);
    procedure ServiceCreate(Sender: TObject);
    procedure ServiceDestroy(Sender: TObject);
    procedure ServiceStart(Sender: TService; var Started: Boolean);
    procedure ServiceStop(Sender: TService; var Stopped: Boolean);
    procedure OnTimer(Sender: TObject);
    procedure ServicePause(Sender: TService; var Paused: Boolean);
    procedure ServiceContinue(Sender: TService; var Continued: Boolean);
  private
    { Private-Deklarationen }
  public
    function GetServiceController: TServiceController; override;
    { Public-Deklarationen }
    procedure WriteLog(LogMsg: string);
  end;


  function IsExeRunning(const AExeName: string): boolean;
  function inilesen(const AbitfarmDir: string): string;
var
  bfaSpoolSrv: TbfaSpoolSrv;
  LogFile: TStringList;
  gsServiceDir, LogFileName, gsBitfarmDir, gsProfil: string;
  giTimerInterval:Integer;
  bSpool, bChangelog: Boolean;

implementation

{$define GPL}

{$R *.DFM}


procedure ServiceController(CtrlCode: DWord); stdcall;
begin
  bfaSpoolSrv.Controller(CtrlCode);
end;

function TbfaSpoolSrv.GetServiceController: TServiceController;
begin
  Result := ServiceController;
end;


procedure TbfaSpoolSrv.ServiceContinue(Sender: TService;
  var Continued: Boolean);
begin
  ServiceTimer.Enabled:= True;
  WriteLog('Dienst wird fortgesetzt');  
end;

procedure TbfaSpoolSrv.ServiceCreate(Sender: TObject);
begin
  LogFile:=TStringList.Create;
  ServiceTimer.Enabled:=False;
end;


procedure TbfaSpoolSrv.ServiceDestroy(Sender: TObject);
begin
  LogFile.Free;
end;

procedure TbfaSpoolSrv.ServiceExecute(Sender: TService);
begin

  ServiceTimer.Interval:= giTimerInterval * 1000;
  ServiceTimer.Enabled:=True;
  while not Terminated do
  begin
    ServiceThread.ProcessRequests(False);
    Sleep(1000);
    
    //WriteLog(DateTimeToStr(Now)+' Timer gestartet');

  end;
end;


procedure TbfaSpoolSrv.ServicePause(Sender: TService; var Paused: Boolean);
begin
  ServiceTimer.Enabled:= False;
  WriteLog('Dienst angehalten');
end;

procedure TbfaSpoolSrv.ServiceStart(Sender: TService; var Started: Boolean);
Var
i: Integer;

begin
  ServiceTimer.Enabled:= False;
  gsServiceDir:= ExtractFilePath(ParamStr(0));
  giTimerInterval:= 53;     // alle 53sec
  
  if bfaSpoolSrv.ParamCount >= 2 then
  begin
    for i := 1 to bfaSpoolSrv.ParamCount - 1 do
    begin
      if Pos('-p:',bfaSpoolSrv.Param[i]) <> 0 then
        gsBitfarmDir:=Copy( bfaSpoolSrv.Param[i],4,Length(  bfaSpoolSrv.Param[i])-3);
      if Pos('-t:',bfaSpoolSrv.Param[i]) <> 0 then
        giTimerInterval:=StrToInt(Copy( bfaSpoolSrv.Param[i],4,Length(  bfaSpoolSrv.Param[i])-3));
      if Pos('-profil:',bfaSpoolSrv.Param[i]) <> 0 then
        gsProfil:=Copy( bfaSpoolSrv.Param[i],9,Length(  bfaSpoolSrv.Param[i])-8);
    end;
  end;


  if gsBitfarmDir = '' then
    gsBitfarmDir:= gsServiceDir;

  if gsProfil = '' then
  begin
    gsProfil := inilesen(gsBitfarmDir);
  end;



  if not FileExists(gsBitfarmDir + 'controler.vbs') then
    bSpool:= False
  else
    bSpool:= True;
  if not FileExists(gsBitfarmDir + 'controler.bat') then
    bSpool:= False
  else
    bSpool:= True;

  if not FileExists(gsBitfarmDir + 'changelogcontroler.vbs') then
    bChangelog:= False
  else
    bChangelog:= True;
  if not FileExists(gsBitfarmDir + 'changelogcontroler.bat') then
    bChangelog:= False
  else
    bChangelog:= True;


  WriteLog(DateTimeToStr(Now)+' Service gestartet');
  WriteLog(gsServiceDir);
  WriteLog('Parameter:' + IntToStr(bfaSpoolSrv.ParamCount));
  for i := 0 to bfaSpoolSrv.ParamCount - 1 do
    WriteLog(DateTimeToStr(Now)+' Param:'+ bfaSpoolSrv.Param[i]);
  WriteLog('Dienstverzeichnis:' + gsServiceDir);
  WriteLog('Programmverzeichnis:' + gsBitfarmDir);
  WriteLog('Profil:' + gsProfil);
  WriteLog('TimerInterval(sec):' + IntToStr(giTimerInterval));

  if not bSpool then
  begin
    WriteLog('Fehler: controler.vbs konnte nicht gefunden werden');
    WriteLog('Der Dienst wird terminiert...');
    LogMessage('Fehler: controler.vbs konnte nicht gefunden werden'+#13+#10+'Der Dienst wird terminiert...'+#13+#10+'Such-Pfad:' + gsBitfarmDir,1,0,4 );
  end;

  {$IFNDEF GPL}
  if not bChangelog then
  begin
    WriteLog('Fehler: changelogcontroler.vbs konnte nicht gefunden werden');
    WriteLog('Der Changelog-Dienst wird nicht ausgefhrt...');
    LogMessage('Fehler: changelogcontroler.vbs konnte nicht gefunden werden'+#13+#10+'Der Changelog-Dienst wird nicht ausgefhrt...'+#13+#10+'Such-Pfad:' + gsBitfarmDir,1,0,4 );
  end;
  {$ENDIF}

  if not bSpool then
     bfaSpoolSrv.DoStop;

end;



procedure TbfaSpoolSrv.ServiceStop(Sender: TService; var Stopped: Boolean);
begin
  ServiceTimer.Enabled:= False;
  ShellExecute(0,'open',PChar(gsBitfarmDir + 'termwscript.vbs'),PChar(''), PChar(0),SW_MINIMIZE);
  Sleep(2000);
  ShellExecute(0,'open',PChar(gsBitfarmDir + 'cleanup.vbs'),PChar(''), PChar(0),SW_MINIMIZE);

  WriteLog('bitfarm-Archiv Spooldienst beendet...');

end;

procedure TbfaSpoolSrv.OnTimer(Sender: TObject);
begin

  
  //if not IsExeRunning('bfapop3.exe') then
  //begin
  //  WriteLog('Archivierung.vbs wird gestartet');
    ShellExecute(0,'open',PChar(gsBitfarmDir + 'controler.bat'),PChar(gsProfil), PChar(0),SW_MINIMIZE);
    if bChangelog then
      ShellExecute(0,'open',PChar(gsBitfarmDir + 'changelogcontroler.bat'),PChar(gsProfil), PChar(0),SW_MINIMIZE);
  //end
  //else
  //  WriteLog('eine Instanz des bfaPop3-Connector ist noch aktiv');
end;

procedure TbfaSpoolSrv.WriteLog(LogMsg: string);
//////////////////////////////////////////////////////////////////
///  10.04.2005 Erstellt LogEvnt by SV.                        ///
///                                                            ///
///  Diese Procedure schreibt LogMsg mit Datum in eine Datei   ///
///                                                            ///
//////////////////////////////////////////////////////////////////
var
   fLogFile: TextFile;
   sLogDate, Slognfile,Slognpath: string;
begin

      try
        DateTimeToString(sLogDate, 'dd.mm.yyyy hh:nn:ss-zzz', now);
        DateTimeToString(Slognfile, 'yyyy-mm-dd', now);
        Slognpath:=ExtractFilePath(ParamStr(0));
        if not DirectoryExists(Slognpath) then
          CreateDir(Slognfile);

        Slognfile:=Slognpath + 'bitfarm-Archiv Spooldienst_'+Slognfile+'.log';

        try
           AssignFile(fLogFile, Slognfile);
           if not FileExists(Slognfile) then
              ReWrite(fLogFile);
           Reset(fLogFile);
           Append(fLogFile);
           WriteLn(fLogFile, sLogDate + '    ' +LogMsg);
           Flush(fLogFile);
           CloseFile(fLogFile);
        except
           
           on e: exception do
              LogMessage('Fehler:' + e.Message + #13
                       + #10 + 'Bitte Prfen Sie die Berechtigung auf die Datei '+Slognfile,0,0,4);
        end;
        
      finally

      end;


end;

function IsExeRunning(const AExeName: string): boolean;
var
   h: THandle;
   p: TProcessEntry32;
begin
   p.dwSize := SizeOf(p);
   h := CreateToolHelp32Snapshot(TH32CS_SnapProcess, 0);
   try
      Process32First(h, p);
      repeat
         Result := AnsiUpperCase(AExeName) = AnsiUpperCase(p.szExeFile);
      until Result or (not Process32Next(h, p));
   finally
      CloseHandle(h);
   end;
end;

function inilesen(const AbitfarmDir: string): string;
var
IniFile : Tinifile;
lsProfil : string;
begin

  Inifile :=  Tinifile.Create(gsBitfarmDir + 'scripts.ini');

  lsProfil := IniFile.ReadString('Scripts','Profil','bitfarm');

  if Assigned(Inifile) then
    FreeAndNil(Inifile);

  Result := lsProfil;


end;



end.
